@@ -14,3 +14,7 @@
 #ifdef SYMANTEC__CFM68K__
 #pragma lib_export on
 #endif
+
+#ifndef HAVE_HYPOT
+extern double hypot Py_PROTO((double, double));
+#endif
