@@ -10,28 +10,31 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
-#ifndef DEBUG
-#define NDEBUG
-#endif
-
 /* Object and type object interface */
 
 /*
@@ -81,7 +84,7 @@ whose size is determined when the object is allocated.
 123456789-123456789-123456789-123456789-123456789-123456789-123456789-12
 */
 
-#ifndef NDEBUG
+#ifdef DEBUG
 
 /* Turn on heavy reference debugging */
 #define Py_TRACE_REFS
@@ -89,7 +92,7 @@ whose size is determined when the object is allocated.
 /* Turn on reference counting */
 #define Py_REF_DEBUG
 
-#endif /* NDEBUG */
+#endif /* DEBUG */
 
 #ifdef Py_TRACE_REFS
 #define PyObject_HEAD \
@@ -97,12 +100,12 @@ whose size is determined when the object is allocated.
 	int ob_refcnt; \
 	struct _typeobject *ob_type;
 #define PyObject_HEAD_INIT(type) 0, 0, 1, type,
-#else
+#else /* !Py_TRACE_REFS */
 #define PyObject_HEAD \
 	int ob_refcnt; \
 	struct _typeobject *ob_type;
 #define PyObject_HEAD_INIT(type) 1, type,
-#endif
+#endif /* !Py_TRACE_REFS */
 
 #define PyObject_VAR_HEAD \
 	PyObject_HEAD \
@@ -189,7 +192,9 @@ typedef struct {
 typedef void (*destructor) Py_PROTO((PyObject *));
 typedef int (*printfunc) Py_PROTO((PyObject *, FILE *, int));
 typedef PyObject *(*getattrfunc) Py_PROTO((PyObject *, char *));
+typedef PyObject *(*getattrofunc) Py_PROTO((PyObject *, PyObject *));
 typedef int (*setattrfunc) Py_PROTO((PyObject *, char *, PyObject *));
+typedef int (*setattrofunc) Py_PROTO((PyObject *, PyObject *, PyObject *));
 typedef int (*cmpfunc) Py_PROTO((PyObject *, PyObject *));
 typedef PyObject *(*reprfunc) Py_PROTO((PyObject *));
 typedef long (*hashfunc) Py_PROTO((PyObject *));
@@ -219,10 +224,10 @@ typedef struct _typeobject {
 	hashfunc tp_hash;
 	ternaryfunc tp_call;
 	reprfunc tp_str;
+	getattrofunc tp_getattro;
+	setattrofunc tp_setattro;
 
 	/* Space for future expansion */
-	long tp_xxx1;
-	long tp_xxx2;
 	long tp_xxx3;
 	long tp_xxx4;
 
@@ -254,6 +259,7 @@ extern int PyObject_SetAttr Py_PROTO((PyObject *, PyObject *, PyObject *));
 extern long PyObject_Hash Py_PROTO((PyObject *));
 extern int PyObject_IsTrue Py_PROTO((PyObject *));
 extern int PyCallable_Check Py_PROTO((PyObject *));
+extern int PyNumber_Coerce Py_PROTO((PyObject **, PyObject **));
 
 /* Flag bits for printing: */
 #define Py_PRINT_RAW	1	/* No string quotes etc. */
@@ -293,48 +299,60 @@ environment the global variable trick is not safe.)
 #endif
 #endif
 
+#ifdef Py_TRACE_REFS
+extern void _Py_Dealloc Py_PROTO((PyObject *));
+extern void _Py_NewReference Py_PROTO((PyObject *));
+extern void _Py_ForgetReference Py_PROTO((PyObject *));
+extern void _Py_PrintReferences Py_PROTO((FILE *));
+#endif
+
 #ifndef Py_TRACE_REFS
 #ifdef COUNT_ALLOCS
 #define _Py_Dealloc(op) ((op)->ob_type->tp_free++, (*(op)->ob_type->tp_dealloc)((PyObject *)(op)))
 #define _Py_ForgetReference(op) ((op)->ob_type->tp_free++)
-#else
+#else /* !COUNT_ALLOCS */
 #define _Py_Dealloc(op) (*(op)->ob_type->tp_dealloc)((PyObject *)(op))
 #define _Py_ForgetReference(op) /*empty*/
-#endif
-#endif
+#endif /* !COUNT_ALLOCS */
+#endif /* !Py_TRACE_REFS */
 
 #ifdef COUNT_ALLOCS
 extern void inc_count Py_PROTO((PyTypeObject *));
 #endif
 
 #ifdef Py_REF_DEBUG
+
 extern long _Py_RefTotal;
+
 #ifndef Py_TRACE_REFS
 #ifdef COUNT_ALLOCS
 #define _Py_NewReference(op) (inc_count((op)->ob_type), _Py_RefTotal++, (op)->ob_refcnt = 1)
 #else
 #define _Py_NewReference(op) (_Py_RefTotal++, (op)->ob_refcnt = 1)
 #endif
-#endif
+#endif /* !Py_TRACE_REFS */
+
 #define Py_INCREF(op) (_Py_RefTotal++, (op)->ob_refcnt++)
 #define Py_DECREF(op) \
 	if (--_Py_RefTotal, --(op)->ob_refcnt != 0) \
 		; \
 	else \
 		_Py_Dealloc(op)
-#else
+#else /* !Py_REF_DEBUG */
+
 #ifdef COUNT_ALLOCS
 #define _Py_NewReference(op) (inc_count((op)->ob_type), (op)->ob_refcnt = 1)
 #else
 #define _Py_NewReference(op) ((op)->ob_refcnt = 1)
 #endif
+
 #define Py_INCREF(op) ((op)->ob_refcnt++)
 #define Py_DECREF(op) \
 	if (--(op)->ob_refcnt != 0) \
 		; \
 	else \
 		_Py_Dealloc(op)
-#endif
+#endif /* !Py_REF_DEBUG */
 
 /* Macros to use in case the object pointer may be NULL: */
 
