@@ -17,10 +17,21 @@
 import string
 import socket
 import regex
+import os
 
 
-__version__ = '1.2'
+__version__ = '1.5'
 
+# Helper for non-unix systems
+if os.name == 'mac':
+	from macurl2path import url2pathname, pathname2url
+elif os.name == 'nt':    
+	from nturl2path import url2pathname, pathname2url 
+else:
+	def url2pathname(pathname):
+		return pathname
+	def pathname2url(pathname):
+		return pathname
 
 # This really consists of two pieces:
 # (1) a class which handles opening of all sorts of URLs
@@ -36,11 +47,14 @@ def urlopen(url):
 	if not _urlopener:
 		_urlopener = FancyURLopener()
 	return _urlopener.open(url)
-def urlretrieve(url):
+def urlretrieve(url, filename=None):
 	global _urlopener
 	if not _urlopener:
 		_urlopener = FancyURLopener()
-	return _urlopener.retrieve(url)
+	if filename:
+	    return _urlopener.retrieve(url, filename)
+	else:
+	    return _urlopener.retrieve(url)
 def urlcleanup():
 	if _urlopener:
 		_urlopener.cleanup()
@@ -56,7 +70,10 @@ ftpcache = {}
 class URLopener:
 
 	# Constructor
-	def __init__(self):
+	def __init__(self, proxies=None):
+		if proxies is None:
+			proxies = getproxies()
+		self.proxies = proxies
 		server_version = "Python-urllib/%s" % __version__
 		self.addheaders = [('User-agent', server_version)]
 		self.tempcache = None
@@ -98,6 +115,12 @@ class URLopener:
 		fullurl = unwrap(fullurl)
 		type, url = splittype(fullurl)
  		if not type: type = 'file'
+		self.openedurl = '%s:%s' % (type, url)
+		if self.proxies.has_key(type):
+			proxy = self.proxies[type]
+			type, proxy = splittype(proxy)
+			host, selector = splithost(proxy)
+			url = (host, fullurl) # Signal special case to open_*()
 		name = 'open_' + type
 		if '-' in name:
 			import regsub
@@ -117,29 +140,31 @@ class URLopener:
 	# External interface
 	# retrieve(url) returns (filename, None) for a local object
 	# or (tempfilename, headers) for a remote object
-	def retrieve(self, url):
+	def retrieve(self, url, filename=None):
 		if self.tempcache and self.tempcache.has_key(url):
 			return self.tempcache[url]
 		url1 = unwrap(url)
+		self.openedurl = url1
 		if self.tempcache and self.tempcache.has_key(url1):
 			self.tempcache[url] = self.tempcache[url1]
 			return self.tempcache[url1]
 		type, url1 = splittype(url1)
-		if not type or type == 'file':
+		if not filename and (not type or type == 'file'):
 			try:
 				fp = self.open_local_file(url1)
 				del fp
-				return splithost(url1)[1], None
+				return url2pathname(splithost(url1)[1]), None
 			except IOError, msg:
 				pass
 		fp = self.open(url)
 		headers = fp.info()
-		import tempfile
-		tfn = tempfile.mktemp()
-		result = tfn, headers
+		if not filename:
+		    import tempfile
+		    filename = tempfile.mktemp()
+		result = filename, headers
 		if self.tempcache is not None:
 			self.tempcache[url] = result
-		tfp = open(tfn, 'w')
+		tfp = open(filename, 'w')
 		bs = 1024*8
 		block = fp.read(bs)
 		while block:
@@ -154,13 +179,20 @@ class URLopener:
 	# Use HTTP protocol
 	def open_http(self, url):
 		import httplib
-		host, selector = splithost(url)
-		if not host: raise IOError, ('http error', 'no host given')
-		i = string.find(host, '@')
-		if i >= 0:
-			user_passwd, host = host[:i], host[i+1:]
+		if type(url) is type(""):
+			host, selector = splithost(url)
+			user_passwd, host = splituser(host)
 		else:
-			user_passwd = None
+			host, selector = url
+			urltype, rest = splittype(selector)
+			if string.lower(urltype) == 'http':
+			    realhost, rest = splithost(rest)
+			    user_passwd, realhost = splituser(realhost)
+			    if user_passwd:
+				selector = "%s://%s%s" % (urltype,
+							  realhost, rest)
+			print "proxy via http:", host, selector
+		if not host: raise IOError, ('http error', 'no host given')
 		if user_passwd:
 			import base64
 			auth = string.strip(base64.encodestring(user_passwd))
@@ -174,7 +206,7 @@ class URLopener:
 		errcode, errmsg, headers = h.getreply()
 		fp = h.getfile()
 		if errcode == 200:
-			return addinfo(fp, headers)
+			return addinfourl(fp, headers, self.openedurl)
 		else:
 			return self.http_error(url,
 					       fp, errcode, errmsg, headers)
@@ -211,7 +243,7 @@ class URLopener:
 			fp = gopherlib.send_query(selector, query, host)
 		else:
 			fp = gopherlib.send_selector(selector, host)
-		return addinfo(fp, noheaders())
+		return addinfourl(fp, noheaders(), self.openedurl)
 
 	# Use local file or FTP depending on form of URL
 	def open_file(self, url):
@@ -223,12 +255,13 @@ class URLopener:
 	# Use local file
 	def open_local_file(self, url):
 		host, file = splithost(url)
-		if not host: return addinfo(open(file, 'r'), noheaders())
+		if not host:
+			return addinfourl(open(url2pathname(file), 'r'), noheaders(), 'file:'+file)
 		host, port = splitport(host)
 		if not port and socket.gethostbyname(host) in (
 			  localhost(), thishost()):
 			file = unquote(file)
-			return addinfo(open(file, 'r'), noheaders())
+			return addinfourl(open(url2pathname(file), 'r'), noheaders(), 'file:'+file)
 		raise IOError, ('local file error', 'not on local host')
 
 	# Use FTP protocol
@@ -260,8 +293,8 @@ class URLopener:
 				if string.lower(attr) == 'type' and \
 				   value in ('a', 'A', 'i', 'I', 'd', 'D'):
 					type = string.upper(value)
-			return addinfo(self.ftpcache[key].retrfile(file, type),
-				  noheaders())
+			return addinfourl(self.ftpcache[key].retrfile(file, type),
+				  noheaders(), self.openedurl)
 		except ftperrors(), msg:
 			raise IOError, ('ftp error', msg)
 
@@ -275,9 +308,9 @@ class FancyURLopener(URLopener):
 
 	# Default error handling -- don't raise an exception
 	def http_error_default(self, url, fp, errcode, errmsg, headers):
-	    return addinfo(fp, headers)
+	    return addinfourl(fp, headers, self.openedurl)
 
-	# Error 302 -- relocated
+	# Error 302 -- relocated (temporarily)
 	def http_error_302(self, url, fp, errcode, errmsg, headers):
 		# XXX The server can force infinite recursion here!
 		if headers.has_key('location'):
@@ -290,6 +323,9 @@ class FancyURLopener(URLopener):
 		fp.close()
 		return self.open(newurl)
 
+	# Error 301 -- also relocated (permanently)
+	http_error_301 = http_error_302
+
 	# Error 401 -- authentication required
 	# See this URL for a description of the basic authentication scheme:
 	# http://www.ics.uci.edu/pub/ietf/http/draft-ietf-http-v10-spec-00.txt
@@ -435,7 +471,7 @@ class ftpwrapper:
 			if file: cmd = 'LIST ' + file
 			else: cmd = 'LIST'
 			conn = self.ftp.transfercmd(cmd)
-		return addclosehook(conn.makefile('r'), self.ftp.voidresp)
+		return addclosehook(conn.makefile('rb'), self.ftp.voidresp)
 
 # Base class for addinfo and addclosehook
 class addbase:
@@ -477,30 +513,65 @@ class addinfo(addbase):
 	def info(self):
 		return self.headers
 
+# class to add info() and geturl() methods to an open file
+class addinfourl(addbase):
+	def __init__(self, fp, headers, url):
+		addbase.__init__(self, fp)
+		self.headers = headers
+		self.url = url
+	def info(self):
+		return self.headers
+	def geturl(self):
+		return self.url
+
 
 # Utility to combine a URL with a base URL to form a new URL
 
 def basejoin(base, url):
 	type, path = splittype(url)
+	if type:
+		# if url is complete (i.e., it contains a type), return it
+		return url
 	host, path = splithost(path)
-	if type and host: return url
-	basetype, basepath = splittype(base)
-	basehost, basepath = splithost(basepath)
-	basepath, basetag = splittag(basepath)
-	basepath, basequery = splitquery(basepath)
-	if not type: type = basetype or 'file'
+	type, basepath = splittype(base) # inherit type from base
+	if host:
+		# if url contains host, just inherit type
+		if type: return type + '://' + host + path
+		else:
+			# no type inherited, so url must have started with //
+			# just return it
+			return url
+	host, basepath = splithost(basepath) # inherit host
+	basepath, basetag = splittag(basepath) # remove extraneuous cruft
+	basepath, basequery = splitquery(basepath) # idem
 	if path[:1] != '/':
-		i = string.rfind(basepath, '/')
-		if i < 0: basepath = '/'
-		else: basepath = basepath[:i+1]
+		# non-absolute path name
+		if path[:1] in ('#', '?'):
+			# path is just a tag or query, attach to basepath
+			i = len(basepath)
+		else:
+			# else replace last component
+			i = string.rfind(basepath, '/')
+		if i < 0:
+			# basepath not absolute
+			if host:
+				# host present, make absolute
+				basepath = '/'
+			else:
+				# else keep non-absolute
+				basepath = ''
+		else:
+			# remove last file component
+			basepath = basepath[:i+1]
 		path = basepath + path
-	if not host: host = basehost
-	if host: return type + '://' + host + path
-	else: return type + ':' + path
+	if type and host: return type + '://' + host + path
+	elif type: return type + ':' + path
+	elif host: return '//' + host + path # don't know what this means
+	else: return path
 
 
 # Utilities to parse URLs (most of these return None for missing parts):
-# unwrap('<URL:type//host/path>') --> 'type//host/path'
+# unwrap('<URL:type://host/path>') --> 'type://host/path'
 # splittype('type:opaquestring') --> 'type', 'opaquestring'
 # splithost('//host[:port]/path') --> 'host[:port]', '/path'
 # splituser('user[:passwd]@host[:port]') --> 'user[:passwd]', 'host[:port]'
@@ -547,6 +618,22 @@ def splitport(host):
 	if _portprog.match(host) >= 0: return _portprog.group(1, 2)
 	return host, None
 
+# Split host and port, returning numeric port.
+# Return given default port if no ':' found; defaults to -1.
+# Return numerical port if a valid number are found after ':'.
+# Return None if ':' but not a valid number.
+_nportprog = regex.compile('^\(.*\):\(.*\)$')
+def splitnport(host, defport=-1):
+	if _nportprog.match(host) >= 0:
+	    host, port = _nportprog.group(1, 2)
+	    try:
+		if not port: raise string.atoi_error, "no digits"
+		nport = string.atoi(port)
+	    except string.atoi_error:
+		nport = None
+	    return host, nport
+	return host, defport
+
 _queryprog = regex.compile('^\(.*\)\?\([^?]*\)$')
 def splitquery(url):
 	if _queryprog.match(url) >= 0: return _queryprog.group(1, 2)
@@ -575,26 +662,44 @@ _quoteprog = regex.compile('%[0-9a-fA-F][0-9a-fA-F]')
 def unquote(s):
 	i = 0
 	n = len(s)
-	res = ''
+	res = []
 	while 0 <= i < n:
 		j = _quoteprog.search(s, i)
 		if j < 0:
-			res = res + s[i:]
+			res.append(s[i:])
 			break
-		res = res + (s[i:j] + chr(eval('0x' + s[j+1:j+3])))
+		res.append(s[i:j] + chr(string.atoi(s[j+1:j+3], 16)))
 		i = j+3
-	return res
+	return string.joinfields(res, '')
 
 always_safe = string.letters + string.digits + '_,.-'
 def quote(s, safe = '/'):
 	safe = always_safe + safe
-	res = ''
+	res = []
 	for c in s:
 		if c in safe:
-			res = res + c
+			res.append(c)
 		else:
-			res = res + '%%%02x' % ord(c)
-	return res
+			res.append('%%%02x' % ord(c))
+	return string.joinfields(res, '')
+
+
+# Proxy handling
+def getproxies():
+	"""Return a dictionary of protocol scheme -> proxy server URL mappings.
+
+	Scan the environment for variables named <scheme>_proxy;
+	this seems to be the standard convention.  If you need a
+	different way, you can pass a proxies dictionary to the
+	[Fancy]URLopener constructor.
+
+	"""
+	proxies = {}
+	for name, value in os.environ.items():
+		if value and name[-6:] == '_proxy':
+			proxies[name[:-6]] = value
+	return proxies
+
 
 # Test and time quote() and unquote()
 def test1():
