@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -60,7 +67,7 @@ newdbmobject(file, flags, mode)
 	if ( (dp->di_dbm = dbm_open(file, flags, mode)) == 0 ) {
 	    err_errno(DbmError);
 	    DECREF(dp);
-	    return 0;
+	    return NULL;
 	}
 	return (object *)dp;
 }
@@ -107,12 +114,12 @@ dbm_subscript(dp, key)
 	drec = dbm_fetch(dp->di_dbm, krec);
 	if ( drec.dptr == 0 ) {
 	    err_setstr(KeyError, GETSTRINGVALUE((stringobject *)key));
-	    return 0;
+	    return NULL;
 	}
 	if ( dbm_error(dp->di_dbm) ) {
 	    dbm_clearerr(dp->di_dbm);
 	    err_setstr(DbmError, "");
-	    return 0;
+	    return NULL;
 	}
 	return newsizedstringobject(drec.dptr, drec.dsize);
 }
@@ -161,6 +168,20 @@ static mapping_methods dbm_as_mapping = {
 	(objobjargproc)dbm_ass_sub,	/*mp_ass_subscript*/
 };
 
+static object *
+dbm__close(dp, args)
+	register dbmobject *dp;
+	object *args;
+{
+	if ( !getnoarg(args) )
+		return NULL;
+        if ( dp->di_dbm )
+	    dbm_close(dp->di_dbm);
+	dp->di_dbm = NULL;
+	INCREF(None);
+	return None;
+}
+
 static object *
 dbm_keys(dp, args)
 	register dbmobject *dp;
@@ -170,10 +191,6 @@ dbm_keys(dp, args)
 	datum key;
 	int err;
 
-	if (dp == NULL || !is_dbmobject(dp)) {
-		err_badcall();
-		return NULL;
-	}
 	if (!getnoarg(args))
 		return NULL;
 	v = newlistobject(0);
@@ -196,7 +213,6 @@ dbm_keys(dp, args)
 	return v;
 }
 
-
 static object *
 dbm_has_key(dp, args)
 	register dbmobject *dp;
@@ -211,6 +227,7 @@ dbm_has_key(dp, args)
 }
 
 static struct methodlist dbm_methods[] = {
+	{"close",	(method)dbm__close},
 	{"keys",	(method)dbm_keys},
 	{"has_key",	(method)dbm_has_key},
 	{NULL,		NULL}		/* sentinel */
@@ -227,7 +244,7 @@ dbm_getattr(dp, name)
 static typeobject Dbmtype = {
 	OB_HEAD_INIT(&Typetype)
 	0,
-	"Dbm_dictionary",
+	"dbm",
 	sizeof(dbmobject),
 	0,
 	(destructor)dbm_dealloc, /*tp_dealloc*/
@@ -248,27 +265,33 @@ dbmopen(self, args)
     object *self;
     object *args;
 {
-    char *name, *flags;
-    int iflags, mode;
+	char *name;
+	char *flags = "r";
+	int iflags;
+	int mode = 0666;
 
-        if ( !getargs(args, "(ssi)", &name, &flags, &mode) )
-	  return 0;
+        if ( !newgetargs(args, "s|si", &name, &flags, &mode) )
+	  return NULL;
 	if ( strcmp(flags, "r") == 0 )
 	  iflags = O_RDONLY;
 	else if ( strcmp(flags, "w") == 0 )
-	  iflags = O_WRONLY|O_CREAT;
-	else if ( strcmp(flags, "rw") == 0 )
+	  iflags = O_RDWR;
+	else if ( strcmp(flags, "rw") == 0 ) /* B/W compat */
+	  iflags = O_RDWR|O_CREAT; 
+	else if ( strcmp(flags, "c") == 0 )
 	  iflags = O_RDWR|O_CREAT;
+	else if ( strcmp(flags, "n") == 0 )
+	  iflags = O_RDWR|O_CREAT|O_TRUNC;
 	else {
 	    err_setstr(DbmError,
-		       "Flags should be one of 'r', 'w' or 'rw'");
-	    return 0;
+		       "Flags should be one of 'r', 'w', 'c' or 'n'");
+	    return NULL;
 	}
         return newdbmobject(name, iflags, mode);
 }
 
 static struct methodlist dbmmodule_methods[] = {
-    { "open", (method)dbmopen },
+    { "open", (method)dbmopen, 1 },
     { 0, 0 },
 };
 
@@ -282,5 +305,3 @@ initdbm() {
     if ( DbmError == NULL || dictinsert(d, "error", DbmError) )
       fatal("can't define dbm.error");
 }
-
-	
