@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -33,20 +40,15 @@ extern char *optarg;
 extern int getopt(); /* PROTO((int, char **, char *)); -- not standardized */
 
 
-extern int Py_DebugFlag; /* For parser.c, declared in pythonrun.c */
-extern int Py_VerboseFlag; /* For import.c, declared in pythonrun.c */
-extern int Py_SuppressPrintingFlag; /* For ceval.c, declared in pythonrun.c */
-
-
 /* Subroutines that live in their own file */
-extern char *getversion();
-extern char *getcopyright();
+extern char *Py_GetVersion();
+extern char *Py_GetCopyright();
 
 
-/* For getprogramname(); set by main() */
+/* For Py_GetProgramName(); set by main() */
 static char *argv0;
 
-/* For getargcargv(); set by main() */
+/* For Py_GetArgcArgv(); set by main() */
 static char **orig_argv;
 static int  orig_argc;
 
@@ -93,9 +95,9 @@ main(argc, argv)
 	int inspect = 0;
 	int unbuffered = 0;
 
-	orig_argc = argc;	/* For getargcargv() */
+	orig_argc = argc;	/* For Py_GetArgcArgv() */
 	orig_argv = argv;
-	argv0 = argv[0];	/* For getprogramname() */
+	argv0 = argv[0];	/* For Py_GetProgramName() */
 
 	if ((p = getenv("PYTHONDEBUG")) && *p != '\0')
 		Py_DebugFlag = 1;
@@ -174,7 +176,7 @@ main(argc, argv)
 	if (Py_VerboseFlag ||
 	    command == NULL && filename == NULL && isatty((int)fileno(fp)))
 		fprintf(stderr, "Python %s\n%s\n",
-			getversion(), getcopyright());
+			Py_GetVersion(), Py_GetCopyright());
 	
 	if (filename != NULL) {
 		if ((fp = fopen(filename, "r")) == NULL) {
@@ -224,10 +226,11 @@ main(argc, argv)
 }
 
 
-/* Return the program name -- some code out there needs this. */
+/* Return the program name -- some code out there needs this
+   (currently _tkinter.c and importdl.c). */
 
 char *
-getprogramname()
+Py_GetProgramName()
 {
 	return argv0;
 }
@@ -237,7 +240,7 @@ getprogramname()
    This is rare, but it is needed by the secureware extension. */
 
 void
-getargcargv(argc,argv)
+Py_GetArgcArgv(argc, argv)
 	int *argc;
 	char ***argv;
 {
