@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -33,16 +40,15 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 #include "mymath.h"
 
+#ifndef _MSC_VER
 #ifndef __STDC__
 extern double fmod PROTO((double, double));
 extern double frexp PROTO((double, int *));
 extern double ldexp PROTO((double, int));
 extern double modf PROTO((double, double *));
-#endif
+#endif /* __STDC__ */
+#endif /* _MSC_VER */
 
-#ifdef HAVE_HYPOT
-extern double hypot PROTO((double, double));
-#endif
 
 #ifdef i860
 /* Cray APP has bogus definition of HUGE_VAL in <math.h> */
@@ -121,12 +127,15 @@ FUNC1(math_ceil, ceil)
 FUNC1(math_cos, cos)
 FUNC1(math_cosh, cosh)
 FUNC1(math_exp, exp)
+#ifdef __MWERKS__
+double myfabs(double x) { return fabs(x); }
+FUNC1(math_fabs, myfabs)
+#else
 FUNC1(math_fabs, fabs)
+#endif
 FUNC1(math_floor, floor)
 FUNC2(math_fmod, fmod)
-#ifdef HAVE_HYPOT
 FUNC2(math_hypot, hypot)
-#endif
 FUNC1(math_log, log)
 FUNC1(math_log10, log10)
 #ifdef MPW_3_1 /* This hack is needed for MPW 3.1 but not for 3.2 ... */
@@ -213,9 +222,7 @@ static struct methodlist math_methods[] = {
 	{"floor", math_floor},
 	{"fmod", math_fmod},
 	{"frexp", math_frexp},
-#ifdef HAVE_HYPOT
 	{"hypot", math_hypot},
-#endif
 	{"ldexp", math_ldexp},
 	{"log", math_log},
 	{"log10", math_log10},
