@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -76,17 +83,22 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 /*
 ** IMHO, mpz_m{div,mod,divmod}() do the wrong things when the denominator < 0
-** I assume that this will be fixed in a future release
+** This has been fixed with gmp release 2.0
 */
 /*#define MPZ_MDIV_BUG fixed the (for me) nexessary parts in libgmp.a */
 /*
 ** IMO, mpz_get_str() assumes a bit too large target space, if he doesn't
 ** allocate it himself
 */
-#undef MPZ_GET_STR_BUG
 
 #include "gmp.h"
-#include "gmp-mparam.h"
+#include "gmp-impl.h"
+
+#if __GNU_MP__ == 2
+#define GMP2
+#else
+#define MPZ_GET_STR_BUG
+#endif
 
 typedef struct {
 	OB_HEAD
@@ -119,7 +131,6 @@ newmpzobject()
 } /* newmpzobject() */
 
 #ifdef MPZ_GET_STR_BUG
-#include "gmp-impl.h"
 #include "longlong.h"
 #endif /* def MPZ_GET_STR_BUG */
 
@@ -164,8 +175,13 @@ mpz_format(objp, base, withname)
 		(int)mpz_sizeinbase(&mpzp->mpz, base));
 #endif /* def MPZ_DEBUG */
 #ifdef MPZ_GET_STR_BUG
+#ifdef GMP2
+	i += ((size_t) abs(mpzp->mpz._mp_size) * BITS_PER_MP_LIMB
+	      * __mp_bases[base].chars_per_bit_exactly) + 1;
+#else
 	i += ((size_t) abs(mpzp->mpz.size) * BITS_PER_MP_LIMB
 	      * __mp_bases[base].chars_per_bit_exactly) + 1;
+#endif
 #else /* def MPZ_GET_STR_BUG */
 	i += (int)mpz_sizeinbase(&mpzp->mpz, base);
 #endif /* def MPZ_GET_STR_BUG else */
@@ -582,7 +598,6 @@ mpz_power(a, b, m)
 {
 	mpzobject *z;
 	int cmpres;
-	long int longtmp1, longtmp2;
 
  	if ((object *)m!=Py_None)
  	  {
@@ -727,7 +742,7 @@ mpz_nonzero(v)
 } /* mpz_nonzero() */
 		
 static object *
-mpz_invert(v)
+py_mpz_invert(v)
 	mpzobject *v;
 {
 	mpzobject *z;
@@ -739,7 +754,7 @@ mpz_invert(v)
 
 	mpz_com(&z->mpz, &v->mpz);
 	return (object *)z;
-} /* mpz_invert() */
+} /* py_mpz_invert() */
 
 static object *
 mpz_lshift(a, b)
@@ -927,7 +942,7 @@ MPZ_mpz(self, args)
 		mpz_init(&mplongdigit);
 		
 		/* how we're gonna handle this? */
-		if (isnegative = ((i = ((longobject *)objp)->ob_size) < 0) )
+		if ((isnegative = ((i = ((longobject *)objp)->ob_size) < 0) ))
 			i = -i;
 
 		while (i--) {
@@ -1222,7 +1237,11 @@ mpz_divm(res, num, den, mod)
 	mpz_init_set(&d0, den);
 	mpz_init_set(&d1, mod);
 
+#ifdef GMP2
+	while (d1._mp_size != 0) {
+#else
 	while (d1.size != 0) {
+#endif
 		mpz_divmod(&q, &r, &d0, &d1);
 		mpz_set(&d0, &d1);
 		mpz_set(&d1, &r);
@@ -1233,8 +1252,13 @@ mpz_divm(res, num, den, mod)
 		mpz_set(&s1, &x);
 	}
 
+#ifdef GMP2
+	if (d0._mp_size != 1 || d0._mp_d[0] != 1)
+		res->_mp_size = 0; /* trouble: the gcd != 1; set s to zero */
+#else
 	if (d0.size != 1 || d0.d[0] != 1)
 		res->size = 0;	/* trouble: the gcd != 1; set s to zero */
+#endif
 	else {
 #ifdef MPZ_MDIV_BUG
 		/* watch out here! first check the signs, and then perform
@@ -1366,7 +1390,7 @@ mpz_long(self)
 
 	/* determine sign, and copy self to scratch var */
 	mpz_init_set(&mpzscratch, &self->mpz);
-	if (isnegative = (mpz_cmp_ui(&self->mpz, (unsigned long int)0) < 0))
+	if ((isnegative = (mpz_cmp_ui(&self->mpz, (unsigned long int)0) < 0)))
 		mpz_neg(&mpzscratch, &mpzscratch);
 
 	/* let those bits come, let those bits go,
@@ -1442,7 +1466,7 @@ mpz_float(self)
 	i = (int)mpz_size(&self->mpz);
 	
 	/* determine sign, and copy abs(self) to scratch var */
-	if (isnegative = (mpz_cmp_ui(&self->mpz, (unsigned long int)0) < 0)) {
+	if ((isnegative = (mpz_cmp_ui(&self->mpz, (unsigned long int)0) < 0))) {
 		mpz_init(&mpzscratch);
 		mpz_neg(&mpzscratch, &self->mpz);
 	}
@@ -1637,7 +1661,7 @@ static number_methods mpz_as_number = {
 	UF mpz_positive,	/*tp_positive*/
 	UF mpz_absolute,	/*tp_absolute*/
 	IF mpz_nonzero,		/*tp_nonzero*/
-	UF mpz_invert,		/*nb_invert*/
+	UF py_mpz_invert,	/*nb_invert*/
 	BF mpz_lshift,		/*nb_lshift*/
 	BF mpz_rshift,		/*nb_rshift*/
 	BF mpz_andfunc,		/*nb_and*/
