@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -27,6 +34,31 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "allobjects.h"
 #include "compile.h"
 
+static char new_instance_doc[] =
+"Create an instance object from (CLASS, DICT) without calling its __init__().";
+
+static object *
+new_instance(unused, args)
+	object* unused;
+	object* args;
+{
+	object* klass;
+	object *dict;
+	instanceobject *inst;
+	if (!newgetargs(args, "O!O!",
+			&Classtype, &klass,
+			&Dicttype, &dict))
+		return NULL;
+	inst = NEWOBJ(instanceobject, &Instancetype);
+	if (inst == NULL)
+		return NULL;
+	INCREF(klass);
+	INCREF(dict);
+	inst->in_class = (classobject *)klass;
+	inst->in_dict = dict;
+	return (object *)inst;
+}
+
 static char new_im_doc[] =
 "Create a instance method object from (FUNCTION, INSTANCE, CLASS).";
 
@@ -144,13 +176,33 @@ new_module(unused, args)
 	return newmoduleobject(name);
 }
 
+static char new_class_doc[] =
+"Create a class object from (NAME, BASE_CLASSES, DICT).";
+
+static object *
+new_class(unused, args)
+	object* unused;
+	object* args;
+{
+	object * name;
+	object * classes;
+	object * dict;
+  
+	if (!newgetargs(args, "SO!O!", &name, &Tupletype, &classes,
+			&Mappingtype, &dict))
+		return NULL;
+	return newclassobject(classes, dict, name);
+}
+
 static struct methodlist new_methods[] = {
+	{"instance",		new_instance,		1, new_instance_doc},
 	{"instancemethod",	new_instancemethod,	1, new_im_doc},
 #if 0
 	{"function",		new_function,		1, new_function_doc},
 #endif
 	{"code",		new_code,		1, new_code_doc},
 	{"module",		new_module,		1, new_module_doc},
+	{"classobj",		new_class,		1, new_class_doc},
 	{NULL,			NULL}		/* sentinel */
 };
 
