@@ -4,34 +4,41 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
 /* POSIX module implementation */
 
-/* This file is also used for Windows NT.  In that case the module
+/* This file is also used for Windows NT and MS-Win.  In that case the module
    actually calls itself 'nt', not 'posix', and a few functions are
    either unimplemented or implemented differently.  The source
-   assumes that for Windows NT, the macro 'NT' is defined independent
+   assumes that for Windows NT, the macro 'MS_WIN32' is defined independent
    of the compiler used.  Different compilers define their own feature
-   test macro, e.g. '__BORLANDC__' or '_MSCVER'. */
+   test macro, e.g. '__BORLANDC__' or '_MSC_VER'. */
 
-/* For MS-DOS and Windows 3.x, use ../Dos/dosmodule.c */
+/* See also ../Dos/dosmodule.c */
 
 #include "allobjects.h"
 #include "modsupport.h"
@@ -41,6 +48,9 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_SYS_WAIT_H
+#include <sys/wait.h>		/* For WNOHANG */
+#endif
 
 #include "mytime.h"		/* For clock_t on some systems */
 
@@ -48,34 +58,86 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <fcntl.h>
 #endif /* HAVE_FCNTL_H */
 
-#ifndef NT
-#define HAVE_FORK	1
+/* Various compilers have only certain posix functions */
+#ifdef __WATCOMC__		/* Watcom compiler */
+#define HAVE_GETCWD     1
+#define HAVE_OPENDIR    1
+#define HAVE_SYSTEM	1
+#if defined(__OS2__)
+#define HAVE_EXECV      1
+#define HAVE_WAIT       1
 #endif
+#include <process.h>
+#else
+#ifdef __BORLANDC__		/* Borland compiler */
+#define HAVE_EXECV      1
+#define HAVE_GETCWD     1
+#define HAVE_GETEGID    1
+#define HAVE_GETEUID    1
+#define HAVE_GETGID     1
+#define HAVE_GETPPID    1
+#define HAVE_GETUID     1
+#define HAVE_KILL       1
+#define HAVE_OPENDIR    1
+#define HAVE_PIPE       1
+#define HAVE_POPEN      1
+#define HAVE_SYSTEM	1
+#define HAVE_WAIT       1
+#else
+#ifdef _MSC_VER		/* Microsoft compiler */
+#define HAVE_GETCWD     1
+#ifdef MS_WIN32
+#define HAVE_EXECV      1
+#define HAVE_PIPE       1
+#define HAVE_POPEN      1
+#define HAVE_SYSTEM	1
+#else /* 16-bit Windows */
+#endif /* !MS_WIN32 */
+#else			/* all other compilers */
+/* Unix functions that the configure script doesn't check for */
+#define HAVE_EXECV      1
+#define HAVE_FORK       1
+#define HAVE_GETCWD     1
+#define HAVE_GETEGID    1
+#define HAVE_GETEUID    1
+#define HAVE_GETGID     1
+#define HAVE_GETPPID    1
+#define HAVE_GETUID     1
+#define HAVE_KILL       1
+#define HAVE_OPENDIR    1
+#define HAVE_PIPE       1
+#define HAVE_POPEN      1
+#define HAVE_SYSTEM	1
+#define HAVE_WAIT       1
+#endif  /* _MSC_VER */
+#endif  /* __BORLANDC__ */
+#endif  /* ! __WATCOMC__ */
 
-#if !defined(NT) || defined(__BORLANDC__)
-/* Unix functions that the configure script doesn't check for
-   and that aren't easily available under NT except with Borland C */
-#define HAVE_GETEGID	1
-#define HAVE_GETEUID	1
-#define HAVE_GETGID	1
-#define HAVE_GETPPID	1
-#define HAVE_GETUID	1
-#define HAVE_KILL	1
-#define HAVE_WAIT	1
-#define HAVE_OPENDIR	1
-#define HAVE_PIPE	1
-#endif
+#ifndef _MSC_VER
 
-#ifndef NT
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+
+#ifdef NeXT
+/* NeXT's <unistd.h> and <utime.h> aren't worth much */
+#undef HAVE_UNISTD_H
+#undef HAVE_UTIME_H
+/* #undef HAVE_GETCWD */
+#endif
+
+#ifdef HAVE_UNISTD_H
 /* XXX These are for SunOS4.1.3 but shouldn't hurt elsewhere */
 extern int rename();
 extern int pclose();
 extern int lstat();
 extern int symlink();
 #else /* !HAVE_UNISTD_H */
+#if defined(__WATCOMC__) || defined(_MSC_VER)
+extern int mkdir PROTO((const char *));
+#else
 extern int mkdir PROTO((const char *, mode_t));
+#endif
 extern int chdir PROTO((const char *));
 extern int rmdir PROTO((const char *));
 extern int chmod PROTO((const char *, mode_t));
@@ -94,7 +156,8 @@ extern int symlink PROTO((const char *, const char *));
 extern int lstat PROTO((const char *, struct stat *));
 #endif /* HAVE_LSTAT */
 #endif /* !HAVE_UNISTD_H */
-#endif /* !NT */
+
+#endif /* !_MSC_VER */
 
 #ifdef HAVE_UTIME_H
 #include <utime.h>
@@ -125,8 +188,13 @@ extern int lstat PROTO((const char *, struct stat *));
 #include <dirent.h>
 #define NAMLEN(dirent) strlen((dirent)->d_name)
 #else
+#ifdef __WATCOMC__
+#include <direct.h>
+#define NAMLEN(dirent) strlen((dirent)->d_name)
+#else
 #define dirent direct
 #define NAMLEN(dirent) (dirent)->d_namlen
+#endif
 #ifdef HAVE_SYS_NDIR_H
 #include <sys/ndir.h>
 #endif
@@ -138,14 +206,19 @@ extern int lstat PROTO((const char *, struct stat *));
 #endif
 #endif
 
-#ifdef NT
+#ifdef _MSC_VER
 #include <direct.h>
 #include <io.h>
 #include <process.h>
 #include <windows.h>
-#define popen   _popen
+#ifdef MS_WIN32
+#define popen	_popen
 #define pclose	_pclose
-#endif /* NT */
+#else /* 16-bit Windows */
+#include <dos.h>
+#include <ctype.h>
+#endif /* MS_WIN32 */
+#endif /* _MSC_VER */
 
 #ifdef OS2
 #include <io.h>
@@ -153,9 +226,9 @@ extern int lstat PROTO((const char *, struct stat *));
 
 /* Return a dictionary corresponding to the POSIX environment table */
 
-#ifndef NT
+#if !defined(_MSC_VER) && !defined(__WATCOMC__)
 extern char **environ;
-#endif /* !NT */
+#endif /* !_MSC_VER */
 
 static object *
 convertenviron()
@@ -329,6 +402,7 @@ posix_chown(self, args)
 }
 #endif /* HAVE_CHOWN */
 
+#ifdef HAVE_GETCWD
 static object *
 posix_getcwd(self, args)
 	object *self;
@@ -345,6 +419,7 @@ posix_getcwd(self, args)
 		return posix_error();
 	return newstringobject(buf);
 }
+#endif
 
 #ifdef HAVE_LINK
 static object *
@@ -361,7 +436,7 @@ posix_listdir(self, args)
 	object *self;
 	object *args;
 {
-#if defined(NT) && !defined(HAVE_OPENDIR)
+#if defined(MS_WIN32) && !defined(HAVE_OPENDIR)
 
 	char *name;
 	int len;
@@ -390,6 +465,11 @@ posix_listdir(self, args)
 		return posix_error();
 	}
 	do {
+		if (FileData.cFileName[0] == '.' &&
+		    (FileData.cFileName[1] == '\0' ||
+		     FileData.cFileName[1] == '.' &&
+		     FileData.cFileName[2] == '\0'))
+			continue;
 		v = newstringobject(FileData.cFileName);
 		if (v == NULL) {
 			DECREF(d);
@@ -412,7 +492,68 @@ posix_listdir(self, args)
 
 	return d;
 
-#else /* !NT */
+#else /* !MS_WIN32 */
+#ifdef _MSC_VER /* 16-bit Windows */
+
+#ifndef MAX_PATH
+#define MAX_PATH	250
+#endif
+	char *name, *pt;
+	int len;
+	object *d, *v;
+	char namebuf[MAX_PATH+5];
+	struct _find_t ep;
+
+	if (!getargs(args, "s#", &name, &len))
+		return NULL;
+	if (len >= MAX_PATH) {
+		err_setstr(ValueError, "path too long");
+		return NULL;
+	}
+	strcpy(namebuf, name);
+	for (pt = namebuf; *pt; pt++)
+		if (*pt == '/')
+			*pt = '\\';
+	if (namebuf[len-1] != '\\')
+		namebuf[len++] = '\\';
+	strcpy(namebuf + len, "*.*");
+
+	if ((d = newlistobject(0)) == NULL)
+		return NULL;
+
+	if (_dos_findfirst(namebuf, _A_RDONLY |
+			_A_HIDDEN | _A_SYSTEM | _A_SUBDIR, &ep) != 0){
+		errno = ENOENT;
+		return posix_error();
+	}
+	do {
+		if (ep.name[0] == '.' &&
+		    (ep.name[1] == '\0' ||
+		     ep.name[1] == '.' &&
+		     ep.name[2] == '\0'))
+			continue;
+		strcpy(namebuf, ep.name);
+		for (pt = namebuf; *pt; pt++)
+			if (isupper(*pt))
+				*pt = tolower(*pt);
+		v = newstringobject(namebuf);
+		if (v == NULL) {
+			DECREF(d);
+			d = NULL;
+			break;
+		}
+		if (addlistitem(d, v) != 0) {
+			DECREF(v);
+			DECREF(d);
+			d = NULL;
+			break;
+		}
+		DECREF(v);
+	} while (_dos_findnext(&ep) == 0);
+
+	return d;
+
+#else
 
 	char *name;
 	object *d, *v;
@@ -431,6 +572,10 @@ posix_listdir(self, args)
 		return NULL;
 	}
 	while ((ep = readdir(dirp)) != NULL) {
+		if (ep->d_name[0] == '.' &&
+		    (NAMLEN(ep) == 1 ||
+		     ep->d_name[1] == '.' && NAMLEN(ep) == 2))
+			continue;
 		v = newsizedstringobject(ep->d_name, NAMLEN(ep));
 		if (v == NULL) {
 			DECREF(d);
@@ -450,7 +595,8 @@ posix_listdir(self, args)
 
 	return d;
 
-#endif /* !NT */
+#endif /* !_MSC_VER */
+#endif /* !MS_WIN32 */
 }
 
 static object *
@@ -458,7 +604,22 @@ posix_mkdir(self, args)
 	object *self;
 	object *args;
 {
-	return posix_strint(args, mkdir);
+	int res;
+	char *path;
+	int mode = 0777;
+	if (!newgetargs(args, "s|i", &path, &mode))
+		return NULL;
+	BGN_SAVE
+#if defined(__WATCOMC__) || defined(_MSC_VER)
+	res = mkdir(path);
+#else
+	res = mkdir(path, mode);
+#endif
+	END_SAVE
+	if (res < 0)
+		return posix_error();
+	INCREF(None);
+	return None;
 }
 
 #ifdef HAVE_NICE
@@ -502,6 +663,7 @@ posix_stat(self, args)
 	return posix_do_stat(self, args, stat);
 }
 
+#ifdef HAVE_SYSTEM
 static object *
 posix_system(self, args)
 	object *self;
@@ -516,6 +678,7 @@ posix_system(self, args)
 	END_SAVE
 	return newintobject(sts);
 }
+#endif
 
 static object *
 posix_umask(self, args)
@@ -616,6 +779,7 @@ posix__exit(self, args)
 	/* NOTREACHED */
 }
 
+#ifdef HAVE_EXECV
 static object *
 posix_execv(self, args)
 	object *self;
@@ -763,6 +927,7 @@ posix_execve(self, args)
 
 	return NULL;
 }
+#endif /* HAVE_EXECV */
 
 #ifdef HAVE_FORK
 static object *
@@ -902,6 +1067,28 @@ posix_kill(self, args)
 }
 #endif
 
+#ifdef HAVE_PLOCK
+
+#ifdef HAVE_SYS_LOCK_H
+#include <sys/lock.h>
+#endif
+
+static object *
+posix_plock(self, args)
+	object *self;
+	object *args;
+{
+	int op;
+	if (!getargs(args, "i", &op))
+		return NULL;
+	if (plock(op) == -1)
+		return posix_error();
+	INCREF(None);
+	return None;
+}
+#endif
+
+#ifdef HAVE_POPEN
 static object *
 posix_popen(self, args)
 	object *self;
@@ -924,6 +1111,7 @@ posix_popen(self, args)
 		setfilebufsize(f, bufsize);
 	return f;
 }
+#endif /* HAVE_POPEN */
 
 #ifdef HAVE_SETUID
 static object *
@@ -963,7 +1151,7 @@ posix_waitpid(self, args)
 	object *self;
 	object *args;
 {
-	int pid, options, sts;
+	int pid, options, sts = 0;
 	if (!getargs(args, "(ii)", &pid, &options))
 		return NULL;
 	BGN_SAVE
@@ -1052,14 +1240,15 @@ posix_times(self, args)
 	c = times(&t);
 	if (c == (clock_t) -1)
 		return posix_error();
-	return mkvalue("dddd",
+	return mkvalue("ddddd",
 		       (double)t.tms_utime / HZ,
 		       (double)t.tms_stime / HZ,
 		       (double)t.tms_cutime / HZ,
-		       (double)t.tms_cstime / HZ);
+		       (double)t.tms_cstime / HZ,
+		       (double)c / HZ);
 }
 #endif /* HAVE_TIMES */
-#if defined(NT) && !defined(HAVE_TIMES)
+#ifdef MS_WIN32
 #define HAVE_TIMES	/* so the method table will pick it up */
 static object *
 posix_times(self, args)
@@ -1072,13 +1261,14 @@ posix_times(self, args)
 		return NULL;
 	hProc = GetCurrentProcess();
 	GetProcessTimes(hProc,&create, &exit, &kernel, &user);
-	return mkvalue("dddd",
+	return mkvalue("ddddd",
 		       (double)(kernel.dwHighDateTime*2E32+kernel.dwLowDateTime) / 2E6,
 		       (double)(user.dwHighDateTime*2E32+user.dwLowDateTime) / 2E6,
 		       (double)0,
+		       (double)0,
 		       (double)0);
 }
-#endif /* NT */
+#endif /* MS_WIN32 */
 
 #ifdef HAVE_SETSID
 static object *
@@ -1247,7 +1437,7 @@ posix_read(self, args)
 	object *self;
 	object *args;
 {
-	int fd, size;
+	int fd, size, n;
 	object *buffer;
 	if (!getargs(args, "(ii)", &fd, &size))
 		return NULL;
@@ -1255,13 +1445,14 @@ posix_read(self, args)
 	if (buffer == NULL)
 		return NULL;
 	BGN_SAVE
-	size = read(fd, getstringvalue(buffer), size);
+	n = read(fd, getstringvalue(buffer), size);
 	END_SAVE
-	if (size < 0) {
+	if (n < 0) {
 		DECREF(buffer);
 		return posix_error();
 	}
-	resizestring(&buffer, size);
+	if (n != size)
+		resizestring(&buffer, n);
 	return buffer;
 }
 
@@ -1334,12 +1525,13 @@ posix_fdopen(self, args)
 	return f;
 }
 
+#ifdef HAVE_PIPE
 static object *
 posix_pipe(self, args)
 	object *self;
 	object *args;
 {
-#if !defined(NT) || defined(HAVE_PIPE)
+#if !defined(MS_WIN32)
 	int fds[2];
 	int res;
 	if (!getargs(args, ""))
@@ -1350,7 +1542,7 @@ posix_pipe(self, args)
 	if (res != 0)
 		return posix_error();
 	return mkvalue("(ii)", fds[0], fds[1]);
-#else /* NT */
+#else /* MS_WIN32 */
 	HANDLE read, write;
 	BOOL ok;
 	if (!getargs(args, ""))
@@ -1361,8 +1553,79 @@ posix_pipe(self, args)
 	if (!ok)
 		return posix_error();
 	return mkvalue("(ii)", read, write);
-#endif /* NT */
+#endif /* MS_WIN32 */
 }
+#endif  /* HAVE_PIPE */
+
+#ifdef HAVE_MKFIFO
+static object *
+posix_mkfifo(self, args)
+	object *self;
+	object *args;
+{
+	char *file;
+	int mode = 0666;
+	int res;
+	if (!newgetargs(args, "s|i", &file, &mode))
+		return NULL;
+	BGN_SAVE
+	res = mkfifo(file, mode);
+	END_SAVE
+	if (res < 0)
+		return posix_error();
+	INCREF(None);
+	return None;
+}
+#endif
+
+#ifdef HAVE_FTRUNCATE
+static object *
+posix_ftruncate(self, args)
+	object *self; /* Not used */
+	object *args;
+{
+	int fd;
+	long length;
+	int res;
+
+	if (!getargs(args, "(il)", &fd, &length))
+		return NULL;
+
+	BGN_SAVE
+	res = ftruncate(fd, length);
+	END_SAVE
+	if (res < 0) {
+		err_errno(IOError);
+		return NULL;
+	}
+	INCREF(None);
+	return None;
+}
+#endif
+
+#ifdef HAVE_PUTENV
+static object * 
+posix_putenv(self,args)
+	object *self;
+	object *args;
+{
+        char *s1, *s2;
+        char *new;
+
+	if (!newgetargs(args, "ss", &s1, &s2))
+		return NULL;
+	/* XXX This leaks memory -- not easy to fix :-( */
+	if ((new = malloc(strlen(s1) + strlen(s2) + 2)) == NULL)
+                return err_nomem();
+	(void) sprintf(new, "%s=%s", s1, s2);
+	if (putenv(new)) {
+                posix_error();
+                return NULL;
+	}
+	INCREF(None);
+        return None;
+}
+#endif
 
 static struct methodlist posix_methods[] = {
 	{"chdir",	posix_chdir},
@@ -1370,13 +1633,15 @@ static struct methodlist posix_methods[] = {
 #ifdef HAVE_CHOWN
 	{"chown",	posix_chown},
 #endif /* HAVE_CHOWN */
+#ifdef HAVE_GETCWD
 	{"getcwd",	posix_getcwd},
+#endif
 #ifdef HAVE_LINK
 	{"link",	posix_link},
 #endif /* HAVE_LINK */
 	{"listdir",	posix_listdir},
 	{"lstat",	posix_lstat},
-	{"mkdir",	posix_mkdir},
+	{"mkdir",	posix_mkdir, 1},
 #ifdef HAVE_NICE
 	{"nice",	posix_nice},
 #endif /* HAVE_NICE */
@@ -1389,19 +1654,24 @@ static struct methodlist posix_methods[] = {
 #ifdef HAVE_SYMLINK
 	{"symlink",	posix_symlink},
 #endif /* HAVE_SYMLINK */
+#ifdef HAVE_SYSTEM
 	{"system",	posix_system},
+#endif
 	{"umask",	posix_umask},
 #ifdef HAVE_UNAME
 	{"uname",	posix_uname},
 #endif /* HAVE_UNAME */
 	{"unlink",	posix_unlink},
+	{"remove",	posix_unlink},
 	{"utime",	posix_utime},
 #ifdef HAVE_TIMES
 	{"times",	posix_times},
 #endif /* HAVE_TIMES */
 	{"_exit",	posix__exit},
+#ifdef HAVE_EXECV
 	{"execv",	posix_execv},
 	{"execve",	posix_execve},
+#endif /* HAVE_EXECV */
 #ifdef HAVE_FORK
 	{"fork",	posix_fork},
 #endif /* HAVE_FORK */
@@ -1427,7 +1697,12 @@ static struct methodlist posix_methods[] = {
 #ifdef HAVE_KILL
 	{"kill",	posix_kill},
 #endif /* HAVE_KILL */
+#ifdef HAVE_PLOCK
+	{"plock",	posix_plock},
+#endif /* HAVE_PLOCK */
+#ifdef HAVE_POPEN
 	{"popen",	posix_popen,	1},
+#endif /* HAVE_POPEN */
 #ifdef HAVE_SETUID
 	{"setuid",	posix_setuid},
 #endif /* HAVE_SETUID */
@@ -1464,12 +1739,23 @@ static struct methodlist posix_methods[] = {
 	{"write",	posix_write},
 	{"fstat",	posix_fstat},
 	{"fdopen",	posix_fdopen,	1},
+#ifdef HAVE_PIPE
 	{"pipe",	posix_pipe},
+#endif
+#ifdef HAVE_MKFIFO
+	{"mkfifo",	posix_mkfifo, 1},
+#endif
+#ifdef HAVE_FTRUNCATE
+	{"ftruncate",	posix_ftruncate, 1},
+#endif
+#ifdef HAVE_PUTENV
+	{"putenv",	posix_putenv, 1},
+#endif
 	{NULL,		NULL}		 /* Sentinel */
 };
 
 
-#ifdef NT
+#if defined(_MSC_VER) || defined(__WATCOMC__)
 void
 initnt()
 {
@@ -1489,7 +1775,7 @@ initnt()
 	if (PosixError == NULL || dictinsert(d, "error", PosixError) != 0)
 		fatal("can't define nt.error");
 }
-#else /* !NT */
+#else /* not a PC port */
 void
 initposix()
 {
@@ -1517,4 +1803,4 @@ initposix()
 	if (PosixError == NULL || dictinsert(d, "error", PosixError) != 0)
 		fatal("can't define posix.error");
 }
-#endif /* !NT */
+#endif /* !_MSC_VER */
