@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -31,6 +38,7 @@ Limitations:
 
 - only AF_INET and AF_UNIX address families are supported
 - no read/write operations (use send/recv or makefile instead)
+- additional restrictions apply on Windows
 
 Module interface:
 
@@ -40,6 +48,10 @@ Module interface:
 - socket.gethostname() --> host name (string: 'spam' or 'spam.domain.com')
 - socket.getservbyname(servicename, protocolname) --> port number
 - socket.socket(family, type [, proto]) --> new socket object
+- socket.ntohs(16 bit value) --> new int object
+- socket.ntohl(32 bit value) --> new int object
+- socket.htons(16 bit value) --> new int object
+- socket.htonl(32 bit value) --> new int object
 - socket.AF_INET, socket.SOCK_STREAM, etc.: constants from <socket.h>
 - an Internet socket address is a pair (hostname, port)
   where hostname can be anything recognized by gethostbyname()
@@ -54,6 +66,7 @@ Socket methods:
 - s.close() --> None
 - s.connect(sockaddr) --> None
 - s.fileno() --> file descriptor
+- s.dup() --> same as socket.fromfd(os.dup(s.fileno(), ...)
 - s.getpeername() --> sockaddr
 - s.getsockname() --> sockaddr
 - s.getsockopt(level, optname[, buflen]) --> int or string
@@ -76,7 +89,7 @@ Socket methods:
 #include "mytime.h"
 
 #include <signal.h>
-#ifndef NT
+#ifndef MS_WINDOWS
 #include <netdb.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -95,16 +108,21 @@ Socket methods:
 #define O_NDELAY O_NONBLOCK	/* For QNX only? */
 #endif
 
+#ifdef USE_GUSI
+/* fdopen() isn't declared in stdio.h (sigh) */
+#include <GUSI.h>
+#endif
+
 
 /* Here we have some hacks to choose between K&R or ANSI style function
    definitions.  For NT to build this as an extension module (ie, DLL)
    it must be compiled by the C++ compiler, as it takes the address of
    a static data item exported from the main Python DLL.
 */
-#ifdef NT
+#ifdef MS_WINDOWS
 /* seem to be a few differences in the API */
 #define close closesocket
-#define NO_DUP	/* Define for NT 3.1, Win3.1 and Win95, Undefine for NT3.5 */
+#define NO_DUP /* Actually it exists on NT 3.5, but what the heck... */
 #define FORCE_ANSI_FUNC_DEFS
 #endif
 
@@ -158,7 +176,7 @@ static PyObject *PySocket_Error;
 static PyObject *
 PySocket_Err()
 {
-#ifdef NT
+#ifdef MS_WINDOWS
 	if (WSAGetLastError()) {
 		PyObject *v;
 		v = Py_BuildValue("(is)", WSAGetLastError(), "winsock error");
@@ -210,6 +228,7 @@ static PySocketSockObject *
 BUILD_FUNC_DEF_4(PySocketSock_New,int,fd, int,family, int,type, int,proto)
 {
 	PySocketSockObject *s;
+	PySocketSock_Type.ob_type = &PyType_Type;
 	s = PyObject_NEW(PySocketSockObject, &PySocketSock_Type);
 	if (s != NULL) {
 		s->sock_fd = fd;
@@ -367,7 +386,7 @@ getsockaddrarg,PySocketSockObject *,s, PyObject *,args, struct sockaddr **,addr_
 		memcpy(addr->sun_path, path, len);
 		addr->sun_path[len] = 0;
 		*addr_ret = (struct sockaddr *) addr;
-		*len_ret = len + sizeof addr->sun_family;
+		*len_ret = len + sizeof(*addr) - sizeof(addr->sun_path);
 		return 1;
 	}
 #endif /* AF_UNIX */
@@ -465,27 +484,6 @@ BUILD_FUNC_DEF_2(PySocketSock_accept,PySocketSockObject *,s, PyObject *,args)
 }
 
 
-#if 0
-/* s.allowbroadcast() method */
-/* XXX obsolete -- will disappear in next release */
-
-static PyObject *
-BUILD_FUNC_DEF_2(PySocketSock_allowbroadcast,PySocketSockObject *,s, PyObject *,args)
-{
-	int flag;
-	int res;
-	if (!PyArg_Parse(args, "i", &flag))
-		return NULL;
-	res = setsockopt(s->sock_fd, SOL_SOCKET, SO_BROADCAST,
-			 (ANY *)&flag, sizeof flag);
-	if (res < 0)
-		return PySocket_Err();
-	Py_INCREF(Py_None);
-	return Py_None;
-}
-#endif
-
-
 /* s.setblocking(1 | 0) method */
 
 static PyObject *
@@ -496,7 +494,7 @@ BUILD_FUNC_DEF_2(PySocketSock_setblocking,PySocketSockObject*,s,PyObject*,args)
 	if (!PyArg_GetInt(args, &block))
 		return NULL;
 	Py_BEGIN_ALLOW_THREADS
-#ifndef NT
+#ifndef MS_WINDOWS
 	delay_flag = fcntl (s->sock_fd, F_GETFL, 0);
 	if (block)
 		delay_flag &= (~O_NDELAY);
@@ -558,10 +556,12 @@ BUILD_FUNC_DEF_2(PySocketSock_getsockopt,PySocketSockObject *,s, PyObject *,args
 	int optname;
 	int res;
 	PyObject *buf;
-	int buflen;
-	int flag;
+	int buflen = 0;
 
-	if (PyArg_Parse(args, "(ii)", &level, &optname)) {
+	if (!PyArg_ParseTuple(args, "ii|i", &level, &optname, &buflen))
+		return NULL;
+	
+	if (buflen == 0) {
 		int flag = 0;
 		int flagsize = sizeof flag;
 		res = getsockopt(s->sock_fd, level, optname,
@@ -570,9 +570,6 @@ BUILD_FUNC_DEF_2(PySocketSock_getsockopt,PySocketSockObject *,s, PyObject *,args
 			return PySocket_Err();
 		return PyInt_FromLong(flag);
 	}
-	PyErr_Clear();
-	if (!PyArg_Parse(args, "(iii)", &level, &optname, &buflen))
-		return NULL;
 	if (buflen <= 0 || buflen > 1024) {
 		PyErr_SetString(PySocket_Error, "getsockopt buflen out of range");
 		return NULL;
@@ -620,9 +617,11 @@ BUILD_FUNC_DEF_2(PySocketSock_close,PySocketSockObject *,s, PyObject *,args)
 {
 	if (!PyArg_NoArgs(args))
 		return NULL;
-	Py_BEGIN_ALLOW_THREADS
-	(void) close(s->sock_fd);
-	Py_END_ALLOW_THREADS
+	if (s->sock_fd != -1) {
+		Py_BEGIN_ALLOW_THREADS
+		(void) close(s->sock_fd);
+		Py_END_ALLOW_THREADS
+	}
 	s->sock_fd = -1;
 	Py_INCREF(Py_None);
 	return Py_None;
@@ -660,6 +659,30 @@ BUILD_FUNC_DEF_2(PySocketSock_fileno,PySocketSockObject *,s, PyObject *,args)
 }
 
 
+#ifndef NO_DUP
+/* s.dup() method */
+
+static PyObject *
+BUILD_FUNC_DEF_2(PySocketSock_dup,PySocketSockObject *,s, PyObject *,args)
+{
+	int newfd;
+	PyObject *sock;
+	if (!PyArg_NoArgs(args))
+		return NULL;
+	newfd = dup(s->sock_fd);
+	if (newfd < 0)
+		return PySocket_Err();
+	sock = (PyObject *) PySocketSock_New(newfd,
+					s->sock_family,
+					s->sock_type,
+					s->sock_proto);
+	if (sock == NULL)
+		close(newfd);
+	return sock;
+}
+#endif
+
+
 /* s.getsockname() method */
 
 static PyObject *
@@ -671,6 +694,7 @@ BUILD_FUNC_DEF_2(PySocketSock_getsockname,PySocketSockObject *,s, PyObject *,arg
 		return NULL;
 	if (!getsockaddrlen(s, &addrlen))
 		return NULL;
+	memset(addrbuf, 0, addrlen);
 	Py_BEGIN_ALLOW_THREADS
 	res = getsockname(s->sock_fd, (struct sockaddr *) addrbuf, &addrlen);
 	Py_END_ALLOW_THREADS
@@ -742,7 +766,7 @@ BUILD_FUNC_DEF_2(PySocketSock_makefile,PySocketSockObject *,s, PyObject *,args)
 
 	if (!PyArg_ParseTuple(args, "|si", &mode, &bufsize))
 		return NULL;
-#ifdef NT
+#ifdef MS_WIN32
    if ( ((fd =  _open_osfhandle( s->sock_fd, _O_BINARY )) < 0) ||
         ((fd = dup(fd)) < 0) || ((fp = fdopen(fd, mode)) == NULL)) {
 #else
@@ -765,14 +789,10 @@ BUILD_FUNC_DEF_2(PySocketSock_makefile,PySocketSockObject *,s, PyObject *,args)
 static PyObject *
 BUILD_FUNC_DEF_2(PySocketSock_recv,PySocketSockObject *,s, PyObject *,args)
 {
-	int len, n, flags;
+	int len, n, flags = 0;
 	PyObject *buf;
-	flags = 0;
-	if (!PyArg_Parse(args, "i", &len)) {
-		PyErr_Clear();
-		if (!PyArg_Parse(args, "(ii)", &len, &flags))
-			return NULL;
-	}
+	if (!PyArg_ParseTuple(args, "i|i", &len, &flags))
+		return NULL;
 	buf = PyString_FromStringAndSize((char *) 0, len);
 	if (buf == NULL)
 		return NULL;
@@ -783,7 +803,7 @@ BUILD_FUNC_DEF_2(PySocketSock_recv,PySocketSockObject *,s, PyObject *,args)
 		Py_DECREF(buf);
 		return PySocket_Err();
 	}
-	if (_PyString_Resize(&buf, n) < 0)
+	if (n != len && _PyString_Resize(&buf, n) < 0)
 		return NULL;
 	return buf;
 }
@@ -796,13 +816,9 @@ BUILD_FUNC_DEF_2(PySocketSock_recvfrom,PySocketSockObject *,s, PyObject *,args)
 {
 	char addrbuf[256];
 	PyObject *buf, *addr, *ret;
-	int addrlen, len, n, flags;
-	flags = 0;
-	if (!PyArg_Parse(args, "i", &len)) {
-		PyErr_Clear();
-		if (!PyArg_Parse(args, "(ii)", &len, &flags))
-		    return NULL;
-	}
+	int addrlen, len, n, flags = 0;
+	if (!PyArg_ParseTuple(args, "i|i", &len, &flags))
+		return NULL;
 	if (!getsockaddrlen(s, &addrlen))
 		return NULL;
 	buf = PyString_FromStringAndSize((char *) 0, len);
@@ -810,7 +826,7 @@ BUILD_FUNC_DEF_2(PySocketSock_recvfrom,PySocketSockObject *,s, PyObject *,args)
 		return NULL;
 	Py_BEGIN_ALLOW_THREADS
 	n = recvfrom(s->sock_fd, PyString_AsString(buf), len, flags,
-#ifndef NT
+#ifndef MS_WINDOWS
 		     (ANY *)addrbuf, &addrlen);
 #else
 		     (struct sockaddr *)addrbuf, &addrlen);
@@ -820,7 +836,7 @@ BUILD_FUNC_DEF_2(PySocketSock_recvfrom,PySocketSockObject *,s, PyObject *,args)
 		Py_DECREF(buf);
 		return PySocket_Err();
 	}
-	if (_PyString_Resize(&buf, n) < 0)
+	if (n != len && _PyString_Resize(&buf, n) < 0)
 		return NULL;
 	addr = makesockaddr((struct sockaddr *)addrbuf, addrlen);
 	ret = Py_BuildValue("OO", buf, addr);
@@ -836,13 +852,9 @@ static PyObject *
 BUILD_FUNC_DEF_2(PySocketSock_send,PySocketSockObject *,s, PyObject *,args)
 {
 	char *buf;
-	int len, n, flags;
-	flags = 0;
-	if (!PyArg_Parse(args, "s#", &buf, &len)) {
-		PyErr_Clear();
-		if (!PyArg_Parse(args, "(s#i)", &buf, &len, &flags))
-			return NULL;
-	}
+	int len, n, flags = 0;
+	if (!PyArg_ParseTuple(args, "s#|i", &buf, &len, &flags))
+		return NULL;
 	Py_BEGIN_ALLOW_THREADS
 	n = send(s->sock_fd, buf, len, flags);
 	Py_END_ALLOW_THREADS
@@ -901,16 +913,16 @@ BUILD_FUNC_DEF_2(PySocketSock_shutdown,PySocketSockObject *,s, PyObject *,args)
 
 static PyMethodDef PySocketSock_methods[] = {
 	{"accept",		(PyCFunction)PySocketSock_accept},
-#if 0
-	{"allowbroadcast",	(PyCFunction)PySocketSock_allowbroadcast},
-#endif
 	{"setblocking",		(PyCFunction)PySocketSock_setblocking},
 	{"setsockopt",		(PyCFunction)PySocketSock_setsockopt},
-	{"getsockopt",		(PyCFunction)PySocketSock_getsockopt},
+	{"getsockopt",		(PyCFunction)PySocketSock_getsockopt, 1},
 	{"bind",		(PyCFunction)PySocketSock_bind},
 	{"close",		(PyCFunction)PySocketSock_close},
 	{"connect",		(PyCFunction)PySocketSock_connect},
 	{"fileno",		(PyCFunction)PySocketSock_fileno},
+#ifndef NO_DUP
+	{"dup",			(PyCFunction)PySocketSock_dup},
+#endif
 	{"getsockname",		(PyCFunction)PySocketSock_getsockname},
 #ifdef HAVE_GETPEERNAME
 	{"getpeername",		(PyCFunction)PySocketSock_getpeername},
@@ -919,9 +931,9 @@ static PyMethodDef PySocketSock_methods[] = {
 #ifndef NO_DUP
 	{"makefile",		(PyCFunction)PySocketSock_makefile, 1},
 #endif
-	{"recv",		(PyCFunction)PySocketSock_recv},
-	{"recvfrom",		(PyCFunction)PySocketSock_recvfrom},
-	{"send",		(PyCFunction)PySocketSock_send},
+	{"recv",		(PyCFunction)PySocketSock_recv, 1},
+	{"recvfrom",		(PyCFunction)PySocketSock_recvfrom, 1},
+	{"send",		(PyCFunction)PySocketSock_send, 1},
 	{"sendto",		(PyCFunction)PySocketSock_sendto},
 	{"shutdown",		(PyCFunction)PySocketSock_shutdown},
 	{NULL,			NULL}		/* sentinel */
@@ -951,23 +963,18 @@ BUILD_FUNC_DEF_2(PySocketSock_getattr,PySocketSockObject *,s, char *,name)
 static PyObject *
 BUILD_FUNC_DEF_1(PySocketSock_repr,PySocketSockObject *,s)
 {
-	PyObject *addro;
-	struct sockaddr *addr;
 	char buf[512];
-	PyObject *t, *comma, *v;
-	int i, len;
 	sprintf(buf, 
 		"<socket object, fd=%d, family=%d, type=%d, protocol=%d>", 
 		s->sock_fd, s->sock_family, s->sock_type, s->sock_proto);
-	t = PyString_FromString(buf);
-	return t;
+	return PyString_FromString(buf);
 }
 
 
 /* Type object for socket objects. */
 
 static PyTypeObject PySocketSock_Type = {
-	PyObject_HEAD_INIT(&PyType_Type)
+	PyObject_HEAD_INIT(0)	/* Must fill in type value later */
 	0,
 	"socket",
 	sizeof(PySocketSockObject),
@@ -1028,8 +1035,7 @@ BUILD_FUNC_DEF_2(PySocket_gethostbyaddr,PyObject *,self, PyObject *, args)
         struct sockaddr_in addr;
 	char *ip_num;
 	struct hostent *h;
-	int d1,d2,d3,d4;
-	char ch, **pch;
+	char **pch;
 	PyObject *rtn_tuple = (PyObject *)NULL;
 	PyObject *name_list = (PyObject *)NULL;
 	PyObject *addr_list = (PyObject *)NULL;
@@ -1111,17 +1117,22 @@ static PyObject *
 BUILD_FUNC_DEF_2(PySocket_socket,PyObject *,self, PyObject *,args)
 {
 	PySocketSockObject *s;
-	int fd, family, type, proto;
-	proto = 0;
-	if (!PyArg_Parse(args, "(ii)", &family, &type)) {
-		PyErr_Clear();
-		if (!PyArg_Parse(args, "(iii)", &family, &type, &proto))
-			return NULL;
-	}
+#ifdef MS_WINDOWS
+	SOCKET fd;
+#else
+	int fd;
+#endif
+	int family, type, proto = 0;
+	if (!PyArg_ParseTuple(args, "ii|i", &family, &type, &proto))
+		return NULL;
 	Py_BEGIN_ALLOW_THREADS
 	fd = socket(family, type, proto);
 	Py_END_ALLOW_THREADS
+#ifdef MS_WINDOWS
+	if (fd == INVALID_SOCKET)
+#else
 	if (fd < 0)
+#endif
 		return PySocket_Err();
 	s = PySocketSock_New(fd, family, type, proto);
 	/* If the object can't be created, don't forget to close the
@@ -1146,13 +1157,9 @@ static PyObject *
 BUILD_FUNC_DEF_2(PySocket_fromfd,PyObject *,self, PyObject *,args)
 {
 	PySocketSockObject *s;
-	int fd, family, type, proto;
-	proto = 0;
-	if (!PyArg_Parse(args, "(iii)", &fd, &family, &type)) {
-		PyErr_Clear();
-		if (!PyArg_Parse(args, "(iiii)", &fd, &family, &type, &proto))
-			return NULL;
-	}
+	int fd, family, type, proto = 0;
+	if (!PyArg_ParseTuple(args, "iii|i", &fd, &family, &type, &proto))
+		return NULL;
 	/* Dup the fd so it and the socket can be closed independently */
 	fd = dup(fd);
 	if (fd < 0)
@@ -1167,6 +1174,54 @@ BUILD_FUNC_DEF_2(PySocket_fromfd,PyObject *,self, PyObject *,args)
 }
 #endif /* NO_DUP */
 
+static PyObject *
+BUILD_FUNC_DEF_2(PySocket_ntohs, PyObject *, self, PyObject *, args)
+{
+	int x1, x2;
+
+	if (!PyArg_Parse(args, "i", &x1)) {
+		return NULL;
+	}
+	x2 = (int)ntohs((short)x1);
+	return PyInt_FromLong(x2);
+}
+
+static PyObject *
+BUILD_FUNC_DEF_2(PySocket_ntohl, PyObject *, self, PyObject *, args)
+{
+	int x1, x2;
+
+	if (!PyArg_Parse(args, "i", &x1)) {
+		return NULL;
+	}
+	x2 = ntohl(x1);
+	return PyInt_FromLong(x2);
+}
+
+static PyObject *
+BUILD_FUNC_DEF_2(PySocket_htons, PyObject *, self, PyObject *, args)
+{
+	int x1, x2;
+
+	if (!PyArg_Parse(args, "i", &x1)) {
+		return NULL;
+	}
+	x2 = (int)htons((short)x1);
+	return PyInt_FromLong(x2);
+}
+
+static PyObject *
+BUILD_FUNC_DEF_2(PySocket_htonl, PyObject *, self, PyObject *, args)
+{
+	int x1, x2;
+
+	if (!PyArg_Parse(args, "i", &x1)) {
+		return NULL;
+	}
+	x2 = htonl(x1);
+	return PyInt_FromLong(x2);
+}
+
 /* List of functions exported by this module. */
 
 static PyMethodDef PySocket_methods[] = {
@@ -1174,10 +1229,14 @@ static PyMethodDef PySocket_methods[] = {
 	{"gethostbyaddr",	PySocket_gethostbyaddr},
 	{"gethostname",		PySocket_gethostname},
 	{"getservbyname",	PySocket_getservbyname},
-	{"socket",		PySocket_socket},
+	{"socket",		PySocket_socket, 1},
 #ifndef NO_DUP
-	{"fromfd",		PySocket_fromfd},
+	{"fromfd",		PySocket_fromfd, 1},
 #endif
+	{"ntohs",		PySocket_ntohs},
+	{"ntohl",		PySocket_ntohl},
+	{"htons",		PySocket_htons},
+	{"htonl",		PySocket_htonl},
 	{NULL,			NULL}		 /* Sentinel */
 };
 
@@ -1200,16 +1259,75 @@ BUILD_FUNC_DEF_3(insint,PyObject *,d, char *,name, int,value)
 }
 
 
+#ifdef MS_WINDOWS
+
+/* Additional initialization and cleanup for NT/Windows */
+
+static void
+NTcleanup()
+{
+	WSACleanup();
+}
+
+static int
+NTinit()
+{
+	WSADATA WSAData;
+	int ret;
+	char buf[100];
+	ret = WSAStartup(0x0101, &WSAData);
+	switch (ret) {
+	case 0:	/* no error */
+		atexit(NTcleanup);
+		return 1;
+	case WSASYSNOTREADY:
+		PyErr_SetString(PyExc_ImportError,
+				"WSAStartup failed: network not ready");
+		break;
+	case WSAVERNOTSUPPORTED:
+	case WSAEINVAL:
+		PyErr_SetString(PyExc_ImportError,
+		    "WSAStartup failed: requested version not supported");
+		break;
+	default:
+		sprintf(buf, "WSAStartup failed: error code %d", ret);
+		PyErr_SetString(PyExc_ImportError, buf);
+		break;
+	}
+	return 0;
+}
+
+#endif /* MS_WINDOWS */
+
+
 /* Initialize this module.
    This is called when the first 'import socket' is done,
    via a table in config.c, if config.c is compiled with USE_SOCKET
-   defined. */
+   defined.
+
+   For MS_WINDOWS (which means any Windows variant), this module
+   is actually called "_socket", and there's a wrapper "socket.py"
+   which implements some missing functionality (such as makefile(),
+   dup() and fromfd()).  The import of "_socket" may fail with an
+   ImportError exception if initialization of WINSOCK fails.  When
+   WINSOCK is initialized succesfully, a call to WSACleanup() is
+   scheduled to be made at exit time.  */
 
 void
+#ifdef MS_WINDOWS
+init_socket()
+#else
 initsocket()
+#endif
 {
 	PyObject *m, *d;
+#ifdef MS_WINDOWS
+	if (!NTinit())
+		return;
+	m = Py_InitModule("_socket", PySocket_methods);
+#else
 	m = Py_InitModule("socket", PySocket_methods);
+#endif
 	d = PyModule_GetDict(m);
 	PySocket_Error = PyString_FromString("socket.error");
 	if (PySocket_Error == NULL || 
@@ -1464,42 +1582,3 @@ initsocket()
 	insint(d, "IP_DROP_MEMBERSHIP", IP_DROP_MEMBERSHIP);
 #endif
 }
-
-#ifdef NT
-BOOL	WINAPI	DllMain (HANDLE hInst, 
-			ULONG ul_reason_for_call,
-			LPVOID lpReserved)
-{
-	const int opt = SO_SYNCHRONOUS_NONALERT;
-	switch (ul_reason_for_call)
-	{
-		case DLL_PROCESS_ATTACH: {
-			WSADATA WSAData;
-			BOOL ok = TRUE;
-			char buf[100] = "Python can't initialize Windows Sockets Module!\n\r";
-			if (WSAStartup(MAKEWORD(1,1), &WSAData)) {
-				wsprintf(buf+strlen(buf), "WSAStartup failed (%d)",WSAGetLastError());
-				ok = FALSE;
-			}
-			/*
-			** Setup sockets in non-overlapped mode by default
-			*/
-			if (ok && setsockopt(INVALID_SOCKET,SOL_SOCKET,SO_OPENTYPE,(const char *)&opt,sizeof(opt)) != 0) {
-				wsprintf(buf+strlen(buf),"setsockopt failed (%d)",WSAGetLastError());
-				ok = FALSE;
-			}
-			if (!ok) {
-				MessageBox(NULL,buf,"WinSock Error",MB_OK|MB_SETFOREGROUND);
-				return FALSE;
-			}
-			break;
-		}
-
-		case DLL_PROCESS_DETACH:
-			WSACleanup();
-			break;
-
-	}
-	return TRUE;
-}
-#endif /* NT */
