@@ -191,9 +191,21 @@ initsyslog()
 	DICT_SET_INT(d, "LOG_DAEMON",	LOG_DAEMON);
 	DICT_SET_INT(d, "LOG_AUTH",	LOG_AUTH);
 	DICT_SET_INT(d, "LOG_LPR",	LOG_LPR);
+#ifdef LOG_NEWS
 	DICT_SET_INT(d, "LOG_NEWS",	LOG_NEWS);
+#else
+	DICT_SET_INT(d, "LOG_NEWS",	LOG_MAIL);
+#endif
+#ifdef LOG_UUCP
 	DICT_SET_INT(d, "LOG_UUCP",	LOG_UUCP);
+#else
+	DICT_SET_INT(d, "LOG_UUCP",	LOG_MAIL);
+#endif
+#ifdef LOG_CRON
 	DICT_SET_INT(d, "LOG_CRON",	LOG_CRON);
+#else
+	DICT_SET_INT(d, "LOG_CRON",	LOG_DAEMON);
+#endif
 	DICT_SET_INT(d, "LOG_LOCAL0",	LOG_LOCAL0);
 	DICT_SET_INT(d, "LOG_LOCAL1",	LOG_LOCAL1);
 	DICT_SET_INT(d, "LOG_LOCAL2",	LOG_LOCAL2);
