@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -28,8 +35,8 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "structmember.h"
 
 /* Forward */
-static object *class_lookup PROTO((classobject *, char *, classobject **));
-static object *instance_getattr1 PROTO((instanceobject *, char *));
+static object *class_lookup PROTO((classobject *, object *, classobject **));
+static object *instance_getattr1 PROTO((instanceobject *, object *));
 
 object *
 newclassobject(bases, dict, name)
@@ -37,11 +44,20 @@ newclassobject(bases, dict, name)
 	object *dict;
 	object *name; /* String; NULL if unknown */
 {
+#ifdef SUPPORT_OBSOLETE_ACCESS
 	int pos;
 	object *key, *value;
+#endif
 	classobject *op, *dummy;
-	if (dictlookup(dict, "__doc__") == NULL) {
-		if (dictinsert(dict, "__doc__", None) < 0)
+	static object *getattrstr, *setattrstr, *delattrstr;
+	static object *docstr;
+	if (docstr == NULL) {
+		docstr= newstringobject("__doc__");
+		if (docstr == NULL)
+			return NULL;
+	}
+	if (mappinglookup(dict, docstr) == NULL) {
+		if (mappinginsert(dict, docstr, None) < 0)
 			return NULL;
 	}
 	if (bases == NULL) {
@@ -61,17 +77,24 @@ newclassobject(bases, dict, name)
 	op->cl_dict = dict;
 	XINCREF(name);
 	op->cl_name = name;
-	op->cl_getattr = class_lookup(op, "__getattr__", &dummy);
-	op->cl_setattr = class_lookup(op, "__setattr__", &dummy);
-	op->cl_delattr = class_lookup(op, "__delattr__", &dummy);
+	if (getattrstr == NULL) {
+		getattrstr = newstringobject("__getattr__");
+		setattrstr = newstringobject("__setattr__");
+		delattrstr = newstringobject("__delattr__");
+	}
+	op->cl_getattr = class_lookup(op, getattrstr, &dummy);
+	op->cl_setattr = class_lookup(op, setattrstr, &dummy);
+	op->cl_delattr = class_lookup(op, delattrstr, &dummy);
 	XINCREF(op->cl_getattr);
 	XINCREF(op->cl_setattr);
 	XINCREF(op->cl_delattr);
+#ifdef SUPPORT_OBSOLETE_ACCESS
 	pos = 0;
 	while (mappinggetnext(dict, &pos, &key, &value)) {
 		if (is_accessobject(value))
 			setaccessowner(value, (object *)op);
 	}
+#endif
 	return (object *) op;
 }
 
@@ -90,11 +113,11 @@ class_dealloc(op)
 static object *
 class_lookup(cp, name, pclass)
 	classobject *cp;
-	char *name;
+	object *name;
 	classobject **pclass;
 {
 	int i, n;
-	object *value = dictlookup(cp->cl_dict, name);
+	object *value = mappinglookup(cp->cl_dict, name);
 	if (value != NULL) {
 		*pclass = cp;
 		return value;
@@ -112,12 +135,13 @@ class_lookup(cp, name, pclass)
 static object *
 class_getattr(op, name)
 	register classobject *op;
-	register char *name;
+	object *name;
 {
 	register object *v;
+	register char *sname = getstringvalue(name);
 	classobject *class;
-	if (name[0] == '_' && name[1] == '_') {
-		if (strcmp(name, "__dict__") == 0) {
+	if (sname[0] == '_' && sname[1] == '_') {
+		if (strcmp(sname, "__dict__") == 0) {
 			if (getrestricted()) {
 				err_setstr(RuntimeError,
 					   "class.__dict__ not accessible in restricted mode");
@@ -126,11 +150,11 @@ class_getattr(op, name)
 			INCREF(op->cl_dict);
 			return op->cl_dict;
 		}
-		if (strcmp(name, "__bases__") == 0) {
+		if (strcmp(sname, "__bases__") == 0) {
 			INCREF(op->cl_bases);
 			return op->cl_bases;
 		}
-		if (strcmp(name, "__name__") == 0) {
+		if (strcmp(sname, "__name__") == 0) {
 			if (op->cl_name == NULL)
 				v = None;
 			else
@@ -141,15 +165,17 @@ class_getattr(op, name)
 	}
 	v = class_lookup(op, name, &class);
 	if (v == NULL) {
-		err_setstr(AttributeError, name);
+		err_setval(AttributeError, name);
 		return NULL;
 	}
+#ifdef SUPPORT_OBSOLETE_ACCESS
 	if (is_accessobject(v)) {
 		v = getaccessvalue(v, getowner());
 		if (v == NULL)
 			return NULL;
 	}
 	else
+#endif
 		INCREF(v);
 	if (is_funcobject(v)) {
 		object *w = newinstancemethodobject(v, (object *)NULL,
@@ -163,13 +189,16 @@ class_getattr(op, name)
 static int
 class_setattr(op, name, v)
 	classobject *op;
-	char *name;
+	object *name;
 	object *v;
 {
+#ifdef SUPPORT_OBSOLETE_ACCESS
 	object *ac;
-	if (name[0] == '_' && name[1] == '_') {
-		int n = strlen(name);
-		if (name[n-1] == '_' && name[n-2] == '_') {
+#endif
+	char *sname = getstringvalue(name);
+	if (sname[0] == '_' && sname[1] == '_') {
+		int n = getstringsize(name);
+		if (sname[n-1] == '_' && sname[n-2] == '_') {
 			err_setstr(TypeError, "read-only special attribute");
 			return -1;
 		}
@@ -179,18 +208,20 @@ class_setattr(op, name, v)
 			   "classes are read-only in restricted mode");
 		return -1;
 	}
-	ac = dictlookup(op->cl_dict, name);
+#ifdef SUPPORT_OBSOLETE_ACCESS
+	ac = mappinglookup(op->cl_dict, name);
 	if (ac != NULL && is_accessobject(ac))
 		return setaccessvalue(ac, getowner(), v);
+#endif
 	if (v == NULL) {
-		int rv = dictremove(op->cl_dict, name);
+		int rv = mappingremove(op->cl_dict, name);
 		if (rv < 0)
 			err_setstr(AttributeError,
 				   "delete non-existing class attribute");
 		return rv;
 	}
 	else
-		return dictinsert(op->cl_dict, name, v);
+		return mappinginsert(op->cl_dict, name, v);
 }
 
 static object *
@@ -215,13 +246,18 @@ typeobject Classtype = {
 	0,
 	(destructor)class_dealloc, /*tp_dealloc*/
 	0,		/*tp_print*/
-	(getattrfunc)class_getattr, /*tp_getattr*/
-	(setattrfunc)class_setattr, /*tp_setattr*/
+	0,		/*tp_getattr*/
+	0,		/*tp_setattr*/
 	0,		/*tp_compare*/
 	(reprfunc)class_repr, /*tp_repr*/
 	0,		/*tp_as_number*/
 	0,		/*tp_as_sequence*/
 	0,		/*tp_as_mapping*/
+	0,		/*tp_hash*/
+	0,		/*tp_call*/
+	0,		/*tp_str*/
+	(getattrofunc)class_getattr, /*tp_getattro*/
+	(setattrofunc)class_setattr, /*tp_setattro*/
 };
 
 int
@@ -247,6 +283,7 @@ issubclass(class, base)
 
 /* Instance objects */
 
+#ifdef SUPPORT_OBSOLETE_ACCESS
 static int
 addaccess(class, inst)
 	classobject *class;
@@ -282,6 +319,7 @@ addaccess(class, inst)
     	}
 	return 0;
 }
+#endif
 
 object *
 newinstanceobject(class, arg, kw)
@@ -291,6 +329,7 @@ newinstanceobject(class, arg, kw)
 {
 	register instanceobject *inst;
 	object *init;
+	static object *initstr;
 	if (!is_classobject(class)) {
 		err_badcall();
 		return NULL;
@@ -301,16 +340,23 @@ newinstanceobject(class, arg, kw)
 	INCREF(class);
 	inst->in_class = (classobject *)class;
 	inst->in_dict = newdictobject();
-	if (inst->in_dict == NULL ||
-	    addaccess((classobject *)class, inst) != 0) {
+	if (inst->in_dict == NULL
+#ifdef SUPPORT_OBSOLETE_ACCESS
+	    || addaccess((classobject *)class, inst) != 0
+#endif
+		) {
 		DECREF(inst);
 		return NULL;
 	}
-	init = instance_getattr1(inst, "__init__");
+	if (initstr == NULL)
+		initstr = newstringobject("__init__");
+	init = instance_getattr1(inst, initstr);
 	if (init == NULL) {
 		err_clear();
 		if (arg != NULL && (!is_tupleobject(arg) ||
-				    gettuplesize(arg) != 0) || kw != NULL) {
+				    gettuplesize(arg) != 0)
+		    || kw != NULL && (!is_dictobject(kw) ||
+				      getdictsize(kw) != 0)) {
 			err_setstr(TypeError,
 				   "this constructor takes no arguments");
 			DECREF(inst);
@@ -345,10 +391,11 @@ instance_dealloc(inst)
 {
 	object *error_type, *error_value, *error_traceback;
 	object *del;
+	static object *delstr;
 	/* Call the __del__ method if it exists.  First temporarily
 	   revive the object and save the current exception, if any. */
-#ifdef TRACE_REFS
-	/* much too complicated if TRACE_REFS defined */
+#ifdef Py_TRACE_REFS
+	/* much too complicated if Py_TRACE_REFS defined */
 	extern long ref_total;
 	inst->ob_type = &Instancetype;
 	NEWREF(inst);
@@ -356,15 +403,27 @@ instance_dealloc(inst)
 #ifdef COUNT_ALLOCS
 	inst->ob_type->tp_alloc--; /* ditto */
 #endif
-#else
+#else /* !Py_TRACE_REFS */
 	INCREF(inst);
-#endif /* TRACE_REFS */
+#endif /* !Py_TRACE_REFS */
 	err_fetch(&error_type, &error_value, &error_traceback);
-	if ((del = instance_getattr1(inst, "__del__")) != NULL) {
+	if (delstr == NULL)
+		delstr = newstringobject("__del__");
+	if ((del = instance_getattr1(inst, delstr)) != NULL) {
 		object *res = call_object(del, (object *)NULL);
 		DECREF(del);
-		XDECREF(res);
-		/* XXX If __del__ raised an exception, it is ignored! */
+		if (res == NULL) {
+			PyObject *f = sysget("stderr");
+			err_clear();
+			if (f != NULL) {
+				writestring("exception in ", f);
+				writestring(PyString_AsString(
+					inst->in_class->cl_name), f);
+				writestring(".__del__() ignored\n", f);
+			}
+		}
+		else
+			DECREF(res);
 	}
 	/* Restore the saved exception and undo the temporary revival */
 	err_restore(error_type, error_value, error_traceback);
@@ -375,13 +434,13 @@ instance_dealloc(inst)
 #endif
 		return; /* __del__ added a reference; don't delete now */
 	}
-#ifdef TRACE_REFS
+#ifdef Py_TRACE_REFS
 #ifdef COUNT_ALLOCS
 	inst->ob_type->tp_free--;	/* compensate for increment in UNREF */
 #endif
 	UNREF(inst);
 	inst->ob_type = NULL;
-#endif /* TRACE_REFS */
+#endif /* Py_TRACE_REFS */
 	DECREF(inst->in_class);
 	XDECREF(inst->in_dict);
 	free((ANY *)inst);
@@ -390,12 +449,13 @@ instance_dealloc(inst)
 static object *
 instance_getattr1(inst, name)
 	register instanceobject *inst;
-	register char *name;
+	object *name;
 {
 	register object *v;
+	register char *sname = getstringvalue(name);
 	classobject *class;
-	if (name[0] == '_' && name[1] == '_') {
-		if (strcmp(name, "__dict__") == 0) {
+	if (sname[0] == '_' && sname[1] == '_') {
+		if (strcmp(sname, "__dict__") == 0) {
 			if (getrestricted()) {
 				err_setstr(RuntimeError,
 					   "instance.__dict__ not accessible in restricted mode");
@@ -404,26 +464,28 @@ instance_getattr1(inst, name)
 			INCREF(inst->in_dict);
 			return inst->in_dict;
 		}
-		if (strcmp(name, "__class__") == 0) {
+		if (strcmp(sname, "__class__") == 0) {
 			INCREF(inst->in_class);
 			return (object *)inst->in_class;
 		}
 	}
 	class = NULL;
-	v = dictlookup(inst->in_dict, name);
+	v = mappinglookup(inst->in_dict, name);
 	if (v == NULL) {
 		v = class_lookup(inst->in_class, name, &class);
 		if (v == NULL) {
-			err_setstr(AttributeError, name);
+			err_setval(AttributeError, name);
 			return NULL;
 		}
 	}
+#ifdef SUPPORT_OBSOLETE_ACCESS
 	if (is_accessobject(v)) {
 		v = getaccessvalue(v, getowner());
 		if (v == NULL)
 			return NULL;
 	}
 	else
+#endif
 		INCREF(v);
 	if (class != NULL) {
 		if (is_funcobject(v)) {
@@ -450,14 +512,14 @@ instance_getattr1(inst, name)
 static object *
 instance_getattr(inst, name)
 	register instanceobject *inst;
-	register char *name;
+	object *name;
 {
 	register object *func, *res;
 	res = instance_getattr1(inst, name);
 	if (res == NULL && (func = inst->in_class->cl_getattr) != NULL) {
 		object *args;
 		err_clear();
-		args = mkvalue("(Os)", inst, name);
+		args = mkvalue("(OO)", inst, name);
 		if (args == NULL)
 			return NULL;
 		res = call_object(func, args);
@@ -469,34 +531,39 @@ instance_getattr(inst, name)
 static int
 instance_setattr1(inst, name, v)
 	instanceobject *inst;
-	char *name;
+	object *name;
 	object *v;
 {
+#ifdef SUPPORT_OBSOLETE_ACCESS
 	object *ac;
-	ac = dictlookup(inst->in_dict, name);
+	ac = mappinglookup(inst->in_dict, name);
 	if (ac != NULL && is_accessobject(ac))
 		return setaccessvalue(ac, getowner(), v);
+#endif
 	if (v == NULL) {
-		int rv = dictremove(inst->in_dict, name);
+		int rv = mappingremove(inst->in_dict, name);
 		if (rv < 0)
 			err_setstr(AttributeError,
 				   "delete non-existing instance attribute");
 		return rv;
 	}
 	else
-		return dictinsert(inst->in_dict, name, v);
+		return mappinginsert(inst->in_dict, name, v);
 }
 
 static int
 instance_setattr(inst, name, v)
 	instanceobject *inst;
-	char *name;
+	object *name;
 	object *v;
 {
 	object *func, *args, *res;
-	if (name[0] == '_' && name[1] == '_') {
-		int n = strlen(name);
-		if (name[n-1] == '_' && name[n-2] == '_') {
+	char *sname = getstringvalue(name);
+	if (sname[0] == '_' && sname[1] == '_'
+	    && (strcmp(sname, "__dict__") == 0 ||
+		strcmp(sname, "__class__") == 0)) {
+	        int n = getstringsize(name);
+		if (sname[n-1] == '_' && sname[n-2] == '_') {
 			err_setstr(TypeError, "read-only special attribute");
 			return -1;
 		}
@@ -508,9 +575,9 @@ instance_setattr(inst, name, v)
 	if (func == NULL)
 		return instance_setattr1(inst, name, v);
 	if (v == NULL)
-		args = mkvalue("(Os)", inst, name);
+		args = mkvalue("(OO)", inst, name);
 	else
-		args = mkvalue("(OsO)", inst, name, v);
+		args = mkvalue("(OOO)", inst, name, v);
 	if (args == NULL)
 		return -1;
 	res = call_object(func, args);
@@ -527,8 +594,11 @@ instance_repr(inst)
 {
 	object *func;
 	object *res;
+	static object *reprstr;
 
-	func = instance_getattr(inst, "__repr__");
+	if (reprstr == NULL)
+		reprstr = newstringobject("__repr__");
+	func = instance_getattr(inst, reprstr);
 	if (func == NULL) {
 		char buf[140];
 		object *classname = inst->in_class->cl_name;
@@ -582,13 +652,18 @@ instance_hash(inst)
 	object *func;
 	object *res;
 	long outcome;
+	static object *hashstr, *cmpstr;
 
-	func = instance_getattr(inst, "__hash__");
+	if (hashstr == NULL)
+		hashstr = newstringobject("__hash__");
+	func = instance_getattr(inst, hashstr);
 	if (func == NULL) {
 		/* If there is no __cmp__ method, we hash on the address.
 		   If a __cmp__ method exists, there must be a __hash__. */
 		err_clear();
-		func = instance_getattr(inst, "__cmp__");
+		if (cmpstr == NULL)
+			cmpstr = newstringobject("__cmp__");
+		func = instance_getattr(inst, cmpstr);
 		if (func == NULL) {
 			err_clear();
 			outcome = (long)inst;
@@ -616,6 +691,8 @@ instance_hash(inst)
 	return outcome;
 }
 
+static object *getitemstr, *setitemstr, *delitemstr, *lenstr;
+
 static int
 instance_length(inst)
 	instanceobject *inst;
@@ -624,7 +701,9 @@ instance_length(inst)
 	object *res;
 	int outcome;
 
-	func = instance_getattr(inst, "__len__");
+	if (lenstr == NULL)
+		lenstr = newstringobject("__len__");
+	func = instance_getattr(inst, lenstr);
 	if (func == NULL)
 		return -1;
 	res = call_object(func, (object *)NULL);
@@ -653,7 +732,9 @@ instance_subscript(inst, key)
 	object *arg;
 	object *res;
 
-	func = instance_getattr(inst, "__getitem__");
+	if (getitemstr == NULL)
+		getitemstr = newstringobject("__getitem__");
+	func = instance_getattr(inst, getitemstr);
 	if (func == NULL)
 		return NULL;
 	arg = mkvalue("(O)", key);
@@ -677,10 +758,16 @@ instance_ass_subscript(inst, key, value)
 	object *arg;
 	object *res;
 
-	if (value == NULL)
-		func = instance_getattr(inst, "__delitem__");
-	else
-		func = instance_getattr(inst, "__setitem__");
+	if (value == NULL) {
+		if (delitemstr == NULL)
+			delitemstr = newstringobject("__delitem__");
+		func = instance_getattr(inst, delitemstr);
+	}
+	else {
+		if (setitemstr == NULL)
+			setitemstr = newstringobject("__setitem__");
+		func = instance_getattr(inst, setitemstr);
+	}
 	if (func == NULL)
 		return -1;
 	if (value == NULL)
@@ -713,7 +800,9 @@ instance_item(inst, i)
 {
 	object *func, *arg, *res;
 
-	func = instance_getattr(inst, "__getitem__");
+	if (getitemstr == NULL)
+		getitemstr = newstringobject("__getitem__");
+	func = instance_getattr(inst, getitemstr);
 	if (func == NULL)
 		return NULL;
 	arg = mkvalue("(i)", i);
@@ -733,8 +822,11 @@ instance_slice(inst, i, j)
 	int i, j;
 {
 	object *func, *arg, *res;
+	static object *getslicestr;
 
-	func = instance_getattr(inst, "__getslice__");
+	if (getslicestr == NULL)
+		getslicestr = newstringobject("__getslice__");
+	func = instance_getattr(inst, getslicestr);
 	if (func == NULL)
 		return NULL;
 	arg = mkvalue("(ii)", i, j);
@@ -756,10 +848,16 @@ instance_ass_item(inst, i, item)
 {
 	object *func, *arg, *res;
 
-	if (item == NULL)
-		func = instance_getattr(inst, "__delitem__");
-	else
-		func = instance_getattr(inst, "__setitem__");
+	if (item == NULL) {
+		if (delitemstr == NULL)
+			delitemstr = newstringobject("__delitem__");
+		func = instance_getattr(inst, delitemstr);
+	}
+	else {
+		if (setitemstr == NULL)
+			setitemstr = newstringobject("__setitem__");
+		func = instance_getattr(inst, setitemstr);
+	}
 	if (func == NULL)
 		return -1;
 	if (item == NULL)
@@ -786,11 +884,18 @@ instance_ass_slice(inst, i, j, value)
 	object *value;
 {
 	object *func, *arg, *res;
+	static object *setslicestr, *delslicestr;
 
-	if (value == NULL)
-		func = instance_getattr(inst, "__delslice__");
-	else
-		func = instance_getattr(inst, "__setslice__");
+	if (value == NULL) {
+		if (delslicestr == NULL)
+			delslicestr = newstringobject("__delslice__");
+		func = instance_getattr(inst, delslicestr);
+	}
+	else {
+		if (setslicestr == NULL)
+			setslicestr = newstringobject("__setslice__");
+		func = instance_getattr(inst, setslicestr);
+	}
 	if (func == NULL)
 		return -1;
 	if (value == NULL)
@@ -823,7 +928,7 @@ static sequence_methods instance_as_sequence = {
 static object *
 generic_unary_op(self, methodname)
 	instanceobject *self;
-	char *methodname;
+	object *methodname;
 {
 	object *func, *res;
 
@@ -836,8 +941,8 @@ generic_unary_op(self, methodname)
 
 
 /* Forward */
-static int halfbinop PROTO((object *, object *, char *, object **,
-			    object * (*) PROTO((object *, object *)), int ));
+static int halfbinop Py_PROTO((object *, object *, char *, object **,
+			  object * (*) Py_PROTO((object *, object *)), int ));
 
 
 /* Implement a binary operator involving at least one class instance. */
@@ -869,6 +974,8 @@ instancebinop(v, w, opname, ropname, thisfunc)
    1  if we could try another operation
 */
 
+static object *coerce_obj;
+
 static int
 halfbinop(v, w, opname, r_result, thisfunc, swapped)
 	object *v;
@@ -886,7 +993,12 @@ halfbinop(v, w, opname, r_result, thisfunc, swapped)
 	
 	if (!is_instanceobject(v))
 		return 1;
-	coerce = getattr(v, "__coerce__");
+	if (coerce_obj == NULL) {
+		coerce_obj = newstringobject("__coerce__");
+		if (coerce_obj == NULL)
+			return -1;
+	}
+	coerce = getattro(v, coerce_obj);
 	if (coerce == NULL) {
 		err_clear();
 	}
@@ -958,7 +1070,12 @@ instance_coerce(pv, pw)
 	object *args;
 	object *coerced;
 
-	coerce = getattr(v, "__coerce__");
+	if (coerce_obj == NULL) {
+		coerce_obj = newstringobject("__coerce__");
+		if (coerce_obj == NULL)
+			return -1;
+	}
+	coerce = getattro(v, coerce_obj);
 	if (coerce == NULL) {
 		/* No __coerce__ method: always OK */
 		err_clear();
@@ -1002,7 +1119,9 @@ instance_coerce(pv, pw)
 
 #define UNARY(funcname, methodname) \
 static object *funcname(self) instanceobject *self; { \
-	return generic_unary_op(self, methodname); \
+	static object *o; \
+	if (o == NULL) o = newstringobject(methodname); \
+	return generic_unary_op(self, o); \
 }
 
 UNARY(instance_neg, "__neg__")
@@ -1015,10 +1134,15 @@ instance_nonzero(self)
 {
 	object *func, *res;
 	long outcome;
+	static object *nonzerostr;
 
-	if ((func = instance_getattr(self, "__nonzero__")) == NULL) {
+	if (nonzerostr == NULL)
+		nonzerostr = newstringobject("__nonzero__");
+	if ((func = instance_getattr(self, nonzerostr)) == NULL) {
 		err_clear();
-		if ((func = instance_getattr(self, "__len__")) == NULL) {
+		if (lenstr == NULL)
+			lenstr = newstringobject("__len__");
+		if ((func = instance_getattr(self, lenstr)) == NULL) {
 			err_clear();
 			/* Fall back to the default behavior:
 			   all instances are nonzero */
@@ -1061,7 +1185,11 @@ instance_pow(v, w, z)
 	object *func;
 	object *args;
 	object *result;
-	func = getattr(v, "__pow__");
+	static object *powstr;
+
+	if (powstr == NULL)
+		powstr = newstringobject("__pow__");
+	func = getattro(v, powstr);
 	if (func == NULL)
 		return NULL;
 	args = mkvalue("(OO)", w, z);
@@ -1109,14 +1237,18 @@ typeobject Instancetype = {
 	0,
 	(destructor)instance_dealloc, /*tp_dealloc*/
 	0,			/*tp_print*/
-	(getattrfunc)instance_getattr, /*tp_getattr*/
-	(setattrfunc)instance_setattr, /*tp_setattr*/
-	instance_compare, /*tp_compare*/
+	0,			/*tp_getattr*/
+	0,			/*tp_setattr*/
+	instance_compare,	/*tp_compare*/
 	(reprfunc)instance_repr, /*tp_repr*/
 	&instance_as_number,	/*tp_as_number*/
 	&instance_as_sequence,	/*tp_as_sequence*/
 	&instance_as_mapping,	/*tp_as_mapping*/
 	(hashfunc)instance_hash, /*tp_hash*/
+	0,			/*tp_call*/
+	0,			/*tp_str*/
+	(getattrofunc)instance_getattr, /*tp_getattro*/
+	(setattrofunc)instance_setattr, /*tp_setattro*/
 };
 
 
@@ -1131,7 +1263,6 @@ typedef struct {
 	object	*im_func;	/* The function implementing the method */
 	object	*im_self;	/* The instance it is bound to, or NULL */
 	object	*im_class;	/* The class that defined the method */
-	object	*im_doc;	/* The documentation string */
 } instancemethodobject;
 
 object *
@@ -1154,8 +1285,6 @@ newinstancemethodobject(func, self, class)
 	im->im_self = self;
 	INCREF(class);
 	im->im_class = class;
-	XINCREF(((funcobject *)func)->func_doc);
-	im->im_doc = ((funcobject *)func)->func_doc;
 	return (object *)im;
 }
 
@@ -1200,22 +1329,35 @@ static struct memberlist instancemethod_memberlist[] = {
 	{"im_func",	T_OBJECT,	OFF(im_func)},
 	{"im_self",	T_OBJECT,	OFF(im_self)},
 	{"im_class",	T_OBJECT,	OFF(im_class)},
-	{"im_doc",	T_OBJECT,	OFF(im_doc)},
-	{"__doc__",	T_OBJECT,	OFF(im_doc)},
+	/* Dummies that are not handled by getattr() except for __members__ */
+	{"__doc__",	T_INT,		0},
+	{"__name__",	T_INT,		0},
 	{NULL}	/* Sentinel */
 };
 
 static object *
 instancemethod_getattr(im, name)
 	register instancemethodobject *im;
-	char *name;
+	object *name;
 {
-	if (name[0] != '_' && getrestricted()) {
+	char *sname = getstringvalue(name);
+	if (sname[0] == '_') {
+		funcobject *func = (funcobject *)(im->im_func);
+		if (strcmp(sname, "__name__") == 0) {
+			INCREF(func->func_name);
+			return func->func_name;
+		}
+		if (strcmp(sname, "__doc__") == 0) {
+			INCREF(func->func_doc);
+			return func->func_doc;
+		}
+	}
+	if (getrestricted()) {
 		err_setstr(RuntimeError,
 			   "instance-method attributes not accessible in restricted mode");
 		return NULL;
 	}
-	return getmember((char *)im, instancemethod_memberlist, name);
+	return getmember((char *)im, instancemethod_memberlist, sname);
 }
 
 static void
@@ -1225,7 +1367,6 @@ instancemethod_dealloc(im)
 	DECREF(im->im_func);
 	XDECREF(im->im_self);
 	DECREF(im->im_class);
-	XDECREF(im->im_doc);
 	free((ANY *)im);
 }
 
@@ -1297,7 +1438,7 @@ typeobject Instancemethodtype = {
 	0,
 	(destructor)instancemethod_dealloc, /*tp_dealloc*/
 	0,			/*tp_print*/
-	(getattrfunc)instancemethod_getattr, /*tp_getattr*/
+	0,			/*tp_getattr*/
 	0,			/*tp_setattr*/
 	(cmpfunc)instancemethod_compare, /*tp_compare*/
 	(reprfunc)instancemethod_repr, /*tp_repr*/
@@ -1305,4 +1446,8 @@ typeobject Instancemethodtype = {
 	0,			/*tp_as_sequence*/
 	0,			/*tp_as_mapping*/
 	(hashfunc)instancemethod_hash, /*tp_hash*/
+	0,			/*tp_call*/
+	0,			/*tp_str*/
+	(getattrofunc)instancemethod_getattr, /*tp_getattro*/
+	0,			/*tp_setattro*/
 };
