@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -394,7 +401,7 @@ string_compare(a, b)
 	int min_len = (len_a < len_b) ? len_a : len_b;
 	int cmp;
 	if (min_len > 0) {
-		cmp = *a->ob_sval - *b->ob_sval;
+		cmp = Py_CHARMASK(*a->ob_sval) - Py_CHARMASK(*b->ob_sval);
 		if (cmp == 0)
 			cmp = memcmp(a->ob_sval, b->ob_sval, min_len);
 		if (cmp != 0)
@@ -419,7 +426,7 @@ string_hash(a)
 	p = (unsigned char *) a->ob_sval;
 	x = *p << 7;
 	while (--len >= 0)
-		x = (3*x) ^ *p++;
+		x = (1000003*x) ^ *p++;
 	x ^= a->ob_size;
 	if (x == -1)
 		x = -2;
@@ -507,7 +514,7 @@ resizestring(pv, newsize)
 		return -1;
 	}
 	/* XXX UNREF/NEWREF interface should be more symmetrical */
-#ifdef REF_DEBUG
+#ifdef Py_REF_DEBUG
 	--_Py_RefTotal;
 #endif
 	UNREF(v);
@@ -613,26 +620,6 @@ formatchar(v)
 	return buf;
 }
 
-/* XXX this could be moved to object.c */
-static object *
-get_mapping_item(mo, ko)
-	object *mo;
-	object *ko;
-{
-	mapping_methods *mm = mo->ob_type->tp_as_mapping;
-	object *val;
-
-	if (!mm || !mm->mp_subscript) {
-		err_setstr(TypeError, "subscript not implemented");
-		return NULL;
-	}
-
-	val = (*mm->mp_subscript)(mo, ko);
-	XDECREF(val);		/* still in mapping */
-
-	return val;
-}
-
 
 /* fmt%(v1,v2,...) is roughly equivalent to sprintf(fmt, v1, v2, ...) */
 
@@ -643,6 +630,7 @@ formatstring(format, args)
 {
 	char *fmt, *res;
 	int fmtcnt, rescnt, reslen, arglen, argidx;
+	int args_owned = 0;
 	object *result;
 	object *dict = NULL;
 	if (format == NULL || !is_stringobject(format) || args == NULL) {
@@ -717,11 +705,16 @@ formatstring(format, args)
 				key = newsizedstringobject(keystart, keylen);
 				if (key == NULL)
 					goto error;
-				args = get_mapping_item(dict, key);
+				if (args_owned) {
+					DECREF(args);
+					args_owned = 0;
+				}
+				args = PyObject_GetItem(dict, key);
 				DECREF(key);
 				if (args == NULL) {
 					goto error;
 				}
+				args_owned = 1;
 				arglen = -1;
 				argidx = -2;
 			}
@@ -925,9 +918,13 @@ formatstring(format, args)
 		err_setstr(TypeError, "not all arguments converted");
 		goto error;
 	}
+	if (args_owned)
+		DECREF(args);
 	resizestring(&result, reslen - rescnt);
 	return result;
  error:
 	DECREF(result);
+	if (args_owned)
+		DECREF(args);
 	return NULL;
 }
