@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -28,15 +35,13 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 #include "node.h"
 #include "graminit.h"
-#include "sysmodule.h"
 #include "bltinmodule.h"
 #include "import.h"
-#include "pythonrun.h"
-#include "ceval.h"
-#include "modsupport.h"
 #include "compile.h"
 #include "eval.h"
 
+#include "mymath.h"
+
 /* Forward */
 static object *filterstring PROTO((object *, object *));
 static object *filtertuple  PROTO((object *, object *));
@@ -80,7 +85,7 @@ builtin_apply(self, args)
 	object *self;
 	object *args;
 {
-	object *func, *alist, *kwdict = NULL;
+	object *func, *alist = NULL, *kwdict = NULL;
 
 	if (!newgetargs(args, "O|OO:apply", &func, &alist, &kwdict))
 		return NULL;
@@ -281,6 +286,59 @@ builtin_compile(self, args)
 	return compile_string(str, filename, start);
 }
 
+#ifndef WITHOUT_COMPLEX
+
+static object *
+builtin_complex(self, args)
+	object *self;
+	object *args;
+{
+	object *r, *i, *tmp;
+	number_methods *nbr, *nbi;
+	Py_complex cr, ci;
+
+	i = NULL;
+	if (!newgetargs(args, "O|O:complex", &r, &i))
+		return NULL;
+	if ((nbr = r->ob_type->tp_as_number) == NULL ||
+	     nbr->nb_float == NULL || (i != NULL &&
+	   ((nbi = i->ob_type->tp_as_number) == NULL ||
+	     nbi->nb_float == NULL))) {
+		err_setstr(TypeError,
+			   "complex() argument can't be converted to complex");
+		return NULL;
+	}
+	if (is_complexobject(r))
+		cr = ((complexobject*)r)->cval;
+	else {
+		tmp = (*nbr->nb_float)(r);
+		if (tmp == NULL)
+			return NULL;
+		cr.real = getfloatvalue(tmp);
+		DECREF(tmp);
+		cr.imag = 0.;
+	}
+	if (i == NULL) {
+		ci.real = 0.;
+		ci.imag = 0.;
+	}
+	else if (is_complexobject(i))
+		ci = ((complexobject*)i)->cval;
+	else {
+		tmp = (*nbr->nb_float)(i);
+		if (tmp == NULL)
+			return NULL;
+		ci.real = getfloatvalue(tmp);
+		DECREF(tmp);
+		ci.imag = 0.;
+	}
+	cr.real -= ci.imag;
+	cr.imag += ci.real;
+	return newcomplexobject(cr);
+}
+
+#endif
+
 static object *
 builtin_dir(self, args)
 	object *self;
@@ -311,7 +369,11 @@ builtin_dir(self, args)
 		}
 	}
 	else {
-		v = newlistobject(0);
+		v = PyObject_CallMethod(d, "keys", NULL);
+		if (v == NULL) {
+			PyErr_Clear();
+			v = newlistobject(0);
+		}
 	}
 	DECREF(d);
 	return v;
@@ -799,6 +861,63 @@ builtin_len(self, args)
 		return newintobject(len);
 }
 
+static object *
+builtin_list(self, args)
+	object *self;
+	object *args;
+{
+	object *v;
+	sequence_methods *sqf;
+
+	if (!newgetargs(args, "O:list", &v))
+		return NULL;
+	if ((sqf = v->ob_type->tp_as_sequence) != NULL) {
+		int n = (*sqf->sq_length)(v);
+		int i;
+		object *l;
+		if (n < 0)
+			return NULL;
+		l = newlistobject(n);
+		if (l == NULL)
+			return NULL;
+		for (i = 0; i < n; i++) {
+			object *item = (*sqf->sq_item)(v, i);
+			if (item == NULL) {
+				DECREF(l);
+				l = NULL;
+				break;
+			}
+			setlistitem(l, i, item);
+		}
+		/* XXX Should support indefinite-length sequences */
+		return l;
+	}
+	err_setstr(TypeError, "list() argument must be a sequence");
+	return NULL;
+}
+
+
+static PyObject *
+builtin_slice(self, args)
+     PyObject *self;
+     PyObject *args;
+{
+  PyObject *start, *stop, *step;
+
+  start = stop = step = NULL;
+
+  if (!PyArg_ParseTuple(args, "O|OO:slice", &start, &stop, &step))
+    return NULL;
+
+  /*This swapping of stop and start is to maintain compatibility with
+    the range builtin.*/
+  if (stop == NULL) {
+    stop = start;
+    start = NULL;
+  }
+  return PySlice_New(start, stop, step);
+}
+
 static object *
 builtin_locals(self, args)
 	object *self;
@@ -954,7 +1073,11 @@ do_pow(v, w)
 		err_setstr(TypeError, "pow() requires numeric arguments");
 		return NULL;
 	}
-	if (is_floatobject(w) && getfloatvalue(v) < 0.0) {
+	if (
+#ifndef WITHOUT_COMPLEX
+            !is_complexobject(v) && 
+#endif
+            is_floatobject(w) && getfloatvalue(v) < 0.0) {
 		if (!err_occurred())
 		    err_setstr(ValueError, "negative number to float power");
 		return NULL;
@@ -1248,8 +1371,6 @@ builtin_round(self, args)
 	object *self;
 	object *args;
 {
-	extern double floor PROTO((double));
-	extern double ceil PROTO((double));
 	double x;
 	double f;
 	int ndigits = 0;
@@ -1394,6 +1515,9 @@ static struct methodlist builtin_methods[] = {
 	{"cmp",		builtin_cmp, 1},
 	{"coerce",	builtin_coerce, 1},
 	{"compile",	builtin_compile, 1},
+#ifndef WITHOUT_COMPLEX
+	{"complex",	builtin_complex, 1},
+#endif
 	{"delattr",	builtin_delattr, 1},
 	{"dir",		builtin_dir, 1},
 	{"divmod",	builtin_divmod, 1},
@@ -1410,6 +1534,7 @@ static struct methodlist builtin_methods[] = {
 	{"input",	builtin_input, 1},
 	{"int",		builtin_int, 1},
 	{"len",		builtin_len, 1},
+	{"list",	builtin_list, 1},
 	{"locals",	builtin_locals, 1},
 	{"long",	builtin_long, 1},
 	{"map",		builtin_map, 1},
@@ -1426,6 +1551,7 @@ static struct methodlist builtin_methods[] = {
 	{"repr",	builtin_repr, 1},
 	{"round",	builtin_round, 1},
 	{"setattr",	builtin_setattr, 1},
+	{"slice",       builtin_slice, 1},
 	{"str",		builtin_str, 1},
 	{"tuple",	builtin_tuple, 1},
 	{"type",	builtin_type, 1},
@@ -1513,6 +1639,7 @@ initbuiltin()
 	INCREF(builtin_dict);
 	initerrors();
 	(void) dictinsert(builtin_dict, "None", None);
+	(void) dictinsert(builtin_dict, "Ellipsis", Py_Ellipsis);
 }
 
 
@@ -1615,7 +1742,7 @@ filterstring(func, strobj)
 				GETSTRINGVALUE((stringobject *)item)[0];
 	}
 
-	if (resizestring(&result, j) < 0)
+	if (j < len && resizestring(&result, j) < 0)
 		return NULL;
 
 	return result;
