@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -26,40 +33,25 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 /* XXX TO DO:
    XXX how to pass arguments to call_trace?
-   XXX access stuff can probably dereference NULL locals?
-   XXX need to extend apply() to be able to pass keyword args
-   XXX need to be able to call built-in functions with keyword args
+   XXX totally get rid of access stuff
    XXX speed up searching for keywords by using a dictionary
-   XXX unknown keyword shouldn't raise KeyError?
    XXX document it!
    */
 
 #include "allobjects.h"
 
-#include "import.h"
-#include "sysmodule.h"
-#include "bltinmodule.h"
 #include "compile.h"
 #include "frameobject.h"
 #include "eval.h"
-#include "ceval.h"
 #include "opcode.h"
-#include "traceback.h"
 #include "graminit.h"
-#include "pythonrun.h"
 
 #include <ctype.h>
 
-extern int suppress_print; /* Declared in pythonrun.c, set in pythonmain.c */
-
 /* Turn this on if your compiler chokes on the big switch: */
 /* #define CASE_TOO_BIG 1 */
 
-/* Turn this on if you want to debug the interpreter: */
-/* (This can be on even if NDEBUG is defined) */
-/* #define DEBUG 1 */
-
-#if defined(DEBUG) || !defined(NDEBUG)
+#ifdef DEBUG
 /* For debugging the interpreter: */
 #define LLTRACE  1	/* Low-level trace feature */
 #define CHECKEXC 1	/* Double-check exception checking */
@@ -82,6 +74,7 @@ static int call_trace
 	PROTO((object **, object **, frameobject *, char *, object *));
 static object *add PROTO((object *, object *));
 static object *sub PROTO((object *, object *));
+static object *powerop PROTO((object *, object *));
 static object *mul PROTO((object *, object *));
 static object *divide PROTO((object *, object *));
 static object *mod PROTO((object *, object *));
@@ -100,6 +93,7 @@ static object *apply_subscript PROTO((object *, object *));
 static object *loop_subscript PROTO((object *, object *));
 static int slice_index PROTO((object *, int, int *));
 static object *apply_slice PROTO((object *, object *, object *));
+static object *build_slice PROTO((object *, object *, object *));
 static int assign_subscript PROTO((object *, object *, object *));
 static int assign_slice PROTO((object *, object *, object *, object *));
 static int cmp_exception PROTO((object *, object *));
@@ -107,7 +101,9 @@ static int cmp_member PROTO((object *, object *));
 static object *cmp_outcome PROTO((int, object *, object *));
 static int import_from PROTO((object *, object *, object *));
 static object *build_class PROTO((object *, object *, object *));
+#ifdef SUPPORT_OBSOLETE_ACCESS
 static int access_statement PROTO((object *, object *, frameobject *));
+#endif
 static int exec_statement PROTO((object *, object *, object *));
 static object *find_from_args PROTO((frameobject *, int));
 
@@ -200,6 +196,8 @@ restore_thread(x)
    thread (the main thread) ever takes things out of the queue.
 */
 
+static int ticker = 0; /* main loop counter to do periodic things */
+
 #define NPENDINGCALLS 32
 static struct {
 	int (*func) PROTO((ANY *));
@@ -228,6 +226,7 @@ Py_AddPendingCall(func, arg)
 	pendingcalls[i].func = func;
 	pendingcalls[i].arg = arg;
 	pendinglast = j;
+	ticker = 0; /* Signal main loop */
 	busy = 0;
 	/* XXX End critical section */
 	return 0;
@@ -238,11 +237,15 @@ Py_MakePendingCalls()
 {
 	static int busy = 0;
 #ifdef WITH_THREAD
-	if (get_thread_ident() != main_thread)
+	if (get_thread_ident() != main_thread) {
+		ticker = 0; /* We're not done yet */
 		return 0;
+	}
 #endif
-	if (busy)
+	if (busy) {
+		ticker = 0; /* We're not done yet */
 		return 0;
+	}
 	busy = 1;
 	for (;;) {
 		int i;
@@ -256,6 +259,7 @@ Py_MakePendingCalls()
 		pendingfirst = (i + 1) % NPENDINGCALLS;
 		if (func(arg) < 0) {
 			busy = 0;
+			ticker = 0; /* We're not done yet */
 			return -1;
 		}
 	}
@@ -294,6 +298,12 @@ eval_code(co, globals, locals)
 
 /* Interpreter main loop */
 
+#ifndef MAX_RECURSION_DEPTH
+#define MAX_RECURSION_DEPTH 10000
+#endif
+
+static int recursion_depth = 0;
+
 static object *
 eval_code2(co, globals, locals,
 	   args, argcount, kws, kwcount, defs, defcount, owner)
@@ -322,7 +332,9 @@ eval_code2(co, globals, locals,
 	register frameobject *f; /* Current frame */
 	register object **fastlocals;
 	object *retval;		/* Return value */
+#ifdef SUPPORT_OBSOLETE_ACCESS
 	int defmode = 0;	/* Default access mode for new variables */
+#endif
 #ifdef LLTRACE
 	int lltrace;
 #endif
@@ -368,6 +380,13 @@ eval_code2(co, globals, locals,
 #define SETLOCAL(i, value)	do { XDECREF(GETLOCAL(i)); \
 				     GETLOCAL(i) = value; } while (0)
 
+#ifdef USE_STACKCHECK
+	if (recursion_depth%10 == 0 && PyOS_CheckStack()) {
+		err_setstr(MemoryError, "Stack overflow");
+		return NULL;
+	}
+#endif
+
 	if (globals == NULL) {
 		err_setstr(SystemError, "eval_code2: NULL globals");
 		return NULL;
@@ -436,7 +455,7 @@ eval_code2(co, globals, locals,
 			}
 			if (j >= co->co_argcount) {
 				if (kwdict == NULL) {
-					err_setval(KeyError/*XXX*/, keyword);
+					err_setval(TypeError, keyword);
 					goto fail;
 				}
 				mappinginsert(kwdict, keyword, value);
@@ -527,6 +546,14 @@ eval_code2(co, globals, locals,
 		}
 	}
 
+	if (++recursion_depth > MAX_RECURSION_DEPTH) {
+		--recursion_depth;
+		err_setstr(RuntimeError, "Maximum recursion depth exceeded");
+		current_frame = f->f_back;
+		DECREF(f);
+		return NULL;
+	}
+
 	next_instr = GETUSTRINGVALUE(f->f_code->co_code);
 	stack_pointer = f->f_valuestack;
 	
@@ -535,22 +562,23 @@ eval_code2(co, globals, locals,
 	x = None;	/* Not a reference, just anything non-NULL */
 	
 	for (;;) {
-		static int ticker;
-		
 		/* Do periodic things.
 		   Doing this every time through the loop would add
 		   too much overhead (a function call per instruction).
-		   So we do it only every Nth instruction. */
-		
-		if (pendingfirst != pendinglast) {
-			if (Py_MakePendingCalls() < 0) {
-				why = WHY_EXCEPTION;
-				goto on_error;
-			}
-		}
+		   So we do it only every Nth instruction.
+
+		   The ticker is reset to zero if there are pending
+		   calls (see Py_AddPendingCalls() and
+		   Py_MakePendingCalls() above). */
 		
 		if (--ticker < 0) {
 			ticker = sys_checkinterval;
+			if (pendingfirst != pendinglast) {
+				if (Py_MakePendingCalls() < 0) {
+					why = WHY_EXCEPTION;
+					goto on_error;
+				}
+			}
 			if (sigcheck()) {
 				why = WHY_EXCEPTION;
 				goto on_error;
@@ -573,7 +601,7 @@ eval_code2(co, globals, locals,
 
 		/* Extract opcode and argument */
 
-#ifdef DEBUG
+#if defined(DEBUG) || defined(LLTRACE)
 		f->f_lasti = INSTR_OFFSET();
 #endif
 		
@@ -675,6 +703,15 @@ eval_code2(co, globals, locals,
 			PUSH(x);
 			break;
 		
+		case BINARY_POWER:
+			w = POP();
+			v = POP();
+			x = powerop(v, w);
+			DECREF(v);
+			DECREF(w);
+			PUSH(x);
+			break;
+		
 		case BINARY_MULTIPLY:
 			w = POP();
 			v = POP();
@@ -1055,6 +1092,7 @@ eval_code2(co, globals, locals,
 				err_setstr(SystemError, "no locals");
 				break;
 			}
+#ifdef SUPPORT_OBSOLETE_ACCESS
 			u = dict2lookup(x, w);
 			if (u == NULL) {
 				if (defmode != 0) {
@@ -1076,6 +1114,7 @@ eval_code2(co, globals, locals,
 				DECREF(v);
 				break;
 			}
+#endif
 			err = dict2insert(x, w, v);
 			DECREF(v);
 			break;
@@ -1086,12 +1125,14 @@ eval_code2(co, globals, locals,
 				err_setstr(SystemError, "no locals");
 				break;
 			}
+#ifdef SUPPORT_OBSOLETE_ACCESS
 			u = dict2lookup(x, w);
 			if (u != NULL && is_accessobject(u)) {
 				err = setaccessvalue(u, x,
 						     (object *)NULL);
 				break;
 			}
+#endif
 			if ((err = dict2remove(x, w)) != 0)
 				err_setval(NameError, w);
 			break;
@@ -1169,6 +1210,7 @@ eval_code2(co, globals, locals,
 		case STORE_GLOBAL:
 			w = GETNAMEV(oparg);
 			v = POP();
+#ifdef SUPPORT_OBSOLETE_ACCESS
 			if (f->f_locals != NULL) {
 				u = dict2lookup(f->f_locals, w);
 				if (u != NULL && is_accessobject(u)) {
@@ -1178,12 +1220,14 @@ eval_code2(co, globals, locals,
 					break;
 				}
 			}
+#endif
 			err = dict2insert(f->f_globals, w, v);
 			DECREF(v);
 			break;
 		
 		case DELETE_GLOBAL:
 			w = GETNAMEV(oparg);
+#ifdef SUPPORT_OBSOLETE_ACCESS
 			if (f->f_locals != NULL) {
 				u = dict2lookup(f->f_locals, w);
 				if (u != NULL && is_accessobject(u)) {
@@ -1192,6 +1236,7 @@ eval_code2(co, globals, locals,
 					break;
 				}
 			}
+#endif
 			if ((err = dict2remove(f->f_globals, w)) != 0)
 				err_setval(NameError, w);
 			break;
@@ -1221,12 +1266,14 @@ eval_code2(co, globals, locals,
 					}
 				}
 			}
+#ifdef SUPPORT_OBSOLETE_ACCESS
 			if (is_accessobject(x)) {
 				x = getaccessvalue(x, f->f_globals /* XXX */);
 				if (x == NULL)
 					break;
 			}
 			else
+#endif
 				INCREF(x);
 			PUSH(x);
 			break;
@@ -1242,12 +1289,14 @@ eval_code2(co, globals, locals,
 					break;
 				}
 			}
+#ifdef SUPPORT_OBSOLETE_ACCESS
 			if (is_accessobject(x)) {
 				x = getaccessvalue(x, f->f_globals);
 				if (x == NULL)
 					break;
 			}
 			else
+#endif
 				INCREF(x);
 			PUSH(x);
 			break;
@@ -1283,28 +1332,33 @@ eval_code2(co, globals, locals,
 							oparg));
 				break;
 			}
+#ifdef SUPPORT_OBSOLETE_ACCESS
 			if (is_accessobject(x)) {
 				x = getaccessvalue(x, f->f_locals);
 				if (x == NULL)
 					break;
 			}
 			else
+#endif
 				INCREF(x);
 			PUSH(x);
 			break;
 
 		case STORE_FAST:
 			v = POP();
+#ifdef SUPPORT_OBSOLETE_ACCESS
 			w = GETLOCAL(oparg);
 			if (w != NULL && is_accessobject(w)) {
 				err = setaccessvalue(w, f->f_locals, v);
 				DECREF(v);
 				break;
 			}
+#endif
 			SETLOCAL(oparg, v);
 			break;
 
 		case DELETE_FAST:
+#ifdef SUPPORT_OBSOLETE_ACCESS
 			x = GETLOCAL(oparg);
 			if (x == NULL) {
 				err_setval(NameError,
@@ -1317,6 +1371,7 @@ eval_code2(co, globals, locals,
 						     (object *)NULL);
 				break;
 			}
+#endif
 			SETLOCAL(oparg, NULL);
 			break;
 		
@@ -1412,6 +1467,7 @@ eval_code2(co, globals, locals,
 			locals_2_fast(f, 0);
 			break;
 
+#ifdef SUPPORT_OBSOLETE_ACCESS
 		case ACCESS_MODE:
 			v = POP();
 			w = GETNAMEV(oparg);
@@ -1421,6 +1477,7 @@ eval_code2(co, globals, locals,
 				err = access_statement(w, v, f);
 			DECREF(v);
 			break;
+#endif
 		
 		case JUMP_FORWARD:
 			JUMPBY(oparg);
@@ -1535,7 +1592,8 @@ eval_code2(co, globals, locals,
 					else {
 						err_setstr(TypeError,
 	   "unbound method must be called with class instance 1st argument");
-						return NULL;
+						x = NULL;
+						break;
 					}
 				}
 			}
@@ -1633,7 +1691,22 @@ eval_code2(co, globals, locals,
 			}
 			PUSH(x);
 			break;
-		
+
+		case BUILD_SLICE:
+			if (oparg == 3)
+				w = POP();
+			else
+				w = NULL;
+			v = POP();
+			u = POP();
+			x = build_slice(u,v,w);
+			DECREF(u);
+			DECREF(v);
+			XDECREF(w);
+			PUSH(x);
+			break;
+
+
 		default:
 			fprintf(stderr,
 				"XXX lineno: %d, opcode: %d\n",
@@ -1796,6 +1869,7 @@ eval_code2(co, globals, locals,
 
 	current_frame = f->f_back;
 	DECREF(f);
+	--recursion_depth;
 	
 	return retval;
 }
@@ -2203,6 +2277,25 @@ mod(v, w)
 	return NULL;
 }
 
+static object *
+powerop(v, w)
+	object *v, *w;
+{
+	object *res;
+	BINOP("__pow__", "__rpow__", powerop);
+	if (v->ob_type->tp_as_number == NULL ||
+	    w->ob_type->tp_as_number == NULL) {
+		err_setstr(TypeError, "pow() requires numeric arguments");
+		return NULL;
+	}
+	if (coerce(&v, &w) != 0)
+		return NULL;
+	res = (*v->ob_type->tp_as_number->nb_power)(v, w, None);
+	DECREF(v);
+	DECREF(w);
+	return res;
+}
+
 static object *
 neg(v)
 	object *v;
@@ -2457,6 +2550,9 @@ call_function(func, arg, kw)
 	return result;
 }
 
+#define SLICE_ERROR_MSG \
+	"standard sequence type does not support step size other than one"
+
 static object *
 apply_subscript(v, w)
 	object *v, *w;
@@ -2471,8 +2567,13 @@ apply_subscript(v, w)
 	}
 	else {
 		int i;
-		if (!is_intobject(w)) {
-			err_setstr(TypeError, "sequence subscript not int");
+		if (!is_intobject(w)) {		  
+			if (PySlice_Check(w)) {
+			        err_setstr(ValueError, SLICE_ERROR_MSG); 
+			} else {
+				err_setstr(TypeError,
+					   "sequence subscript not int");
+			}	
 			return NULL;
 		}
 		i = getintvalue(w);
@@ -2523,6 +2624,13 @@ slice_index(v, isize, pi)
 	return 0;
 }
 
+static object *
+build_slice(u, v, w) /* u:v:w */
+	object *u, *v, *w;
+{
+  return PySlice_New(u,v,w);
+}
+
 static object *
 apply_slice(u, v, w) /* return u[v:w] */
 	object *u, *v, *w;
@@ -2739,6 +2847,7 @@ import_from(locals, v, name)
 			if (!is_stringobject(name) ||
 			    getstringvalue(name)[0] == '_')
 				continue;
+#ifdef SUPPORT_OBSOLETE_ACCESS
 			if (is_accessobject(value)) {
 				value = getaccessvalue(value, (object *)NULL);
 				if (value == NULL) {
@@ -2747,6 +2856,7 @@ import_from(locals, v, name)
 				}
 			}
 			else
+#endif
 				INCREF(value);
 			err = dict2insert(locals, name, value);
 			DECREF(value);
@@ -2815,6 +2925,7 @@ build_class(methods, bases, name)
 	return newclassobject(bases, methods, name);
 }
 
+#ifdef SUPPORT_OBSOLETE_ACCESS
 static int
 access_statement(name, vmode, f)
 	object *name;
@@ -2844,6 +2955,7 @@ access_statement(name, vmode, f)
 	locals_2_fast(f, 0);
 	return ret;
 }
+#endif
 
 static int
 exec_statement(prog, globals, locals)
