@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -35,6 +42,10 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
    XXX other JAR tricks?
 */
 
+#ifndef NO_PRIVATE_NAME_MANGLING
+#define PRIVATE_NAME_MANGLING
+#endif
+
 #include "allobjects.h"
 
 #include "node.h"
@@ -47,6 +58,10 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <ctype.h>
 #include <errno.h>
 
+#define OP_DELETE 0
+#define OP_ASSIGN 1
+#define OP_APPLY 2
+
 #define OFF(x) offsetof(codeobject, x)
 
 static struct memberlist code_memberlist[] = {
@@ -248,9 +263,43 @@ struct compiling {
 	int c_nblocks;		/* current block stack level */
 	char *c_filename;	/* filename of current node */
 	char *c_name;		/* name of object (e.g. function) */
+	int c_lineno;		/* Current line number */
+#ifdef PRIVATE_NAME_MANGLING
+	char *c_private;	/* for private name mangling */
+#endif
 };
 
 
+/* Error message including line number */
+
+static void
+com_error(c, exc, msg)
+	struct compiling *c;
+	object *exc;
+	char *msg;
+{
+	int n = strlen(msg);
+	object *v;
+	char buffer[30];
+	char *s;
+	if (c->c_lineno <= 1) {
+		/* Unknown line number or single interactive command */
+		err_setstr(exc, msg);
+		return;
+	}
+	sprintf(buffer, " (line %d)", c->c_lineno);
+	v = newsizedstringobject((char *)NULL, n + strlen(buffer));
+	if (v == NULL)
+		return; /* MemoryError, too bad */
+	s = GETSTRINGVALUE((stringobject *)v);
+	strcpy(s, msg);
+	strcat(s, buffer);
+	err_setval(exc, v);
+	DECREF(v);
+	c->c_errors++;
+}
+
+
 /* Interface to the block stack */
 
 static void
@@ -259,8 +308,7 @@ block_push(c, type)
 	int type;
 {
 	if (c->c_nblocks >= MAXBLOCKS) {
-		err_setstr(SystemError, "too many statically nested blocks");
-		c->c_errors++;
+		com_error(c, SystemError, "too many statically nested blocks");
 	}
 	else {
 		c->c_block[c->c_nblocks++] = type;
@@ -275,8 +323,7 @@ block_pop(c, type)
 	if (c->c_nblocks > 0)
 		c->c_nblocks--;
 	if (c->c_block[c->c_nblocks] != type && c->c_errors == 0) {
-		err_setstr(SystemError, "bad block pop");
-		c->c_errors++;
+		com_error(c, SystemError, "bad block pop");
 	}
 }
 
@@ -287,6 +334,7 @@ static int com_init PROTO((struct compiling *, char *));
 static void com_free PROTO((struct compiling *));
 static void com_done PROTO((struct compiling *));
 static void com_node PROTO((struct compiling *, struct _node *));
+static void com_factor PROTO((struct compiling *, struct _node *));
 static void com_addbyte PROTO((struct compiling *, int));
 static void com_addint PROTO((struct compiling *, int));
 static void com_addoparg PROTO((struct compiling *, int, int));
@@ -299,6 +347,8 @@ static void com_addopname PROTO((struct compiling *, int, node *));
 static void com_list PROTO((struct compiling *, node *, int));
 static int com_argdefs PROTO((struct compiling *, node *));
 static int com_newlocal PROTO((struct compiling *, char *));
+static codeobject *icompile PROTO((struct _node *, struct compiling *));
+static codeobject *jcompile PROTO((struct _node *, char *, struct compiling *));
 
 static int
 com_init(c, filename)
@@ -329,6 +379,7 @@ com_init(c, filename)
 	c->c_nblocks = 0;
 	c->c_filename = filename;
 	c->c_name = "?";
+	c->c_lineno = 0;
 	return 1;
 	
   fail_000:
@@ -377,8 +428,7 @@ com_addbyte(c, byte)
 		fprintf(stderr, "XXX compiling bad byte: %d\n", byte);
 		fatal("com_addbyte: byte out of range");
 		*/
-		err_setstr(SystemError, "com_addbyte: byte out of range");
-		c->c_errors++;
+		com_error(c, SystemError, "com_addbyte: byte out of range");
 	}
 	if (c->c_code == NULL)
 		return;
@@ -407,6 +457,8 @@ com_addoparg(c, op, arg)
 	int op;
 	int arg;
 {
+	if (op == SET_LINENO)
+		c->c_lineno = arg;
 	com_addbyte(c, op);
 	com_addint(c, arg);
 }
@@ -484,6 +536,41 @@ com_addname(c, v)
 	return com_add(c, c->c_names, v);
 }
 
+#ifdef PRIVATE_NAME_MANGLING
+static int
+com_mangle(c, name, buffer, maxlen)
+	struct compiling *c;
+	char *name;
+	char *buffer;
+	int maxlen;
+{
+	/* Name mangling: __private becomes _classname__private.
+	   This is independent from how the name is used. */
+	char *p;
+	int nlen, plen;
+	nlen = strlen(name);
+	if (nlen+2 >= maxlen)
+		return 0; /* Don't mangle __extremely_long_names */
+	if (name[nlen-1] == '_' && name[nlen-2] == '_')
+		return 0; /* Don't mangle __whatever__ */
+	p = c->c_private;
+	/* Strip leading underscores from class name */
+	while (*p == '_')
+		p++;
+	if (*p == '\0')
+		return 0; /* Don't mangle if class is just underscores */
+	plen = strlen(p);
+	if (plen + nlen >= maxlen)
+		plen = maxlen-nlen-2; /* Truncate class name if too long */
+	/* buffer = "_" + p[:plen] + name # i.e. 1+plen+nlen bytes */
+	buffer[0] = '_';
+	strncpy(buffer+1, p, plen);
+	strcpy(buffer+1+plen, name);
+	/* fprintf(stderr, "mangle %s -> %s\n", name, buffer); */
+	return 1;
+}
+#endif
+
 static void
 com_addopnamestr(c, op, name)
 	struct compiling *c;
@@ -492,6 +579,13 @@ com_addopnamestr(c, op, name)
 {
 	object *v;
 	int i;
+#ifdef PRIVATE_NAME_MANGLING
+	char buffer[256];
+	if (name != NULL && name[0] == '_' && name[1] == '_' &&
+	    c->c_private != NULL &&
+	    com_mangle(c, name, buffer, (int)sizeof(buffer)))
+		name = buffer;
+#endif
 	if (name == NULL || (v = newstringobject(name)) == NULL) {
 		c->c_errors++;
 		i = 255;
@@ -536,8 +630,8 @@ com_addopname(c, op, n)
 		for (i = 0; i < NCH(n); i += 2) {
 			char *s = STR(CHILD(n, i));
 			if (p + strlen(s) > buffer + (sizeof buffer) - 2) {
-				err_setstr(MemoryError,
-					   "dotted_name too long");
+				com_error(c, MemoryError,
+					  "dotted_name too long");
 				name = NULL;
 				break;
 			}
@@ -555,7 +649,8 @@ com_addopname(c, op, n)
 }
 
 static object *
-parsenumber(s)
+parsenumber(co, s)
+	struct compiling *co;
 	char *s;
 {
 	extern long mystrtol PROTO((const char *, char **, int));
@@ -563,8 +658,16 @@ parsenumber(s)
 	extern double atof PROTO((const char *));
 	char *end;
 	long x;
+#ifndef WITHOUT_COMPLEX
+	Py_complex c;
+	int imflag;
+#endif
+
 	errno = 0;
 	end = s + strlen(s) - 1;
+#ifndef WITHOUT_COMPLEX
+	imflag = *end == 'j' || *end == 'J';
+#endif
 	if (*end == 'l' || *end == 'L')
 		return long_scan(s, 0);
 	if (s[0] == '0')
@@ -573,14 +676,22 @@ parsenumber(s)
 		x = mystrtol(s, &end, 0);
 	if (*end == '\0') {
 		if (errno != 0) {
-			err_setstr(OverflowError,
-				   "integer literal too large");
+			com_error(co, OverflowError,
+				  "integer literal too large");
 			return NULL;
 		}
 		return newintobject(x);
 	}
 	/* XXX Huge floats may silently fail */
-	return newfloatobject(atof(s));
+#ifndef WITHOUT_COMPLEX
+	if (imflag) {
+		c.real = 0.;
+		c.imag = atof(s);
+		return newcomplexobject(c);
+	}
+	else
+#endif
+		return newfloatobject(atof(s));
 }
 
 static object *
@@ -746,8 +857,7 @@ com_atom(c, n)
 		com_addbyte(c, UNARY_CONVERT);
 		break;
 	case NUMBER:
-		if ((v = parsenumber(STR(ch))) == NULL) {
-			c->c_errors++;
+		if ((v = parsenumber(c, STR(ch))) == NULL) {
 			i = 255;
 		}
 		else {
@@ -773,8 +883,7 @@ com_atom(c, n)
 		break;
 	default:
 		fprintf(stderr, "node type %d\n", TYPE(ch));
-		err_setstr(SystemError, "com_atom: unexpected node type");
-		c->c_errors++;
+		com_error(c, SystemError, "com_atom: unexpected node type");
 	}
 }
 
@@ -804,23 +913,6 @@ com_slice(c, n, op)
 	}
 }
 
-static void
-com_apply_subscript(c, n)
-	struct compiling *c;
-	node *n;
-{
-	REQ(n, subscript);
-	if (NCH(n) == 1 && TYPE(CHILD(n, 0)) != COLON) {
-		/* It's a single subscript */
-		com_node(c, CHILD(n, 0));
-		com_addbyte(c, BINARY_SUBSCR);
-	}
-	else {
-		/* It's a slice: [expr] ':' [expr] */
-		com_slice(c, n, SLICE);
-	}
-}
-
 static int
 com_argument(c, n, inkeywords)
 	struct compiling *c;
@@ -831,9 +923,8 @@ com_argument(c, n, inkeywords)
 	REQ(n, argument); /* [test '='] test; really [ keyword '='] keyword */
 	if (NCH(n) == 1) {
 		if (inkeywords) {
-			err_setstr(SyntaxError,
+			com_error(c, SyntaxError,
 				   "non-keyword arg after keyword arg");
-			c->c_errors++;
 		}
 		else {
 			com_node(c, CHILD(n, 0));
@@ -845,8 +936,7 @@ com_argument(c, n, inkeywords)
 		m = CHILD(m, 0);
 	} while (NCH(m) == 1);
 	if (TYPE(m) != NAME) {
-		err_setstr(SyntaxError, "keyword can't be an expression");
-		c->c_errors++;
+		com_error(c, SyntaxError, "keyword can't be an expression");
 	}
 	else {
 		object *v = newstringobject(STR(m));
@@ -883,8 +973,7 @@ com_call_function(c, n)
 				nk++;
 		}
 		if (na > 255 || nk > 255) {
-			err_setstr(SyntaxError, "more than 255 arguments");
-			c->c_errors++;
+			com_error(c, SyntaxError, "more than 255 arguments");
 		}
 		com_addoparg(c, CALL_FUNCTION, na | (nk << 8));
 	}
@@ -898,6 +987,107 @@ com_select_member(c, n)
 	com_addopname(c, LOAD_ATTR, n);
 }
 
+static void
+com_sliceobj(c, n)
+	struct compiling *c;
+	node *n;
+{
+	int i=0;
+	int ns=2; /* number of slice arguments */
+	int first_missing=0;
+	node *ch;
+
+	/* first argument */
+	if (TYPE(CHILD(n,i)) == COLON) {
+		com_addoparg(c, LOAD_CONST, com_addconst(c, None));
+		i++;
+	}
+	else {
+		com_node(c, CHILD(n,i));
+		i++;
+		REQ(CHILD(n,i),COLON);
+		i++;
+	}
+	/* second argument */
+	if (i < NCH(n) && TYPE(CHILD(n,i)) == test) {
+		com_node(c, CHILD(n,i));
+		i++;
+	}
+	else com_addoparg(c, LOAD_CONST, com_addconst(c, None));
+	/* remaining arguments */
+	for (; i < NCH(n); i++) {
+		ns++;
+		ch=CHILD(n,i);
+		REQ(ch, sliceop);
+		if (NCH(ch) == 1) {
+			/* right argument of ':' missing */
+			com_addoparg(c, LOAD_CONST, com_addconst(c, None));
+		}
+		else
+			com_node(c, CHILD(ch,1));
+	}
+	com_addoparg(c, BUILD_SLICE, ns);
+}
+
+static void
+com_subscript(c, n)
+	struct compiling *c;
+	node *n;
+{
+	node *ch;
+	REQ(n, subscript);
+	ch = CHILD(n,0);
+	/* check for rubber index */
+	if (TYPE(ch) == DOT && TYPE(CHILD(n,1)) == DOT)
+		com_addoparg(c, LOAD_CONST, com_addconst(c, Py_Ellipsis));
+	else {
+		/* check for slice */
+		if ((TYPE(ch) == COLON || NCH(n) > 1))
+			com_sliceobj(c, n);
+		else {
+			REQ(ch, test);
+			com_node(c, ch);
+		}
+	}
+}
+
+static void
+com_subscriptlist(c, n, assigning)
+	struct compiling *c;
+	node *n;
+	int assigning;
+{
+	int i, op;
+	REQ(n, subscriptlist);
+	/* Check to make backward compatible slice behavior for '[i:j]' */
+	if (NCH(n) == 1) {
+		node *sub = CHILD(n, 0); /* subscript */
+		/* Make it is a simple slice.
+           should have exactly one colon. */
+        if ((TYPE(CHILD(sub, 0)) == COLON
+             || (NCH(sub) > 1 && TYPE(CHILD(sub, 1)) == COLON))
+            && (TYPE(CHILD(sub,NCH(sub)-1)) != sliceop)) {
+			if (assigning == OP_APPLY)
+				op = SLICE;
+			else
+				op = ((assigning == OP_ASSIGN) ? STORE_SLICE : DELETE_SLICE);
+			com_slice(c, sub, op);
+			return;
+		}
+	}
+	/* Else normal subscriptlist.  Compile each subscript. */
+	for (i = 0; i < NCH(n); i += 2)
+		com_subscript(c, CHILD(n, i));
+	/* Put multiple subscripts into a tuple */
+	if (NCH(n) > 1)
+		com_addoparg(c, BUILD_TUPLE, (NCH(n)+1) / 2);
+	if (assigning == OP_APPLY)
+		op = BINARY_SUBSCR;
+	else
+		op = ((assigning == OP_ASSIGN) ? STORE_SUBSCR : DELETE_SUBSCR);
+	com_addbyte(c, op);
+}
+
 static void
 com_apply_trailer(c, n)
 	struct compiling *c;
@@ -912,12 +1102,30 @@ com_apply_trailer(c, n)
 		com_select_member(c, CHILD(n, 1));
 		break;
 	case LSQB:
-		com_apply_subscript(c, CHILD(n, 1));
+		com_subscriptlist(c, CHILD(n, 1), OP_APPLY);
 		break;
 	default:
-		err_setstr(SystemError,
-			"com_apply_trailer: unknown trailer type");
-		c->c_errors++;
+		com_error(c, SystemError,
+			  "com_apply_trailer: unknown trailer type");
+	}
+}
+
+static void
+com_power(c, n)
+	struct compiling *c;
+	node *n;
+{
+	int i;
+	REQ(n, power);
+	com_atom(c, CHILD(n, 0));
+	for (i = 1; i < NCH(n); i++) {
+		if (TYPE(CHILD(n, i)) == DOUBLESTAR) {
+			com_factor(c, CHILD(n, i+1));
+			com_addbyte(c, BINARY_POWER);
+			break;
+		}
+		else
+			com_apply_trailer(c, CHILD(n, i));
 	}
 }
 
@@ -926,7 +1134,6 @@ com_factor(c, n)
 	struct compiling *c;
 	node *n;
 {
-	int i;
 	REQ(n, factor);
 	if (TYPE(CHILD(n, 0)) == PLUS) {
 		com_factor(c, CHILD(n, 1));
@@ -941,9 +1148,7 @@ com_factor(c, n)
 		com_addbyte(c, UNARY_INVERT);
 	}
 	else {
-		com_atom(c, CHILD(n, 0));
-		for (i = 1; i < NCH(n); i++)
-			com_apply_trailer(c, CHILD(n, i));
+		com_power(c, CHILD(n, 0));
 	}
 }
 
@@ -969,9 +1174,8 @@ com_term(c, n)
 			op = BINARY_MODULO;
 			break;
 		default:
-			err_setstr(SystemError,
-				"com_term: operator not *, / or %");
-			c->c_errors++;
+			com_error(c, SystemError,
+				  "com_term: operator not *, / or %");
 			op = 255;
 		}
 		com_addbyte(c, op);
@@ -997,9 +1201,8 @@ com_arith_expr(c, n)
 			op = BINARY_SUBTRACT;
 			break;
 		default:
-			err_setstr(SystemError,
-				"com_arith_expr: operator not + or -");
-			c->c_errors++;
+			com_error(c, SystemError,
+				  "com_arith_expr: operator not + or -");
 			op = 255;
 		}
 		com_addbyte(c, op);
@@ -1025,9 +1228,8 @@ com_shift_expr(c, n)
 			op = BINARY_RSHIFT;
 			break;
 		default:
-			err_setstr(SystemError,
-				"com_shift_expr: operator not << or >>");
-			c->c_errors++;
+			com_error(c, SystemError,
+				  "com_shift_expr: operator not << or >>");
 			op = 255;
 		}
 		com_addbyte(c, op);
@@ -1049,9 +1251,8 @@ com_and_expr(c, n)
 			op = BINARY_AND;
 		}
 		else {
-			err_setstr(SystemError,
-				"com_and_expr: operator not &");
-			c->c_errors++;
+			com_error(c, SystemError,
+				  "com_and_expr: operator not &");
 			op = 255;
 		}
 		com_addbyte(c, op);
@@ -1073,9 +1274,8 @@ com_xor_expr(c, n)
 			op = BINARY_XOR;
 		}
 		else {
-			err_setstr(SystemError,
-				"com_xor_expr: operator not ^");
-			c->c_errors++;
+			com_error(c, SystemError,
+				  "com_xor_expr: operator not ^");
 			op = 255;
 		}
 		com_addbyte(c, op);
@@ -1097,9 +1297,8 @@ com_expr(c, n)
 			op = BINARY_OR;
 		}
 		else {
-			err_setstr(SystemError,
-				"com_expr: expr operator not |");
-			c->c_errors++;
+			com_error(c, SystemError,
+				  "com_expr: expr operator not |");
 			op = 255;
 		}
 		com_addbyte(c, op);
@@ -1194,9 +1393,8 @@ com_comparison(c, n)
 		}
 		op = cmp_type(CHILD(n, i-1));
 		if (op == BAD) {
-			err_setstr(SystemError,
-				"com_comparison: unknown comparison op");
-			c->c_errors++;
+			com_error(c, SystemError,
+				  "com_comparison: unknown comparison op");
 		}
 		com_addoparg(c, COMPARE_OP, op);
 		if (i+2 < NCH(n)) {
@@ -1261,7 +1459,7 @@ com_test(c, n)
 		object *v;
 		int i;
 		int ndefs = com_argdefs(c, CHILD(n, 0));
-		v = (object *) compile(CHILD(n, 0), c->c_filename);
+		v = (object *) icompile(CHILD(n, 0), c);
 		if (v == NULL) {
 			c->c_errors++;
 			i = 255;
@@ -1323,25 +1521,6 @@ com_assign_attr(c, n, assigning)
 	com_addopname(c, assigning ? STORE_ATTR : DELETE_ATTR, n);
 }
 
-static void
-com_assign_slice(c, n, assigning)
-	struct compiling *c;
-	node *n;
-	int assigning;
-{
-	com_slice(c, n, assigning ? STORE_SLICE : DELETE_SLICE);
-}
-
-static void
-com_assign_subscript(c, n, assigning)
-	struct compiling *c;
-	node *n;
-	int assigning;
-{
-	com_node(c, n);
-	com_addbyte(c, assigning ? STORE_SUBSCR : DELETE_SUBSCR);
-}
-
 static void
 com_assign_trailer(c, n, assigning)
 	struct compiling *c;
@@ -1351,23 +1530,16 @@ com_assign_trailer(c, n, assigning)
 	REQ(n, trailer);
 	switch (TYPE(CHILD(n, 0))) {
 	case LPAR: /* '(' [exprlist] ')' */
-		err_setstr(SyntaxError, "can't assign to function call");
-		c->c_errors++;
+		com_error(c, SyntaxError, "can't assign to function call");
 		break;
 	case DOT: /* '.' NAME */
 		com_assign_attr(c, CHILD(n, 1), assigning);
 		break;
-	case LSQB: /* '[' subscript ']' */
-		n = CHILD(n, 1);
-		REQ(n, subscript); /* subscript: expr | [expr] ':' [expr] */
-		if (NCH(n) > 1 || TYPE(CHILD(n, 0)) == COLON)
-			com_assign_slice(c, n, assigning);
-		else
-			com_assign_subscript(c, CHILD(n, 0), assigning);
+	case LSQB: /* '[' subscriptlist ']' */
+		com_subscriptlist(c, CHILD(n, 1), assigning);
 		break;
 	default:
-		err_setstr(SystemError, "unknown trailer type");
-		c->c_errors++;
+		com_error(c, SystemError, "unknown trailer type");
 	}
 }
 
@@ -1438,26 +1610,31 @@ com_assign(c, n, assigning)
 		case shift_expr:
 		case arith_expr:
 		case term:
+		case factor:
 			if (NCH(n) > 1) {
-				err_setstr(SyntaxError,
-					"can't assign to operator");
-				c->c_errors++;
+				com_error(c, SyntaxError,
+					  "can't assign to operator");
 				return;
 			}
 			n = CHILD(n, 0);
 			break;
 		
-		case factor: /* ('+'|'-'|'~') factor | atom trailer* */
-			if (TYPE(CHILD(n, 0)) != atom) { /* '+'|'-'|'~' */
-				err_setstr(SyntaxError,
-					"can't assign to operator");
-				c->c_errors++;
+		case power: /* atom trailer* ('**' power)* */
+/* ('+'|'-'|'~') factor | atom trailer* */
+			if (TYPE(CHILD(n, 0)) != atom) {
+				com_error(c, SyntaxError,
+					  "can't assign to operator");
 				return;
 			}
-			if (NCH(n) > 1) { /* trailer present */
+			if (NCH(n) > 1) { /* trailer or exponent present */
 				int i;
 				com_node(c, CHILD(n, 0));
 				for (i = 1; i+1 < NCH(n); i++) {
+					if (TYPE(CHILD(n, i)) == DOUBLESTAR) {
+						com_error(c, SyntaxError,
+						  "can't assign to operator");
+						return;
+					}
 					com_apply_trailer(c, CHILD(n, i));
 				} /* NB i is still alive */
 				com_assign_trailer(c,
@@ -1473,18 +1650,16 @@ com_assign(c, n, assigning)
 				n = CHILD(n, 1);
 				if (TYPE(n) == RPAR) {
 					/* XXX Should allow () = () ??? */
-					err_setstr(SyntaxError,
-						"can't assign to ()");
-					c->c_errors++;
+					com_error(c, SyntaxError,
+						  "can't assign to ()");
 					return;
 				}
 				break;
 			case LSQB:
 				n = CHILD(n, 1);
 				if (TYPE(n) == RSQB) {
-					err_setstr(SyntaxError,
-						"can't assign to []");
-					c->c_errors++;
+					com_error(c, SyntaxError,
+						  "can't assign to []");
 					return;
 				}
 				com_assign_list(c, n, assigning);
@@ -1493,17 +1668,19 @@ com_assign(c, n, assigning)
 				com_assign_name(c, CHILD(n, 0), assigning);
 				return;
 			default:
-				err_setstr(SyntaxError,
-						"can't assign to literal");
-				c->c_errors++;
+				com_error(c, SyntaxError,
+					  "can't assign to literal");
 				return;
 			}
 			break;
+
+		case lambdef:
+			com_error(c, SyntaxError, "can't assign to lambda");
+			return;
 		
 		default:
 			fprintf(stderr, "node type %d\n", TYPE(n));
-			err_setstr(SystemError, "com_assign: bad node");
-			c->c_errors++;
+			com_error(c, SystemError, "com_assign: bad node");
 			return;
 		
 		}
@@ -1528,7 +1705,7 @@ com_expr_stmt(c, n)
 		for (i = 0; i < NCH(n)-2; i+=2) {
 			if (i+2 < NCH(n)-2)
 				com_addbyte(c, DUP_TOP);
-			com_assign(c, CHILD(n, i), 1/*assign*/);
+			com_assign(c, CHILD(n, i), OP_ASSIGN);
 		}
 	}
 }
@@ -1556,8 +1733,7 @@ com_return_stmt(c, n)
 {
 	REQ(n, return_stmt); /* 'return' [testlist] */
 	if (!c->c_infunction) {
-		err_setstr(SyntaxError, "'return' outside function");
-		c->c_errors++;
+		com_error(c, SyntaxError, "'return' outside function");
 	}
 	if (NCH(n) < 2)
 		com_addoparg(c, LOAD_CONST, com_addconst(c, None));
@@ -1618,9 +1794,15 @@ com_global_stmt(c, n)
 	/* 'global' NAME (',' NAME)* */
 	for (i = 1; i < NCH(n); i += 2) {
 		char *s = STR(CHILD(n, i));
+#ifdef PRIVATE_NAME_MANGLING
+		char buffer[256];
+		if (s != NULL && s[0] == '_' && s[1] == '_' &&
+		    c->c_private != NULL &&
+		    com_mangle(c, s, buffer, (int)sizeof(buffer)))
+			s = buffer;
+#endif
 		if (dictlookup(c->c_locals, s) != NULL) {
-			err_setstr(SyntaxError, "name is local and global");
-			c->c_errors++;
+			com_error(c, SyntaxError, "name is local and global");
 		}
 		else if (dictinsert(c->c_globals, s, None) != 0)
 			c->c_errors++;
@@ -1637,8 +1819,7 @@ com_newlocal_o(c, nameval)
 	if (getlistsize(c->c_varnames) != c->c_nlocals) {
 		/* This is usually caused by an error on a previous call */
 		if (c->c_errors == 0) {
-			err_setstr(SystemError, "mixed up var name/index");
-			c->c_errors++;
+			com_error(c, SystemError, "mixed up var name/index");
 		}
 		return 0;
 	}
@@ -1682,12 +1863,12 @@ com_newlocal(c, name)
 
 #define strequ(a, b) (strcmp((a), (b)) == 0)
 
+#ifdef SUPPORT_OBSOLETE_ACCESS
 static void
 com_access_stmt(c, n)
 	struct compiling *c;
 	node *n;
 {
-#if 0
 	int i, j, k, mode, imode;
 	object *vmode;
 	REQ(n, access_stmt);
@@ -1738,8 +1919,8 @@ com_access_stmt(c, n)
 		com_addoparg(c, LOAD_CONST, imode);
 		com_addopname(c, ACCESS_MODE, CHILD(n, i));
 	}
-#endif
 }
+#endif
 
 static void
 com_exec_stmt(c, n)
@@ -1839,7 +2020,7 @@ com_for_stmt(c, n)
 	c->c_begin = c->c_nexti;
 	com_addoparg(c, SET_LINENO, n->n_lineno);
 	com_addfwref(c, FOR_LOOP, &anchor);
-	com_assign(c, CHILD(n, 1), 1/*assigning*/);
+	com_assign(c, CHILD(n, 1), OP_ASSIGN);
 	c->c_loops++;
 	com_node(c, CHILD(n, 5));
 	c->c_loops--;
@@ -1942,9 +2123,8 @@ com_try_except(c, n)
 	     i += 3) {
 		/* except_clause: 'except' [expr [',' expr]] */
 		if (except_anchor == 0) {
-			err_setstr(SyntaxError,
-				"default 'except:' must be last");
-			c->c_errors++;
+			com_error(c, SyntaxError,
+				  "default 'except:' must be last");
 			break;
 		}
 		except_anchor = 0;
@@ -1958,7 +2138,7 @@ com_try_except(c, n)
 		}
 		com_addbyte(c, POP_TOP);
 		if (NCH(ch) > 3)
-			com_assign(c, CHILD(ch, 3), 1/*assigning*/);
+			com_assign(c, CHILD(ch, 3), OP_ASSIGN);
 		else
 			com_addbyte(c, POP_TOP);
 		com_addbyte(c, POP_TOP);
@@ -2059,6 +2239,7 @@ get_docstring(n)
 	case arith_expr:
 	case term:
 	case factor:
+	case power:
 		if (NCH(n) == 1)
 			return get_docstring(CHILD(n, 0));
 		break;
@@ -2103,8 +2284,7 @@ com_continue_stmt(c, n)
 		com_addoparg(c, JUMP_ABSOLUTE, c->c_begin);
 	}
 	else {
-		err_setstr(SyntaxError, "'continue' not properly in loop");
-		c->c_errors++;
+		com_error(c, SyntaxError, "'continue' not properly in loop");
 	}
 	/* XXX Could allow it inside a 'finally' clause
 	   XXX if we could pop the exception still on the stack */
@@ -2136,7 +2316,7 @@ com_argdefs(c, n)
 	ndefs = 0;
 	for (i = 0; i < nch; i++) {
 		int t;
-		if (TYPE(CHILD(n, i)) == STAR)
+		if (TYPE(CHILD(n, i)) == STAR || TYPE(CHILD(n, i)) == DOUBLESTAR)
 			break;
 		nargs++;
 		i++;
@@ -2174,7 +2354,7 @@ com_funcdef(c, n)
 {
 	object *v;
 	REQ(n, funcdef); /* funcdef: 'def' NAME parameters ':' suite */
-	v = (object *)compile(n, c->c_filename);
+	v = (object *)icompile(n, c);
 	if (v == NULL)
 		c->c_errors++;
 	else {
@@ -2222,7 +2402,7 @@ com_classdef(c, n)
 		com_addoparg(c, BUILD_TUPLE, 0);
 	else
 		com_bases(c, CHILD(n, 3));
-	v = (object *)compile(n, c->c_filename);
+	v = (object *)icompile(n, c);
 	if (v == NULL)
 		c->c_errors++;
 	else {
@@ -2284,14 +2464,13 @@ com_node(c, n)
 		com_print_stmt(c, n);
 		break;
 	case del_stmt: /* 'del' exprlist */
-		com_assign(c, CHILD(n, 1), 0/*delete*/);
+		com_assign(c, CHILD(n, 1), OP_DELETE);
 		break;
 	case pass_stmt:
 		break;
 	case break_stmt:
 		if (c->c_loops == 0) {
-			err_setstr(SyntaxError, "'break' outside loop");
-			c->c_errors++;
+			com_error(c, SyntaxError, "'break' outside loop");
 		}
 		com_addbyte(c, BREAK_LOOP);
 		break;
@@ -2310,9 +2489,11 @@ com_node(c, n)
 	case global_stmt:
 		com_global_stmt(c, n);
 		break;
+#ifdef SUPPORT_OBSOLETE_ACCESS
 	case access_stmt:
 		com_access_stmt(c, n);
 		break;
+#endif
 	case exec_stmt:
 		com_exec_stmt(c, n);
 		break;
@@ -2373,14 +2554,16 @@ com_node(c, n)
 	case factor:
 		com_factor(c, n);
 		break;
+	case power:
+		com_power(c, n);
+		break;
 	case atom:
 		com_atom(c, n);
 		break;
 	
 	default:
 		fprintf(stderr, "node type %d\n", TYPE(n));
-		err_setstr(SystemError, "com_node: unexpected node type");
-		c->c_errors++;
+		com_error(c, SystemError, "com_node: unexpected node type");
 	}
 }
 
@@ -2431,7 +2614,7 @@ com_arglist(c, n)
 		node *ch = CHILD(n, i);
 		node *fp;
 		char *name;
-		if (TYPE(ch) == STAR)
+		if (TYPE(ch) == STAR || TYPE(ch) == DOUBLESTAR)
 			break;
 		REQ(ch, fpdef); /* fpdef: NAME | '(' fplist ')' */
 		fp = CHILD(ch, 0);
@@ -2439,7 +2622,7 @@ com_arglist(c, n)
 			name = STR(fp);
 		else {
 			name = "";
-			complex= 1;
+			complex = 1;
 		}
 		com_newlocal(c, name);
 		c->c_argcount++;
@@ -2455,22 +2638,28 @@ com_arglist(c, n)
 	if (i < nch) {
 		node *ch;
 		ch = CHILD(n, i);
-		REQ(ch, STAR);
-		ch = CHILD(n, i+1);
-		if (TYPE(ch) == NAME) {
-			c->c_flags |= CO_VARARGS;
-			i += 3;
-			com_newlocal(c, STR(ch));
+		if (TYPE(ch) != DOUBLESTAR) {
+			REQ(ch, STAR);
+			ch = CHILD(n, i+1);
+			if (TYPE(ch) == NAME) {
+				c->c_flags |= CO_VARARGS;
+				i += 3;
+				com_newlocal(c, STR(ch));
+			}
 		}
 	}
 	/* Handle **keywords */
 	if (i < nch) {
 		node *ch;
 		ch = CHILD(n, i);
-		REQ(ch, STAR);
-		ch = CHILD(n, i+1);
-		REQ(ch, STAR);
-		ch = CHILD(n, i+2);
+		if (TYPE(ch) != DOUBLESTAR) {
+			REQ(ch, STAR);
+			ch = CHILD(n, i+1);
+			REQ(ch, STAR);
+			ch = CHILD(n, i+2);
+		}
+		else
+			ch = CHILD(n, i+1);
 		REQ(ch, NAME);
 		c->c_flags |= CO_VARKEYWORDS;
 		com_newlocal(c, STR(ch));
@@ -2482,7 +2671,7 @@ com_arglist(c, n)
 		for (i = 0; i < nch; i++) {
 			node *ch = CHILD(n, i);
 			node *fp;
-			if (TYPE(ch) == STAR)
+			if (TYPE(ch) == STAR || TYPE(ch) == DOUBLESTAR)
 				break;
 			REQ(ch, fpdef); /* fpdef: NAME | '(' fplist ')' */
 			fp = CHILD(ch, 0);
@@ -2584,6 +2773,9 @@ compile_classdef(c, n)
 	REQ(n, classdef);
 	/* classdef: 'class' NAME ['(' testlist ')'] ':' suite */
 	c->c_name = STR(CHILD(n, 1));
+#ifdef PRIVATE_NAME_MANGLING
+	c->c_private = c->c_name;
+#endif
 	ch = CHILD(n, NCH(n)-1); /* The suite */
 	doc = get_docstring(ch);
 	if (doc != NULL) {
@@ -2644,8 +2836,8 @@ compile_node(c, n)
 	
 	default:
 		fprintf(stderr, "node type %d\n", TYPE(n));
-		err_setstr(SystemError, "compile_node: unexpected node type");
-		c->c_errors++;
+		com_error(c, SystemError,
+			  "compile_node: unexpected node type");
 	}
 }
 
@@ -2755,11 +2947,34 @@ codeobject *
 compile(n, filename)
 	node *n;
 	char *filename;
+{
+	return jcompile(n, filename, NULL);
+}
+
+static codeobject *
+icompile(n, base)
+	node *n;
+	struct compiling *base;
+{
+	return jcompile(n, base->c_filename, base);
+}
+
+static codeobject *
+jcompile(n, filename, base)
+	node *n;
+	char *filename;
+	struct compiling *base;
 {
 	struct compiling sc;
 	codeobject *co;
 	if (!com_init(&sc, filename))
 		return NULL;
+#ifdef PRIVATE_NAME_MANGLING
+	if (base)
+		sc.c_private = base->c_private;
+	else
+		sc.c_private = NULL;
+#endif
 	compile_node(&sc, n);
 	com_done(&sc);
 	if ((TYPE(n) == funcdef || TYPE(n) == lambdef) && sc.c_errors == 0) {
