@@ -88,58 +88,61 @@ static state states_4[4] = {
 	{1, arcs_4_2},
 	{1, arcs_4_3},
 };
-static arc arcs_5_0[2] = {
+static arc arcs_5_0[3] = {
 	{19, 1},
 	{23, 2},
+	{24, 3},
 };
 static arc arcs_5_1[3] = {
-	{20, 3},
-	{22, 4},
+	{20, 4},
+	{22, 5},
 	{0, 1},
 };
 static arc arcs_5_2[2] = {
-	{12, 5},
-	{23, 6},
+	{12, 6},
+	{23, 3},
 };
 static arc arcs_5_3[1] = {
-	{21, 7},
+	{12, 7},
 };
-static arc arcs_5_4[3] = {
+static arc arcs_5_4[1] = {
+	{21, 8},
+};
+static arc arcs_5_5[4] = {
 	{19, 1},
 	{23, 2},
-	{0, 4},
-};
-static arc arcs_5_5[2] = {
-	{22, 8},
+	{24, 3},
 	{0, 5},
 };
-static arc arcs_5_6[1] = {
-	{12, 9},
+static arc arcs_5_6[2] = {
+	{22, 9},
+	{0, 6},
 };
-static arc arcs_5_7[2] = {
-	{22, 4},
+static arc arcs_5_7[1] = {
 	{0, 7},
 };
-static arc arcs_5_8[1] = {
+static arc arcs_5_8[2] = {
+	{22, 5},
+	{0, 8},
+};
+static arc arcs_5_9[2] = {
+	{24, 3},
 	{23, 10},
 };
-static arc arcs_5_9[1] = {
-	{0, 9},
-};
 static arc arcs_5_10[1] = {
-	{23, 6},
+	{23, 3},
 };
 static state states_5[11] = {
-	{2, arcs_5_0},
+	{3, arcs_5_0},
 	{3, arcs_5_1},
 	{2, arcs_5_2},
 	{1, arcs_5_3},
-	{3, arcs_5_4},
-	{2, arcs_5_5},
-	{1, arcs_5_6},
-	{2, arcs_5_7},
-	{1, arcs_5_8},
-	{1, arcs_5_9},
+	{1, arcs_5_4},
+	{4, arcs_5_5},
+	{2, arcs_5_6},
+	{1, arcs_5_7},
+	{2, arcs_5_8},
+	{2, arcs_5_9},
 	{1, arcs_5_10},
 };
 static arc arcs_6_0[2] = {
@@ -150,7 +153,7 @@ static arc arcs_6_1[1] = {
 	{0, 1},
 };
 static arc arcs_6_2[1] = {
-	{24, 3},
+	{25, 3},
 };
 static arc arcs_6_3[1] = {
 	{18, 1},
@@ -189,14 +192,14 @@ static state states_8[2] = {
 	{1, arcs_8_1},
 };
 static arc arcs_9_0[1] = {
-	{25, 1},
+	{26, 1},
 };
 static arc arcs_9_1[2] = {
-	{26, 2},
+	{27, 2},
 	{2, 3},
 };
 static arc arcs_9_2[2] = {
-	{25, 1},
+	{26, 1},
 	{2, 3},
 };
 static arc arcs_9_3[1] = {
@@ -208,8 +211,7 @@ static state states_9[4] = {
 	{2, arcs_9_2},
 	{1, arcs_9_3},
 };
-static arc arcs_10_0[9] = {
-	{27, 1},
+static arc arcs_10_0[8] = {
 	{28, 1},
 	{29, 1},
 	{30, 1},
@@ -223,7 +225,7 @@ static arc arcs_10_1[1] = {
 	{0, 1},
 };
 static state states_10[2] = {
-	{9, arcs_10_0},
+	{8, arcs_10_0},
 	{1, arcs_10_1},
 };
 static arc arcs_11_0[1] = {
@@ -428,636 +430,638 @@ static state states_22[3] = {
 static arc arcs_23_0[1] = {
 	{53, 1},
 };
-static arc arcs_23_1[2] = {
-	{23, 2},
-	{12, 3},
+static arc arcs_23_1[1] = {
+	{54, 2},
 };
-static arc arcs_23_2[1] = {
-	{14, 4},
+static arc arcs_23_2[2] = {
+	{55, 3},
+	{0, 2},
 };
-static arc arcs_23_3[2] = {
+static arc arcs_23_3[1] = {
+	{21, 4},
+};
+static arc arcs_23_4[2] = {
 	{22, 5},
-	{14, 4},
-};
-static arc arcs_23_4[1] = {
-	{54, 6},
+	{0, 4},
 };
 static arc arcs_23_5[1] = {
-	{12, 3},
+	{21, 6},
 };
-static arc arcs_23_6[2] = {
-	{22, 4},
+static arc arcs_23_6[1] = {
 	{0, 6},
 };
 static state states_23[7] = {
 	{1, arcs_23_0},
-	{2, arcs_23_1},
-	{1, arcs_23_2},
-	{2, arcs_23_3},
-	{1, arcs_23_4},
+	{1, arcs_23_1},
+	{2, arcs_23_2},
+	{1, arcs_23_3},
+	{2, arcs_23_4},
 	{1, arcs_23_5},
-	{2, arcs_23_6},
+	{1, arcs_23_6},
 };
-static arc arcs_24_0[1] = {
-	{12, 1},
+static arc arcs_24_0[6] = {
+	{56, 1},
+	{57, 1},
+	{58, 1},
+	{59, 1},
+	{10, 1},
+	{60, 1},
 };
-static arc arcs_24_1[2] = {
-	{12, 1},
+static arc arcs_24_1[1] = {
 	{0, 1},
 };
 static state states_24[2] = {
-	{1, arcs_24_0},
-	{2, arcs_24_1},
+	{6, arcs_24_0},
+	{1, arcs_24_1},
 };
 static arc arcs_25_0[1] = {
-	{55, 1},
+	{61, 1},
 };
 static arc arcs_25_1[1] = {
-	{56, 2},
+	{21, 2},
 };
-static arc arcs_25_2[2] = {
-	{57, 3},
-	{0, 2},
+static arc arcs_25_2[1] = {
+	{14, 3},
 };
 static arc arcs_25_3[1] = {
-	{21, 4},
+	{15, 4},
 };
-static arc arcs_25_4[2] = {
-	{22, 5},
+static arc arcs_25_4[3] = {
+	{62, 1},
+	{63, 5},
 	{0, 4},
 };
 static arc arcs_25_5[1] = {
-	{21, 6},
-};
-static arc arcs_25_6[1] = {
-	{0, 6},
-};
-static state states_25[7] = {
-	{1, arcs_25_0},
-	{1, arcs_25_1},
-	{2, arcs_25_2},
-	{1, arcs_25_3},
-	{2, arcs_25_4},
-	{1, arcs_25_5},
-	{1, arcs_25_6},
-};
-static arc arcs_26_0[6] = {
-	{58, 1},
-	{59, 1},
-	{60, 1},
-	{61, 1},
-	{10, 1},
-	{62, 1},
-};
-static arc arcs_26_1[1] = {
-	{0, 1},
-};
-static state states_26[2] = {
-	{6, arcs_26_0},
-	{1, arcs_26_1},
-};
-static arc arcs_27_0[1] = {
-	{63, 1},
-};
-static arc arcs_27_1[1] = {
-	{21, 2},
-};
-static arc arcs_27_2[1] = {
-	{14, 3},
-};
-static arc arcs_27_3[1] = {
-	{15, 4},
-};
-static arc arcs_27_4[3] = {
-	{64, 1},
-	{65, 5},
-	{0, 4},
-};
-static arc arcs_27_5[1] = {
 	{14, 6},
 };
-static arc arcs_27_6[1] = {
+static arc arcs_25_6[1] = {
 	{15, 7},
 };
-static arc arcs_27_7[1] = {
+static arc arcs_25_7[1] = {
 	{0, 7},
 };
-static state states_27[8] = {
+static state states_25[8] = {
+	{1, arcs_25_0},
+	{1, arcs_25_1},
+	{1, arcs_25_2},
+	{1, arcs_25_3},
+	{3, arcs_25_4},
+	{1, arcs_25_5},
+	{1, arcs_25_6},
+	{1, arcs_25_7},
+};
+static arc arcs_26_0[1] = {
+	{64, 1},
+};
+static arc arcs_26_1[1] = {
+	{21, 2},
+};
+static arc arcs_26_2[1] = {
+	{14, 3},
+};
+static arc arcs_26_3[1] = {
+	{15, 4},
+};
+static arc arcs_26_4[2] = {
+	{63, 5},
+	{0, 4},
+};
+static arc arcs_26_5[1] = {
+	{14, 6},
+};
+static arc arcs_26_6[1] = {
+	{15, 7},
+};
+static arc arcs_26_7[1] = {
+	{0, 7},
+};
+static state states_26[8] = {
+	{1, arcs_26_0},
+	{1, arcs_26_1},
+	{1, arcs_26_2},
+	{1, arcs_26_3},
+	{2, arcs_26_4},
+	{1, arcs_26_5},
+	{1, arcs_26_6},
+	{1, arcs_26_7},
+};
+static arc arcs_27_0[1] = {
+	{65, 1},
+};
+static arc arcs_27_1[1] = {
+	{38, 2},
+};
+static arc arcs_27_2[1] = {
+	{55, 3},
+};
+static arc arcs_27_3[1] = {
+	{9, 4},
+};
+static arc arcs_27_4[1] = {
+	{14, 5},
+};
+static arc arcs_27_5[1] = {
+	{15, 6},
+};
+static arc arcs_27_6[2] = {
+	{63, 7},
+	{0, 6},
+};
+static arc arcs_27_7[1] = {
+	{14, 8},
+};
+static arc arcs_27_8[1] = {
+	{15, 9},
+};
+static arc arcs_27_9[1] = {
+	{0, 9},
+};
+static state states_27[10] = {
 	{1, arcs_27_0},
 	{1, arcs_27_1},
 	{1, arcs_27_2},
 	{1, arcs_27_3},
-	{3, arcs_27_4},
+	{1, arcs_27_4},
 	{1, arcs_27_5},
-	{1, arcs_27_6},
+	{2, arcs_27_6},
 	{1, arcs_27_7},
+	{1, arcs_27_8},
+	{1, arcs_27_9},
 };
 static arc arcs_28_0[1] = {
 	{66, 1},
 };
 static arc arcs_28_1[1] = {
-	{21, 2},
+	{14, 2},
 };
 static arc arcs_28_2[1] = {
-	{14, 3},
+	{15, 3},
 };
-static arc arcs_28_3[1] = {
-	{15, 4},
+static arc arcs_28_3[2] = {
+	{67, 4},
+	{68, 5},
 };
-static arc arcs_28_4[2] = {
-	{65, 5},
-	{0, 4},
-};
-static arc arcs_28_5[1] = {
+static arc arcs_28_4[1] = {
 	{14, 6},
 };
+static arc arcs_28_5[1] = {
+	{14, 7},
+};
 static arc arcs_28_6[1] = {
-	{15, 7},
+	{15, 8},
 };
 static arc arcs_28_7[1] = {
-	{0, 7},
+	{15, 9},
 };
-static state states_28[8] = {
+static arc arcs_28_8[3] = {
+	{67, 4},
+	{63, 5},
+	{0, 8},
+};
+static arc arcs_28_9[1] = {
+	{0, 9},
+};
+static state states_28[10] = {
 	{1, arcs_28_0},
 	{1, arcs_28_1},
 	{1, arcs_28_2},
-	{1, arcs_28_3},
-	{2, arcs_28_4},
+	{2, arcs_28_3},
+	{1, arcs_28_4},
 	{1, arcs_28_5},
 	{1, arcs_28_6},
 	{1, arcs_28_7},
+	{3, arcs_28_8},
+	{1, arcs_28_9},
 };
 static arc arcs_29_0[1] = {
-	{67, 1},
+	{69, 1},
 };
-static arc arcs_29_1[1] = {
-	{38, 2},
-};
-static arc arcs_29_2[1] = {
-	{57, 3},
-};
-static arc arcs_29_3[1] = {
-	{9, 4},
-};
-static arc arcs_29_4[1] = {
-	{14, 5},
-};
-static arc arcs_29_5[1] = {
-	{15, 6},
-};
-static arc arcs_29_6[2] = {
-	{65, 7},
-	{0, 6},
-};
-static arc arcs_29_7[1] = {
-	{14, 8},
-};
-static arc arcs_29_8[1] = {
-	{15, 9},
-};
-static arc arcs_29_9[1] = {
-	{0, 9},
-};
-static state states_29[10] = {
-	{1, arcs_29_0},
-	{1, arcs_29_1},
-	{1, arcs_29_2},
-	{1, arcs_29_3},
-	{1, arcs_29_4},
-	{1, arcs_29_5},
-	{2, arcs_29_6},
-	{1, arcs_29_7},
-	{1, arcs_29_8},
-	{1, arcs_29_9},
-};
-static arc arcs_30_0[1] = {
-	{68, 1},
-};
-static arc arcs_30_1[1] = {
-	{14, 2},
-};
-static arc arcs_30_2[1] = {
-	{15, 3},
-};
-static arc arcs_30_3[2] = {
-	{69, 4},
-	{70, 5},
-};
-static arc arcs_30_4[1] = {
-	{14, 6},
-};
-static arc arcs_30_5[1] = {
-	{14, 7},
-};
-static arc arcs_30_6[1] = {
-	{15, 8},
-};
-static arc arcs_30_7[1] = {
-	{15, 9},
-};
-static arc arcs_30_8[3] = {
-	{69, 4},
-	{65, 5},
-	{0, 8},
-};
-static arc arcs_30_9[1] = {
-	{0, 9},
-};
-static state states_30[10] = {
-	{1, arcs_30_0},
-	{1, arcs_30_1},
-	{1, arcs_30_2},
-	{2, arcs_30_3},
-	{1, arcs_30_4},
-	{1, arcs_30_5},
-	{1, arcs_30_6},
-	{1, arcs_30_7},
-	{3, arcs_30_8},
-	{1, arcs_30_9},
-};
-static arc arcs_31_0[1] = {
-	{71, 1},
-};
-static arc arcs_31_1[2] = {
+static arc arcs_29_1[2] = {
 	{21, 2},
 	{0, 1},
 };
-static arc arcs_31_2[2] = {
+static arc arcs_29_2[2] = {
 	{22, 3},
 	{0, 2},
 };
-static arc arcs_31_3[1] = {
+static arc arcs_29_3[1] = {
 	{21, 4},
 };
-static arc arcs_31_4[1] = {
+static arc arcs_29_4[1] = {
 	{0, 4},
 };
-static state states_31[5] = {
-	{1, arcs_31_0},
-	{2, arcs_31_1},
-	{2, arcs_31_2},
-	{1, arcs_31_3},
-	{1, arcs_31_4},
+static state states_29[5] = {
+	{1, arcs_29_0},
+	{2, arcs_29_1},
+	{2, arcs_29_2},
+	{1, arcs_29_3},
+	{1, arcs_29_4},
 };
-static arc arcs_32_0[2] = {
+static arc arcs_30_0[2] = {
 	{3, 1},
 	{2, 2},
 };
-static arc arcs_32_1[1] = {
+static arc arcs_30_1[1] = {
 	{0, 1},
 };
-static arc arcs_32_2[1] = {
-	{72, 3},
+static arc arcs_30_2[1] = {
+	{70, 3},
 };
-static arc arcs_32_3[1] = {
+static arc arcs_30_3[1] = {
 	{6, 4},
 };
-static arc arcs_32_4[2] = {
+static arc arcs_30_4[2] = {
 	{6, 4},
-	{73, 1},
+	{71, 1},
 };
-static state states_32[5] = {
-	{2, arcs_32_0},
-	{1, arcs_32_1},
-	{1, arcs_32_2},
-	{1, arcs_32_3},
-	{2, arcs_32_4},
+static state states_30[5] = {
+	{2, arcs_30_0},
+	{1, arcs_30_1},
+	{1, arcs_30_2},
+	{1, arcs_30_3},
+	{2, arcs_30_4},
+};
+static arc arcs_31_0[2] = {
+	{72, 1},
+	{74, 2},
+};
+static arc arcs_31_1[2] = {
+	{73, 3},
+	{0, 1},
+};
+static arc arcs_31_2[1] = {
+	{0, 2},
+};
+static arc arcs_31_3[1] = {
+	{72, 1},
+};
+static state states_31[4] = {
+	{2, arcs_31_0},
+	{2, arcs_31_1},
+	{1, arcs_31_2},
+	{1, arcs_31_3},
+};
+static arc arcs_32_0[1] = {
+	{75, 1},
+};
+static arc arcs_32_1[2] = {
+	{76, 0},
+	{0, 1},
+};
+static state states_32[2] = {
+	{1, arcs_32_0},
+	{2, arcs_32_1},
 };
 static arc arcs_33_0[2] = {
-	{74, 1},
-	{76, 2},
+	{77, 1},
+	{78, 2},
 };
-static arc arcs_33_1[2] = {
-	{75, 3},
-	{0, 1},
+static arc arcs_33_1[1] = {
+	{75, 2},
 };
 static arc arcs_33_2[1] = {
 	{0, 2},
 };
-static arc arcs_33_3[1] = {
-	{74, 1},
-};
-static state states_33[4] = {
+static state states_33[3] = {
 	{2, arcs_33_0},
-	{2, arcs_33_1},
+	{1, arcs_33_1},
 	{1, arcs_33_2},
-	{1, arcs_33_3},
 };
 static arc arcs_34_0[1] = {
-	{77, 1},
+	{54, 1},
 };
 static arc arcs_34_1[2] = {
-	{78, 0},
+	{79, 0},
 	{0, 1},
 };
 static state states_34[2] = {
 	{1, arcs_34_0},
 	{2, arcs_34_1},
 };
-static arc arcs_35_0[2] = {
-	{79, 1},
-	{80, 2},
+static arc arcs_35_0[10] = {
+	{80, 1},
+	{81, 1},
+	{82, 1},
+	{83, 1},
+	{84, 1},
+	{85, 1},
+	{86, 1},
+	{55, 1},
+	{77, 2},
+	{87, 3},
 };
 static arc arcs_35_1[1] = {
-	{77, 2},
+	{0, 1},
 };
 static arc arcs_35_2[1] = {
-	{0, 2},
+	{55, 1},
 };
-static state states_35[3] = {
-	{2, arcs_35_0},
+static arc arcs_35_3[2] = {
+	{77, 1},
+	{0, 3},
+};
+static state states_35[4] = {
+	{10, arcs_35_0},
 	{1, arcs_35_1},
 	{1, arcs_35_2},
+	{2, arcs_35_3},
 };
 static arc arcs_36_0[1] = {
-	{56, 1},
+	{88, 1},
 };
 static arc arcs_36_1[2] = {
-	{81, 0},
+	{89, 0},
 	{0, 1},
 };
 static state states_36[2] = {
 	{1, arcs_36_0},
 	{2, arcs_36_1},
 };
-static arc arcs_37_0[10] = {
-	{82, 1},
-	{83, 1},
-	{84, 1},
-	{85, 1},
-	{86, 1},
-	{87, 1},
-	{88, 1},
-	{57, 1},
-	{79, 2},
-	{89, 3},
-};
-static arc arcs_37_1[1] = {
-	{0, 1},
-};
-static arc arcs_37_2[1] = {
-	{57, 1},
-};
-static arc arcs_37_3[2] = {
-	{79, 1},
-	{0, 3},
-};
-static state states_37[4] = {
-	{10, arcs_37_0},
-	{1, arcs_37_1},
-	{1, arcs_37_2},
-	{2, arcs_37_3},
-};
-static arc arcs_38_0[1] = {
+static arc arcs_37_0[1] = {
 	{90, 1},
 };
-static arc arcs_38_1[2] = {
+static arc arcs_37_1[2] = {
 	{91, 0},
 	{0, 1},
 };
+static state states_37[2] = {
+	{1, arcs_37_0},
+	{2, arcs_37_1},
+};
+static arc arcs_38_0[1] = {
+	{92, 1},
+};
+static arc arcs_38_1[2] = {
+	{93, 0},
+	{0, 1},
+};
 static state states_38[2] = {
 	{1, arcs_38_0},
 	{2, arcs_38_1},
 };
 static arc arcs_39_0[1] = {
-	{92, 1},
+	{94, 1},
 };
-static arc arcs_39_1[2] = {
-	{93, 0},
+static arc arcs_39_1[3] = {
+	{95, 0},
+	{96, 0},
 	{0, 1},
 };
 static state states_39[2] = {
 	{1, arcs_39_0},
-	{2, arcs_39_1},
+	{3, arcs_39_1},
 };
 static arc arcs_40_0[1] = {
-	{94, 1},
+	{97, 1},
 };
-static arc arcs_40_1[2] = {
-	{95, 0},
+static arc arcs_40_1[3] = {
+	{98, 0},
+	{99, 0},
 	{0, 1},
 };
 static state states_40[2] = {
 	{1, arcs_40_0},
-	{2, arcs_40_1},
+	{3, arcs_40_1},
 };
 static arc arcs_41_0[1] = {
-	{96, 1},
+	{100, 1},
 };
-static arc arcs_41_1[3] = {
-	{97, 0},
-	{98, 0},
+static arc arcs_41_1[4] = {
+	{23, 0},
+	{101, 0},
+	{102, 0},
 	{0, 1},
 };
 static state states_41[2] = {
 	{1, arcs_41_0},
-	{3, arcs_41_1},
+	{4, arcs_41_1},
 };
-static arc arcs_42_0[1] = {
+static arc arcs_42_0[4] = {
+	{98, 1},
 	{99, 1},
+	{103, 1},
+	{104, 2},
 };
-static arc arcs_42_1[3] = {
-	{100, 0},
-	{101, 0},
-	{0, 1},
+static arc arcs_42_1[1] = {
+	{100, 2},
 };
-static state states_42[2] = {
-	{1, arcs_42_0},
-	{3, arcs_42_1},
-};
-static arc arcs_43_0[1] = {
-	{102, 1},
-};
-static arc arcs_43_1[4] = {
-	{23, 0},
-	{103, 0},
-	{104, 0},
-	{0, 1},
-};
-static state states_43[2] = {
-	{1, arcs_43_0},
-	{4, arcs_43_1},
-};
-static arc arcs_44_0[4] = {
-	{100, 1},
-	{101, 1},
-	{105, 1},
-	{106, 2},
-};
-static arc arcs_44_1[1] = {
-	{102, 3},
-};
-static arc arcs_44_2[2] = {
-	{107, 2},
+static arc arcs_42_2[1] = {
 	{0, 2},
 };
-static arc arcs_44_3[1] = {
+static state states_42[3] = {
+	{4, arcs_42_0},
+	{1, arcs_42_1},
+	{1, arcs_42_2},
+};
+static arc arcs_43_0[1] = {
+	{105, 1},
+};
+static arc arcs_43_1[3] = {
+	{106, 1},
+	{24, 2},
+	{0, 1},
+};
+static arc arcs_43_2[1] = {
+	{100, 3},
+};
+static arc arcs_43_3[2] = {
+	{24, 2},
 	{0, 3},
 };
-static state states_44[4] = {
-	{4, arcs_44_0},
-	{1, arcs_44_1},
-	{2, arcs_44_2},
-	{1, arcs_44_3},
+static state states_43[4] = {
+	{1, arcs_43_0},
+	{3, arcs_43_1},
+	{1, arcs_43_2},
+	{2, arcs_43_3},
 };
-static arc arcs_45_0[7] = {
+static arc arcs_44_0[7] = {
 	{16, 1},
-	{108, 2},
-	{110, 3},
-	{113, 4},
+	{107, 2},
+	{109, 3},
+	{112, 4},
 	{12, 5},
-	{114, 5},
-	{115, 6},
+	{113, 5},
+	{114, 6},
 };
-static arc arcs_45_1[2] = {
+static arc arcs_44_1[2] = {
 	{9, 7},
 	{18, 5},
 };
-static arc arcs_45_2[2] = {
+static arc arcs_44_2[2] = {
 	{9, 8},
-	{109, 5},
+	{108, 5},
 };
-static arc arcs_45_3[2] = {
-	{111, 9},
-	{112, 5},
+static arc arcs_44_3[2] = {
+	{110, 9},
+	{111, 5},
 };
-static arc arcs_45_4[1] = {
+static arc arcs_44_4[1] = {
 	{9, 10},
 };
-static arc arcs_45_5[1] = {
+static arc arcs_44_5[1] = {
 	{0, 5},
 };
-static arc arcs_45_6[2] = {
-	{115, 6},
+static arc arcs_44_6[2] = {
+	{114, 6},
 	{0, 6},
 };
-static arc arcs_45_7[1] = {
+static arc arcs_44_7[1] = {
 	{18, 5},
 };
-static arc arcs_45_8[1] = {
-	{109, 5},
+static arc arcs_44_8[1] = {
+	{108, 5},
 };
-static arc arcs_45_9[1] = {
+static arc arcs_44_9[1] = {
+	{111, 5},
+};
+static arc arcs_44_10[1] = {
 	{112, 5},
 };
-static arc arcs_45_10[1] = {
-	{113, 5},
+static state states_44[11] = {
+	{7, arcs_44_0},
+	{2, arcs_44_1},
+	{2, arcs_44_2},
+	{2, arcs_44_3},
+	{1, arcs_44_4},
+	{1, arcs_44_5},
+	{2, arcs_44_6},
+	{1, arcs_44_7},
+	{1, arcs_44_8},
+	{1, arcs_44_9},
+	{1, arcs_44_10},
 };
-static state states_45[11] = {
-	{7, arcs_45_0},
-	{2, arcs_45_1},
-	{2, arcs_45_2},
-	{2, arcs_45_3},
-	{1, arcs_45_4},
-	{1, arcs_45_5},
-	{2, arcs_45_6},
-	{1, arcs_45_7},
-	{1, arcs_45_8},
-	{1, arcs_45_9},
-	{1, arcs_45_10},
+static arc arcs_45_0[1] = {
+	{115, 1},
 };
-static arc arcs_46_0[1] = {
-	{116, 1},
-};
-static arc arcs_46_1[2] = {
+static arc arcs_45_1[2] = {
 	{17, 2},
 	{14, 3},
 };
-static arc arcs_46_2[1] = {
+static arc arcs_45_2[1] = {
 	{14, 3},
 };
-static arc arcs_46_3[1] = {
+static arc arcs_45_3[1] = {
 	{21, 4},
 };
-static arc arcs_46_4[1] = {
+static arc arcs_45_4[1] = {
 	{0, 4},
 };
-static state states_46[5] = {
-	{1, arcs_46_0},
+static state states_45[5] = {
+	{1, arcs_45_0},
+	{2, arcs_45_1},
+	{1, arcs_45_2},
+	{1, arcs_45_3},
+	{1, arcs_45_4},
+};
+static arc arcs_46_0[3] = {
+	{16, 1},
+	{107, 2},
+	{51, 3},
+};
+static arc arcs_46_1[2] = {
+	{116, 4},
+	{18, 5},
+};
+static arc arcs_46_2[1] = {
+	{117, 6},
+};
+static arc arcs_46_3[1] = {
+	{12, 5},
+};
+static arc arcs_46_4[1] = {
+	{18, 5},
+};
+static arc arcs_46_5[1] = {
+	{0, 5},
+};
+static arc arcs_46_6[1] = {
+	{108, 5},
+};
+static state states_46[7] = {
+	{3, arcs_46_0},
 	{2, arcs_46_1},
 	{1, arcs_46_2},
 	{1, arcs_46_3},
 	{1, arcs_46_4},
+	{1, arcs_46_5},
+	{1, arcs_46_6},
 };
-static arc arcs_47_0[3] = {
-	{16, 1},
-	{108, 2},
-	{51, 3},
+static arc arcs_47_0[1] = {
+	{118, 1},
 };
 static arc arcs_47_1[2] = {
-	{117, 4},
-	{18, 5},
-};
-static arc arcs_47_2[1] = {
-	{118, 6},
-};
-static arc arcs_47_3[1] = {
-	{12, 5},
-};
-static arc arcs_47_4[1] = {
-	{18, 5},
-};
-static arc arcs_47_5[1] = {
-	{0, 5},
-};
-static arc arcs_47_6[1] = {
-	{109, 5},
-};
-static state states_47[7] = {
-	{3, arcs_47_0},
-	{2, arcs_47_1},
-	{1, arcs_47_2},
-	{1, arcs_47_3},
-	{1, arcs_47_4},
-	{1, arcs_47_5},
-	{1, arcs_47_6},
-};
-static arc arcs_48_0[2] = {
-	{21, 1},
-	{14, 2},
-};
-static arc arcs_48_1[2] = {
-	{14, 2},
-	{0, 1},
-};
-static arc arcs_48_2[2] = {
-	{21, 3},
-	{0, 2},
-};
-static arc arcs_48_3[1] = {
-	{0, 3},
-};
-static state states_48[4] = {
-	{2, arcs_48_0},
-	{2, arcs_48_1},
-	{2, arcs_48_2},
-	{1, arcs_48_3},
-};
-static arc arcs_49_0[1] = {
-	{56, 1},
-};
-static arc arcs_49_1[2] = {
 	{22, 2},
 	{0, 1},
 };
-static arc arcs_49_2[2] = {
-	{56, 1},
+static arc arcs_47_2[2] = {
+	{118, 1},
+	{0, 2},
+};
+static state states_47[3] = {
+	{1, arcs_47_0},
+	{2, arcs_47_1},
+	{2, arcs_47_2},
+};
+static arc arcs_48_0[3] = {
+	{51, 1},
+	{21, 2},
+	{14, 3},
+};
+static arc arcs_48_1[1] = {
+	{51, 4},
+};
+static arc arcs_48_2[2] = {
+	{14, 3},
+	{0, 2},
+};
+static arc arcs_48_3[3] = {
+	{21, 5},
+	{119, 6},
+	{0, 3},
+};
+static arc arcs_48_4[1] = {
+	{51, 6},
+};
+static arc arcs_48_5[2] = {
+	{119, 6},
+	{0, 5},
+};
+static arc arcs_48_6[1] = {
+	{0, 6},
+};
+static state states_48[7] = {
+	{3, arcs_48_0},
+	{1, arcs_48_1},
+	{2, arcs_48_2},
+	{3, arcs_48_3},
+	{1, arcs_48_4},
+	{2, arcs_48_5},
+	{1, arcs_48_6},
+};
+static arc arcs_49_0[1] = {
+	{14, 1},
+};
+static arc arcs_49_1[2] = {
+	{21, 2},
+	{0, 1},
+};
+static arc arcs_49_2[1] = {
 	{0, 2},
 };
 static state states_49[3] = {
 	{1, arcs_49_0},
 	{2, arcs_49_1},
-	{2, arcs_49_2},
+	{1, arcs_49_2},
 };
 static arc arcs_50_0[1] = {
-	{21, 1},
+	{54, 1},
 };
 static arc arcs_50_1[2] = {
 	{22, 2},
 	{0, 1},
 };
 static arc arcs_50_2[2] = {
-	{21, 1},
+	{54, 1},
 	{0, 2},
 };
 static state states_50[3] = {
@@ -1068,122 +1072,138 @@ static state states_50[3] = {
 static arc arcs_51_0[1] = {
 	{21, 1},
 };
-static arc arcs_51_1[1] = {
+static arc arcs_51_1[2] = {
+	{22, 2},
+	{0, 1},
+};
+static arc arcs_51_2[2] = {
+	{21, 1},
+	{0, 2},
+};
+static state states_51[3] = {
+	{1, arcs_51_0},
+	{2, arcs_51_1},
+	{2, arcs_51_2},
+};
+static arc arcs_52_0[1] = {
+	{21, 1},
+};
+static arc arcs_52_1[1] = {
 	{14, 2},
 };
-static arc arcs_51_2[1] = {
+static arc arcs_52_2[1] = {
 	{21, 3},
 };
-static arc arcs_51_3[2] = {
+static arc arcs_52_3[2] = {
 	{22, 4},
 	{0, 3},
 };
-static arc arcs_51_4[2] = {
+static arc arcs_52_4[2] = {
 	{21, 1},
 	{0, 4},
 };
-static state states_51[5] = {
-	{1, arcs_51_0},
-	{1, arcs_51_1},
-	{1, arcs_51_2},
-	{2, arcs_51_3},
-	{2, arcs_51_4},
-};
-static arc arcs_52_0[1] = {
-	{119, 1},
-};
-static arc arcs_52_1[1] = {
-	{12, 2},
-};
-static arc arcs_52_2[2] = {
-	{16, 3},
-	{14, 4},
-};
-static arc arcs_52_3[1] = {
-	{9, 5},
-};
-static arc arcs_52_4[1] = {
-	{15, 6},
-};
-static arc arcs_52_5[1] = {
-	{18, 7},
-};
-static arc arcs_52_6[1] = {
-	{0, 6},
-};
-static arc arcs_52_7[1] = {
-	{14, 4},
-};
-static state states_52[8] = {
+static state states_52[5] = {
 	{1, arcs_52_0},
 	{1, arcs_52_1},
-	{2, arcs_52_2},
-	{1, arcs_52_3},
-	{1, arcs_52_4},
-	{1, arcs_52_5},
-	{1, arcs_52_6},
-	{1, arcs_52_7},
+	{1, arcs_52_2},
+	{2, arcs_52_3},
+	{2, arcs_52_4},
 };
 static arc arcs_53_0[1] = {
 	{120, 1},
 };
-static arc arcs_53_1[2] = {
+static arc arcs_53_1[1] = {
+	{12, 2},
+};
+static arc arcs_53_2[2] = {
+	{16, 3},
+	{14, 4},
+};
+static arc arcs_53_3[1] = {
+	{9, 5},
+};
+static arc arcs_53_4[1] = {
+	{15, 6},
+};
+static arc arcs_53_5[1] = {
+	{18, 7},
+};
+static arc arcs_53_6[1] = {
+	{0, 6},
+};
+static arc arcs_53_7[1] = {
+	{14, 4},
+};
+static state states_53[8] = {
+	{1, arcs_53_0},
+	{1, arcs_53_1},
+	{2, arcs_53_2},
+	{1, arcs_53_3},
+	{1, arcs_53_4},
+	{1, arcs_53_5},
+	{1, arcs_53_6},
+	{1, arcs_53_7},
+};
+static arc arcs_54_0[1] = {
+	{121, 1},
+};
+static arc arcs_54_1[2] = {
 	{22, 2},
 	{0, 1},
 };
-static arc arcs_53_2[2] = {
-	{120, 1},
+static arc arcs_54_2[2] = {
+	{121, 1},
 	{0, 2},
 };
-static state states_53[3] = {
-	{1, arcs_53_0},
-	{2, arcs_53_1},
-	{2, arcs_53_2},
+static state states_54[3] = {
+	{1, arcs_54_0},
+	{2, arcs_54_1},
+	{2, arcs_54_2},
 };
-static arc arcs_54_0[1] = {
+static arc arcs_55_0[1] = {
 	{21, 1},
 };
-static arc arcs_54_1[2] = {
+static arc arcs_55_1[2] = {
 	{20, 2},
 	{0, 1},
 };
-static arc arcs_54_2[1] = {
+static arc arcs_55_2[1] = {
 	{21, 3},
 };
-static arc arcs_54_3[1] = {
+static arc arcs_55_3[1] = {
 	{0, 3},
 };
-static state states_54[4] = {
-	{1, arcs_54_0},
-	{2, arcs_54_1},
-	{1, arcs_54_2},
-	{1, arcs_54_3},
+static state states_55[4] = {
+	{1, arcs_55_0},
+	{2, arcs_55_1},
+	{1, arcs_55_2},
+	{1, arcs_55_3},
 };
-static dfa dfas[55] = {
+static dfa dfas[56] = {
 	{256, "single_input", 0, 3, states_0,
-	 "\004\030\001\000\260\360\265\200\034\200\000\000\060\122\236\000"},
+	 "\004\030\001\000\260\360\065\040\007\040\000\000\214\050\017\001"},
 	{257, "file_input", 0, 2, states_1,
-	 "\204\030\001\000\260\360\265\200\034\200\000\000\060\122\236\000"},
+	 "\204\030\001\000\260\360\065\040\007\040\000\000\214\050\017\001"},
 	{258, "eval_input", 0, 3, states_2,
-	 "\000\020\001\000\000\000\000\000\000\200\000\000\060\122\036\000"},
+	 "\000\020\001\000\000\000\000\000\000\040\000\000\214\050\017\000"},
 	{259, "funcdef", 0, 6, states_3,
 	 "\000\010\000\000\000\000\000\000\000\000\000\000\000\000\000\000"},
 	{260, "parameters", 0, 4, states_4,
 	 "\000\000\001\000\000\000\000\000\000\000\000\000\000\000\000\000"},
 	{261, "varargslist", 0, 11, states_5,
-	 "\000\020\201\000\000\000\000\000\000\000\000\000\000\000\000\000"},
+	 "\000\020\201\001\000\000\000\000\000\000\000\000\000\000\000\000"},
 	{262, "fpdef", 0, 4, states_6,
 	 "\000\020\001\000\000\000\000\000\000\000\000\000\000\000\000\000"},
 	{263, "fplist", 0, 3, states_7,
 	 "\000\020\001\000\000\000\000\000\000\000\000\000\000\000\000\000"},
 	{264, "stmt", 0, 2, states_8,
-	 "\000\030\001\000\260\360\265\200\034\200\000\000\060\122\236\000"},
+	 "\000\030\001\000\260\360\065\040\007\040\000\000\214\050\017\001"},
 	{265, "simple_stmt", 0, 4, states_9,
-	 "\000\020\001\000\260\360\265\000\000\200\000\000\060\122\036\000"},
+	 "\000\020\001\000\260\360\065\000\000\040\000\000\214\050\017\000"},
 	{266, "small_stmt", 0, 2, states_10,
-	 "\000\020\001\000\260\360\265\000\000\200\000\000\060\122\036\000"},
+	 "\000\020\001\000\260\360\065\000\000\040\000\000\214\050\017\000"},
 	{267, "expr_stmt", 0, 2, states_11,
-	 "\000\020\001\000\000\000\000\000\000\200\000\000\060\122\036\000"},
+	 "\000\020\001\000\000\000\000\000\000\040\000\000\214\050\017\000"},
 	{268, "print_stmt", 0, 3, states_12,
 	 "\000\000\000\000\020\000\000\000\000\000\000\000\000\000\000\000"},
 	{269, "del_stmt", 0, 3, states_13,
@@ -1206,96 +1226,99 @@ static dfa dfas[55] = {
 	 "\000\020\000\000\000\000\000\000\000\000\000\000\000\000\000\000"},
 	{278, "global_stmt", 0, 3, states_22,
 	 "\000\000\000\000\000\000\020\000\000\000\000\000\000\000\000\000"},
-	{279, "access_stmt", 0, 7, states_23,
+	{279, "exec_stmt", 0, 7, states_23,
 	 "\000\000\000\000\000\000\040\000\000\000\000\000\000\000\000\000"},
-	{280, "accesstype", 0, 2, states_24,
-	 "\000\020\000\000\000\000\000\000\000\000\000\000\000\000\000\000"},
-	{281, "exec_stmt", 0, 7, states_25,
-	 "\000\000\000\000\000\000\200\000\000\000\000\000\000\000\000\000"},
-	{282, "compound_stmt", 0, 2, states_26,
-	 "\000\010\000\000\000\000\000\200\034\000\000\000\000\000\200\000"},
-	{283, "if_stmt", 0, 8, states_27,
-	 "\000\000\000\000\000\000\000\200\000\000\000\000\000\000\000\000"},
-	{284, "while_stmt", 0, 8, states_28,
+	{280, "compound_stmt", 0, 2, states_24,
+	 "\000\010\000\000\000\000\000\040\007\000\000\000\000\000\000\001"},
+	{281, "if_stmt", 0, 8, states_25,
+	 "\000\000\000\000\000\000\000\040\000\000\000\000\000\000\000\000"},
+	{282, "while_stmt", 0, 8, states_26,
+	 "\000\000\000\000\000\000\000\000\001\000\000\000\000\000\000\000"},
+	{283, "for_stmt", 0, 10, states_27,
+	 "\000\000\000\000\000\000\000\000\002\000\000\000\000\000\000\000"},
+	{284, "try_stmt", 0, 10, states_28,
 	 "\000\000\000\000\000\000\000\000\004\000\000\000\000\000\000\000"},
-	{285, "for_stmt", 0, 10, states_29,
-	 "\000\000\000\000\000\000\000\000\010\000\000\000\000\000\000\000"},
-	{286, "try_stmt", 0, 10, states_30,
-	 "\000\000\000\000\000\000\000\000\020\000\000\000\000\000\000\000"},
-	{287, "except_clause", 0, 5, states_31,
-	 "\000\000\000\000\000\000\000\000\200\000\000\000\000\000\000\000"},
-	{288, "suite", 0, 5, states_32,
-	 "\004\020\001\000\260\360\265\000\000\200\000\000\060\122\036\000"},
-	{289, "test", 0, 4, states_33,
-	 "\000\020\001\000\000\000\000\000\000\200\000\000\060\122\036\000"},
-	{290, "and_test", 0, 2, states_34,
-	 "\000\020\001\000\000\000\000\000\000\200\000\000\060\122\016\000"},
-	{291, "not_test", 0, 3, states_35,
-	 "\000\020\001\000\000\000\000\000\000\200\000\000\060\122\016\000"},
-	{292, "comparison", 0, 2, states_36,
-	 "\000\020\001\000\000\000\000\000\000\000\000\000\060\122\016\000"},
-	{293, "comp_op", 0, 4, states_37,
-	 "\000\000\000\000\000\000\000\002\000\200\374\003\000\000\000\000"},
-	{294, "expr", 0, 2, states_38,
-	 "\000\020\001\000\000\000\000\000\000\000\000\000\060\122\016\000"},
-	{295, "xor_expr", 0, 2, states_39,
-	 "\000\020\001\000\000\000\000\000\000\000\000\000\060\122\016\000"},
-	{296, "and_expr", 0, 2, states_40,
-	 "\000\020\001\000\000\000\000\000\000\000\000\000\060\122\016\000"},
-	{297, "shift_expr", 0, 2, states_41,
-	 "\000\020\001\000\000\000\000\000\000\000\000\000\060\122\016\000"},
-	{298, "arith_expr", 0, 2, states_42,
-	 "\000\020\001\000\000\000\000\000\000\000\000\000\060\122\016\000"},
-	{299, "term", 0, 2, states_43,
-	 "\000\020\001\000\000\000\000\000\000\000\000\000\060\122\016\000"},
-	{300, "factor", 0, 4, states_44,
-	 "\000\020\001\000\000\000\000\000\000\000\000\000\060\122\016\000"},
-	{301, "atom", 0, 11, states_45,
-	 "\000\020\001\000\000\000\000\000\000\000\000\000\000\120\016\000"},
-	{302, "lambdef", 0, 5, states_46,
-	 "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\020\000"},
-	{303, "trailer", 0, 7, states_47,
-	 "\000\000\001\000\000\000\010\000\000\000\000\000\000\020\000\000"},
-	{304, "subscript", 0, 4, states_48,
-	 "\000\120\001\000\000\000\000\000\000\200\000\000\060\122\036\000"},
-	{305, "exprlist", 0, 3, states_49,
-	 "\000\020\001\000\000\000\000\000\000\000\000\000\060\122\016\000"},
-	{306, "testlist", 0, 3, states_50,
-	 "\000\020\001\000\000\000\000\000\000\200\000\000\060\122\036\000"},
-	{307, "dictmaker", 0, 5, states_51,
-	 "\000\020\001\000\000\000\000\000\000\200\000\000\060\122\036\000"},
-	{308, "classdef", 0, 8, states_52,
-	 "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\200\000"},
-	{309, "arglist", 0, 3, states_53,
-	 "\000\020\001\000\000\000\000\000\000\200\000\000\060\122\036\000"},
-	{310, "argument", 0, 4, states_54,
-	 "\000\020\001\000\000\000\000\000\000\200\000\000\060\122\036\000"},
+	{285, "except_clause", 0, 5, states_29,
+	 "\000\000\000\000\000\000\000\000\040\000\000\000\000\000\000\000"},
+	{286, "suite", 0, 5, states_30,
+	 "\004\020\001\000\260\360\065\000\000\040\000\000\214\050\017\000"},
+	{287, "test", 0, 4, states_31,
+	 "\000\020\001\000\000\000\000\000\000\040\000\000\214\050\017\000"},
+	{288, "and_test", 0, 2, states_32,
+	 "\000\020\001\000\000\000\000\000\000\040\000\000\214\050\007\000"},
+	{289, "not_test", 0, 3, states_33,
+	 "\000\020\001\000\000\000\000\000\000\040\000\000\214\050\007\000"},
+	{290, "comparison", 0, 2, states_34,
+	 "\000\020\001\000\000\000\000\000\000\000\000\000\214\050\007\000"},
+	{291, "comp_op", 0, 4, states_35,
+	 "\000\000\000\000\000\000\200\000\000\040\377\000\000\000\000\000"},
+	{292, "expr", 0, 2, states_36,
+	 "\000\020\001\000\000\000\000\000\000\000\000\000\214\050\007\000"},
+	{293, "xor_expr", 0, 2, states_37,
+	 "\000\020\001\000\000\000\000\000\000\000\000\000\214\050\007\000"},
+	{294, "and_expr", 0, 2, states_38,
+	 "\000\020\001\000\000\000\000\000\000\000\000\000\214\050\007\000"},
+	{295, "shift_expr", 0, 2, states_39,
+	 "\000\020\001\000\000\000\000\000\000\000\000\000\214\050\007\000"},
+	{296, "arith_expr", 0, 2, states_40,
+	 "\000\020\001\000\000\000\000\000\000\000\000\000\214\050\007\000"},
+	{297, "term", 0, 2, states_41,
+	 "\000\020\001\000\000\000\000\000\000\000\000\000\214\050\007\000"},
+	{298, "factor", 0, 3, states_42,
+	 "\000\020\001\000\000\000\000\000\000\000\000\000\214\050\007\000"},
+	{299, "power", 0, 4, states_43,
+	 "\000\020\001\000\000\000\000\000\000\000\000\000\000\050\007\000"},
+	{300, "atom", 0, 11, states_44,
+	 "\000\020\001\000\000\000\000\000\000\000\000\000\000\050\007\000"},
+	{301, "lambdef", 0, 5, states_45,
+	 "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\010\000"},
+	{302, "trailer", 0, 7, states_46,
+	 "\000\000\001\000\000\000\010\000\000\000\000\000\000\010\000\000"},
+	{303, "subscriptlist", 0, 3, states_47,
+	 "\000\120\001\000\000\000\010\000\000\040\000\000\214\050\017\000"},
+	{304, "subscript", 0, 7, states_48,
+	 "\000\120\001\000\000\000\010\000\000\040\000\000\214\050\017\000"},
+	{305, "sliceop", 0, 3, states_49,
+	 "\000\100\000\000\000\000\000\000\000\000\000\000\000\000\000\000"},
+	{306, "exprlist", 0, 3, states_50,
+	 "\000\020\001\000\000\000\000\000\000\000\000\000\214\050\007\000"},
+	{307, "testlist", 0, 3, states_51,
+	 "\000\020\001\000\000\000\000\000\000\040\000\000\214\050\017\000"},
+	{308, "dictmaker", 0, 5, states_52,
+	 "\000\020\001\000\000\000\000\000\000\040\000\000\214\050\017\000"},
+	{309, "classdef", 0, 8, states_53,
+	 "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\001"},
+	{310, "arglist", 0, 3, states_54,
+	 "\000\020\001\000\000\000\000\000\000\040\000\000\214\050\017\000"},
+	{311, "argument", 0, 4, states_55,
+	 "\000\020\001\000\000\000\000\000\000\040\000\000\214\050\017\000"},
 };
-static label labels[121] = {
+static label labels[122] = {
 	{0, "EMPTY"},
 	{256, 0},
 	{4, 0},
 	{265, 0},
-	{282, 0},
+	{280, 0},
 	{257, 0},
 	{264, 0},
 	{0, 0},
 	{258, 0},
-	{306, 0},
+	{307, 0},
 	{259, 0},
 	{1, "def"},
 	{1, 0},
 	{260, 0},
 	{11, 0},
-	{288, 0},
+	{286, 0},
 	{7, 0},
 	{261, 0},
 	{8, 0},
 	{262, 0},
 	{22, 0},
-	{289, 0},
+	{287, 0},
 	{12, 0},
 	{16, 0},
+	{36, 0},
 	{263, 0},
 	{266, 0},
 	{13, 0},
@@ -1307,10 +1330,9 @@ static label labels[121] = {
 	{276, 0},
 	{278, 0},
 	{279, 0},
-	{281, 0},
 	{1, "print"},
 	{1, "del"},
-	{305, 0},
+	{306, 0},
 	{1, "pass"},
 	{272, 0},
 	{273, 0},
@@ -1325,35 +1347,33 @@ static label labels[121] = {
 	{1, "from"},
 	{23, 0},
 	{1, "global"},
-	{1, "access"},
-	{280, 0},
 	{1, "exec"},
-	{294, 0},
+	{292, 0},
 	{1, "in"},
+	{281, 0},
+	{282, 0},
 	{283, 0},
 	{284, 0},
-	{285, 0},
-	{286, 0},
-	{308, 0},
+	{309, 0},
 	{1, "if"},
 	{1, "elif"},
 	{1, "else"},
 	{1, "while"},
 	{1, "for"},
 	{1, "try"},
-	{287, 0},
+	{285, 0},
 	{1, "finally"},
 	{1, "except"},
 	{5, 0},
 	{6, 0},
-	{290, 0},
+	{288, 0},
 	{1, "or"},
-	{302, 0},
-	{291, 0},
+	{301, 0},
+	{289, 0},
 	{1, "and"},
 	{1, "not"},
-	{292, 0},
-	{293, 0},
+	{290, 0},
+	{291, 0},
 	{20, 0},
 	{21, 0},
 	{28, 0},
@@ -1362,41 +1382,44 @@ static label labels[121] = {
 	{29, 0},
 	{29, 0},
 	{1, "is"},
-	{295, 0},
+	{293, 0},
 	{18, 0},
-	{296, 0},
+	{294, 0},
 	{33, 0},
-	{297, 0},
+	{295, 0},
 	{19, 0},
-	{298, 0},
+	{296, 0},
 	{34, 0},
 	{35, 0},
-	{299, 0},
+	{297, 0},
 	{14, 0},
 	{15, 0},
-	{300, 0},
+	{298, 0},
 	{17, 0},
 	{24, 0},
 	{32, 0},
-	{301, 0},
-	{303, 0},
+	{299, 0},
+	{300, 0},
+	{302, 0},
 	{9, 0},
 	{10, 0},
 	{26, 0},
-	{307, 0},
+	{308, 0},
 	{27, 0},
 	{25, 0},
 	{2, 0},
 	{3, 0},
 	{1, "lambda"},
-	{309, 0},
-	{304, 0},
-	{1, "class"},
 	{310, 0},
+	{303, 0},
+	{304, 0},
+	{305, 0},
+	{1, "class"},
+	{311, 0},
 };
 grammar gram = {
-	55,
+	56,
 	dfas,
-	{121, labels},
+	{122, labels},
 	256
 };
