@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -29,8 +36,6 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "osdefs.h"
 #include "importdl.h"
 
-extern int verbose; /* Defined in pythonrun.c */
-
 /* Explanation of some of the the various #defines used by dynamic linking...
 
    symbol	-- defined for:
@@ -40,13 +45,15 @@ extern int verbose; /* Defined in pythonrun.c */
    USE_DL	-- Jack's dl for IRIX 4 or GNU dld with emulation for Jack's dl
    USE_SHLIB	-- SunOS or IRIX 5 (SVR4?) shared libraries
    _AIX		-- AIX style dynamic linking
-   NT		-- NT style dynamic linking (using DLLs)
+   MS_WIN32	-- Windows NT style dynamic linking (using DLLs)
+   MS_WIN16	-- Windows 16-bit dynamic linking (using DLLs)
    _DL_FUNCPTR_DEFINED	-- if the typedef dl_funcptr has been defined
    USE_MAC_DYNAMIC_LOADING -- Mac CFM shared libraries
    SHORT_EXT	-- short extension for dynamic module, e.g. ".so"
    LONG_EXT	-- long extension, e.g. "module.so"
    hpux		-- HP-UX Dynamic Linking - defined by the compiler
    __NetBSD__	-- NetBSD shared libraries (not quite SVR4 compatible)
+   __FreeBSD__	-- FreeBSD shared libraries
 
    (The other WITH_* symbols are used only once, to set the
    appropriate symbols.)
@@ -54,6 +61,12 @@ extern int verbose; /* Defined in pythonrun.c */
 
 /* Configure dynamic linking */
 
+#ifdef __hpux
+#ifndef hpux
+#define hpux
+#endif
+#endif
+
 #ifdef hpux
 #define DYNAMIC_LINK
 #include <errno.h>
@@ -63,24 +76,20 @@ typedef void (*dl_funcptr)();
 #define LONG_EXT "module.sl"
 #endif 
 
-#ifdef __NetBSD__
+#if defined(__NetBSD__) || defined(__FreeBSD__)
 #define DYNAMIC_LINK
 #define USE_SHLIB
 
 #define dlerror() "error in dynamic linking"
 #endif
 
-#ifdef __WIN32__
-#define NT
-#endif
-
-#ifdef NT
+#ifdef MS_WINDOWS /* i.e. MS_WIN32 or MS_WIN16 */
 #define DYNAMIC_LINK
 #include <windows.h>
 typedef FARPROC dl_funcptr;
 #define _DL_FUNCPTR_DEFINED
 #define SHORT_EXT ".pyd"
-#define LONG_EXT "module.pyd"
+#define LONG_EXT ".dll"
 #endif
 
 #ifdef NeXT
@@ -98,6 +107,10 @@ typedef FARPROC dl_funcptr;
 #define USE_DL
 #endif
 
+#ifdef __powerc
+#define USE_MAC_DYNAMIC_LOADING
+#endif
+
 #ifdef __CFM68K__
 #define USE_MAC_DYNAMIC_LOADING
 #endif
@@ -105,23 +118,31 @@ typedef FARPROC dl_funcptr;
 #ifdef USE_MAC_DYNAMIC_LOADING
 #define DYNAMIC_LINK
 #define SHORT_EXT ".slb"
-#define LONG_EXT "module.slb"
+#ifdef __CFM68K__
+#define LONG_EXT ".CFM68K.slb"
+#else
+#define LONG_EXT ".ppc.slb"
+#endif
 #ifndef _DL_FUNCPTR_DEFINED
 typedef void (*dl_funcptr)();
 #endif
 #endif
 
-#if !defined(DYNAMIC_LINK) && defined(HAVE_DLFCN_H) && defined(HAVE_DLOPEN)
+#if !defined(DYNAMIC_LINK) && (defined(HAVE_DLOPEN) || defined(M_UNIX))
 #define DYNAMIC_LINK
 #define USE_SHLIB
 #endif
 
 #ifdef _AIX
 #define DYNAMIC_LINK
+#define SHORT_EXT ".so"
+#define LONG_EXT "module.so"
 #include <sys/ldr.h>
 typedef void (*dl_funcptr)();
 #define _DL_FUNCPTR_DEFINED
-static void aix_loaderror(char *name);
+static int  aix_getoldmodules(void **);
+static int  aix_bindnewmodule(void *, void *);
+static void aix_loaderror(char *);
 #endif
 
 #ifdef DYNAMIC_LINK
@@ -129,12 +150,14 @@ static void aix_loaderror(char *name);
 #ifdef USE_SHLIB
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef __NetBSD__
+#if defined(__NetBSD__) || defined(__FreeBSD__)
 #include <nlist.h>
 #include <link.h>
 #else
+#ifdef HAVE_DLFCN_H
 #include <dlfcn.h>
 #endif
+#endif
 #ifndef _DL_FUNCPTR_DEFINED
 typedef void (*dl_funcptr)();
 #endif
@@ -150,6 +173,7 @@ typedef void (*dl_funcptr)();
 #endif
 
 #ifdef USE_MAC_DYNAMIC_LOADING
+#include <Aliases.h>
 #include <CodeFragments.h>
 #ifdef SYMANTEC__CFM68K__ /* Really just an older version of Universal Headers */
 #define CFragConnectionID ConnectionID
@@ -171,7 +195,7 @@ typedef void (*dl_funcptr)();
 extern char *getprogramname();
 
 #ifndef FUNCNAME_PATTERN
-#if defined(__hp9000s300) || defined(__NetBSD__) || defined(__BORLANDC__)
+#if defined(__hp9000s300) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__BORLANDC__)
 #define FUNCNAME_PATTERN "_init%.200s"
 #else
 #define FUNCNAME_PATTERN "init%.200s"
@@ -205,6 +229,10 @@ struct filedescr import_filetab[] = {
 	{0, 0}
 };
 
+#ifdef NO_DYNAMIC_LINK
+#undef DYNAMIC_LINK
+#endif
+
 object *
 load_dynamic_module(name, pathname, fp)
 	char *name;
@@ -215,7 +243,7 @@ load_dynamic_module(name, pathname, fp)
 	err_setstr(ImportError, "dynamically linked modules not supported");
 	return NULL;
 #else
-	object *m;
+	object *m, *d, *s;
 	char funcname[258];
 	dl_funcptr p = NULL;
 #ifdef USE_SHLIB
@@ -225,6 +253,12 @@ load_dynamic_module(name, pathname, fp)
 		void *handle;
 	} handles[128];
 	static int nhandles = 0;
+	char pathbuf[260];
+	if (strchr(pathname, '/') == NULL) {
+		/* Prefix bare filename with "./" */
+		sprintf(pathbuf, "./%-.255s", pathname);
+		pathname = pathbuf;
+	}
 #endif
 	sprintf(funcname, FUNCNAME_PATTERN, name);
 #ifdef USE_SHLIB
@@ -268,10 +302,7 @@ load_dynamic_module(name, pathname, fp)
 		
 		/* First resolve any aliases to find the real file */
 		(void)FSMakeFSSpec(0, 0, Pstring(pathname), &libspec);
-#if !(defined(__MWERKS__) && defined(__CFM68K__))
-		/* Bug: not in library */
 		err = ResolveAliasFile(&libspec, 1, &isfolder, &didsomething);
-#endif
 		if ( err ) {
 			sprintf(buf, "%s: %s", pathname, PyMac_StrError(err));
 			err_setstr(ImportError, buf);
@@ -325,13 +356,32 @@ load_dynamic_module(name, pathname, fp)
 	}
 #endif /* USE_SHLIB */
 #ifdef _AIX
-	p = (dl_funcptr) load(pathname, 1, 0);
-	if (p == NULL) {
-		aix_loaderror(pathname);
-		return NULL;
+	/*
+	-- Invoke load() with L_NOAUTODEFER leaving the imported symbols
+	-- of the shared module unresolved. Thus we have to resolve them
+	-- explicitely with loadbind. The new module is loaded, then we
+	-- resolve its symbols using the list of already loaded modules
+	-- (only those that belong to the python executable). Get these
+	-- with loadquery(L_GETINFO).
+	*/
+	{
+		static void *staticmodlistptr = NULL;
+
+		if (!staticmodlistptr)
+			if (aix_getoldmodules(&staticmodlistptr) == -1)
+				return NULL;
+		p = (dl_funcptr) load(pathname, L_NOAUTODEFER, 0);
+		if (p == NULL) {
+			aix_loaderror(pathname);
+			return NULL;
+		}
+		if (aix_bindnewmodule((void *)p, staticmodlistptr) == -1) {
+			aix_loaderror(pathname);
+			return NULL;
+		}
 	}
 #endif /* _AIX */
-#ifdef NT
+#ifdef MS_WIN32
 	{
 		HINSTANCE hDLL;
 		hDLL = LoadLibrary(pathname);
@@ -373,7 +423,20 @@ load_dynamic_module(name, pathname, fp)
 		}
 		p = GetProcAddress(hDLL, funcname);
 	}
-#endif /* NT */
+#endif /* MS_WIN32 */
+#ifdef MS_WIN16
+	{
+		HINSTANCE hDLL;
+		hDLL = LoadLibrary(pathname);
+		if (hDLL < HINSTANCE_ERROR){
+			char errBuf[256];
+			sprintf(errBuf, "DLL load failed with error code %d", hDLL);
+			err_setstr(ImportError, errBuf);
+			return NULL;
+		}
+		p = GetProcAddress(hDLL, funcname);
+	}
+#endif /* MS_WIN16 */
 #ifdef USE_DL
 	p =  dl_loadmod(getprogramname(), pathname, funcname);
 #endif /* USE_DL */
@@ -455,6 +518,12 @@ load_dynamic_module(name, pathname, fp)
 				   "dynamic module not initialized properly");
 		return NULL;
 	}
+	/* Remember the filename as the __file__ attribute */
+	d = getmoduledict(m);
+	s = newstringobject(pathname);
+	if (s == NULL || dictinsert(d, "__file__", s) != 0)
+		err_clear(); /* Not important enough to report */
+	XDECREF(s);
 	if (verbose)
 		fprintf(stderr,
 			"import %s # dynamically loaded from %s\n",
@@ -467,18 +536,101 @@ load_dynamic_module(name, pathname, fp)
 
 #ifdef _AIX
 
-#include <ctype.h>	/* for isdigit()	*/
-#include <errno.h>	/* for global errno	*/
-#include <string.h>	/* for strerror()	*/
+#include <ctype.h>	/*  for isdigit()	  */
+#include <errno.h>	/*  for global errno      */
+#include <string.h>	/*  for strerror()        */
+#include <stdlib.h>	/*  for malloc(), free()  */
 
-void aix_loaderror(char *pathname)
+typedef struct Module {
+	struct Module *next;
+	void          *entry;
+} Module, *ModulePtr;
+
+static int
+aix_getoldmodules(modlistptr)
+	void **modlistptr;
+{
+	register ModulePtr       modptr, prevmodptr;
+	register struct ld_info  *ldiptr;
+	register char            *ldibuf;
+	register int             errflag, bufsize = 1024;
+	register unsigned int    offset;
+	
+	/*
+	-- Get the list of loaded modules into ld_info structures.
+	*/
+	if ((ldibuf = malloc(bufsize)) == NULL) {
+		err_setstr(ImportError, strerror(errno));
+		return -1;
+	}
+	while ((errflag = loadquery(L_GETINFO, ldibuf, bufsize)) == -1
+	       && errno == ENOMEM) {
+		free(ldibuf);
+		bufsize += 1024;
+		if ((ldibuf = malloc(bufsize)) == NULL) {
+			err_setstr(ImportError, strerror(errno));
+			return -1;
+		}
+	}
+	if (errflag == -1) {
+		err_setstr(ImportError, strerror(errno));
+		return -1;
+	}
+	/*
+	-- Make the modules list from the ld_info structures.
+	*/
+	ldiptr = (struct ld_info *)ldibuf;
+	prevmodptr = NULL;
+	do {
+		if ((modptr = (ModulePtr)malloc(sizeof(Module))) == NULL) {
+			err_setstr(ImportError, strerror(errno));
+			while (*modlistptr) {
+				modptr = (ModulePtr)*modlistptr;
+				*modlistptr = (void *)modptr->next;
+				free(modptr);
+			}
+			return -1;
+		}
+		modptr->entry = ldiptr->ldinfo_dataorg;
+		modptr->next  = NULL;
+		if (prevmodptr == NULL)
+			*modlistptr = (void *)modptr;
+		else
+			prevmodptr->next = modptr;
+		prevmodptr = modptr;
+		offset = (unsigned int)ldiptr->ldinfo_next;
+		ldiptr = (struct ld_info *)((unsigned int)ldiptr + offset);
+	} while (offset);
+	free(ldibuf);
+	return 0;
+}
+
+static int
+aix_bindnewmodule(newmoduleptr, modlistptr)
+	void *newmoduleptr;
+	void *modlistptr;        
+{
+	register ModulePtr modptr;
+
+	/*
+	-- Bind the new module with the list of loaded modules.
+	*/
+	for (modptr = (ModulePtr)modlistptr; modptr; modptr = modptr->next)
+		if (loadbind(0, modptr->entry, newmoduleptr) != 0)
+			return -1;
+	return 0;
+}
+
+static void
+aix_loaderror(pathname)
+	char *pathname;
 {
 
-	char *message[8], errbuf[1024];
-	int i,j;
+	char *message[1024], errbuf[1024];
+	register int i,j;
 
 	struct errtab { 
-		int errno;
+		int errNo;
 		char *errstr;
 	} load_errtab[] = {
 		{L_ERROR_TOOMANY,	"too many errors, rest skipped."},
@@ -490,7 +642,7 @@ void aix_loaderror(char *pathname)
 		{L_ERROR_MEMBER,
 		 "file not an archive or does not contain requested member:"},
 		{L_ERROR_TYPE,		"symbol table mismatch:"},
-		{L_ERROR_ALIGN,		"text allignment in file is wrong."},
+		{L_ERROR_ALIGN,		"text alignment in file is wrong."},
 		{L_ERROR_SYSTEM,	"System error:"},
 		{L_ERROR_ERRNO,		NULL}
 	};
@@ -498,15 +650,17 @@ void aix_loaderror(char *pathname)
 #define LOAD_ERRTAB_LEN	(sizeof(load_errtab)/sizeof(load_errtab[0]))
 #define ERRBUF_APPEND(s) strncat(errbuf, s, sizeof(errbuf)-strlen(errbuf)-1)
 
-	sprintf(errbuf, " from module %.200s ", pathname);
+	sprintf(errbuf, "from module %.200s ", pathname);
 
-	if (!loadquery(1, &message[0], sizeof(message))) 
+	if (!loadquery(L_GETMESSAGES, &message[0], sizeof(message))) {
 		ERRBUF_APPEND(strerror(errno));
+		ERRBUF_APPEND("\n");
+	}
 	for(i = 0; message[i] && *message[i]; i++) {
 		int nerr = atoi(message[i]);
 		for (j=0; j<LOAD_ERRTAB_LEN ; j++) {
-		    if (nerr == load_errtab[i].errno && load_errtab[i].errstr)
-			ERRBUF_APPEND(load_errtab[i].errstr);
+		    if (nerr == load_errtab[j].errNo && load_errtab[j].errstr)
+			ERRBUF_APPEND(load_errtab[j].errstr);
 		}
 		while (isdigit(*message[i])) message[i]++ ; 
 		ERRBUF_APPEND(message[i]);
