@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -37,7 +44,6 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "compile.h"
 #include "eval.h"
 #include "ceval.h"
-#include "pythonrun.h"
 #include "import.h"
 #include "marshal.h"
 
@@ -45,16 +51,9 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <signal.h>
 #endif
 
-#ifdef THINK_C
-#include <console.h>
-#endif
-
-#ifdef __MWERKS__
-#include <SIOUX.h>
-#endif
-
-#ifdef NT
+#ifdef MS_WIN32
 #undef BYTE
+#undef arglist
 #include "windows.h"
 #endif
 
@@ -155,7 +154,7 @@ run_tty_loop(fp, filename)
 	}
 	for (;;) {
 		ret = run_tty_1(fp, filename);
-#ifdef REF_DEBUG
+#ifdef Py_REF_DEBUG
 		fprintf(stderr, "[%ld refs]\n", _Py_RefTotal);
 #endif
 		if (ret == E_EOF)
@@ -570,7 +569,7 @@ fatal(msg)
 #ifdef macintosh
 	for (;;);
 #endif
-#ifdef NT
+#ifdef MS_WIN32
 	OutputDebugString("Fatal Python error:");
 	OutputDebugString(msg);
 	OutputDebugString("\n");
@@ -660,30 +659,21 @@ goaway(sts)
 	
 	err_clear();
 
-#ifdef REF_DEBUG
+#ifdef Py_REF_DEBUG
 	fprintf(stderr, "[%ld refs]\n", _Py_RefTotal);
 #endif
 
-#ifdef TRACE_REFS
+#ifdef Py_TRACE_REFS
 	if (askyesno("Print left references?")) {
-		printrefs(stderr);
+		_Py_PrintReferences(stderr);
 	}
-#endif /* TRACE_REFS */
+#endif /* Py_TRACE_REFS */
 
-	/* XXXX Jack thinks it would be nicer to pause if any output has
-	** been generated since the last interaction with the user...
-	*/
-#ifdef THINK_C
-	if (sts == 0)
-		console_options.pause_atexit = 0;
-#endif
-#ifdef __MWERKS__
-	if (sts == 0)
-		SIOUXSettings.autocloseonquit = 1;
-	else
-		printf("\n[Terminated]\n");
-#endif
+#ifdef macintosh
+	PyMac_Exit(sts);
+#else
 	exit(sts);
+#endif
 #endif /* WITH_THREAD */
 	/*NOTREACHED*/
 }
@@ -730,7 +720,7 @@ initsigs()
 	initintr(); /* May imply initsignal() */
 }
 
-#ifdef TRACE_REFS
+#ifdef Py_TRACE_REFS
 /* Ask a yes/no question */
 
 int
