@@ -4,21 +4,28 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
@@ -49,6 +56,10 @@ int sys_checkinterval = 10;
 
 static object *sysdict;
 
+#ifdef MS_COREDLL
+extern void *PyWin_DLLhModule;
+#endif
+
 object *
 sysget(name)
 	char *name;
@@ -178,24 +189,26 @@ sys_getcounts(self, args)
 }
 #endif
 
-#ifdef TRACE_REFS
-extern PyObject *getobjects Py_PROTO((PyObject *, PyObject *));
+#ifdef Py_TRACE_REFS
+/* Defined in objects.c because it uses static globals if that file */
+extern PyObject *_Py_GetObjects Py_PROTO((PyObject *, PyObject *));
 #endif
+
 static struct methodlist sys_methods[] = {
 	{"exit",	sys_exit, 0},
 	{"getrefcount",	sys_getrefcount, 0},
 #ifdef COUNT_ALLOCS
 	{"getcounts",	sys_getcounts, 0},
 #endif
-#ifdef TRACE_REFS
-	{"getobjects",	getobjects, 1},
+#ifdef Py_TRACE_REFS
+	{"getobjects",	_Py_GetObjects, 1},
 #endif
 #ifdef USE_MALLOPT
 	{"mdebug",	sys_mdebug, 0},
 #endif
 	{"setprofile",	sys_setprofile, 0},
 	{"settrace",	sys_settrace, 0},
-	{"setcheckinterval",	sys_setcheckinterval, 0},
+	{"setcheckinterval",	sys_setcheckinterval, 1},
 	{NULL,		NULL}		/* sentinel */
 };
 
@@ -229,6 +242,8 @@ initsys()
 	extern char *getversion PROTO((void));
 	extern char *getcopyright PROTO((void));
 	extern char *getplatform PROTO((void));
+	extern char *Py_GetPrefix PROTO((void));
+	extern char *Py_GetExecPrefix PROTO((void));
 	extern int fclose PROTO((FILE *));
 	object *m = initmodule("sys", sys_methods);
 	object *v;
@@ -250,12 +265,23 @@ initsys()
 	XDECREF(v);
 	dictinsert(sysdict, "platform", v = newstringobject(getplatform()));
 	XDECREF(v);
+	dictinsert(sysdict, "prefix", v = newstringobject(Py_GetPrefix()));
+	XDECREF(v);
+	dictinsert(sysdict, "exec_prefix",
+		   v = newstringobject(Py_GetExecPrefix()));
+	XDECREF(v);
 	dictinsert(sysdict, "maxint", v = newintobject(getmaxint()));
 	XDECREF(v);
 	dictinsert(sysdict, "modules", get_modules());
 	dictinsert(sysdict, "builtin_module_names",
 		   v = list_builtin_module_names());
 	XDECREF(v);
+#ifdef MS_COREDLL
+	dictinsert(sysdict, "dllhandle", v = newintobject((int)PyWin_DLLhModule));
+	XDECREF(v);
+	dictinsert(sysdict, "winver", v = newstringobject(MS_DLL_ID));
+	XDECREF(v);
+#endif
 	if (err_occurred())
 		fatal("can't insert sys.* objects in sys dict");
 }
@@ -341,9 +367,46 @@ setpythonargv(argc, argv)
 	char **argv;
 {
 	object *av = makeargvobject(argc, argv);
+	object *path = sysget("path");
 	if (av == NULL)
 		fatal("no mem for sys.argv");
 	if (sysset("argv", av) != 0)
 		fatal("can't assign sys.argv");
+	if (path != NULL) {
+		char *p = NULL;
+		int n = 0;
+		object *a;
+#if SEP == '\\' /* Special case for MS filename syntax */
+		if (argc > 0 && argv[0] != NULL) {
+			char *q;
+			p = strrchr(argv[0], SEP);
+			/* Test for alternate separator */
+			q = strrchr(p ? p : argv[0], '/');
+			if (q != NULL)
+				p = q;
+			if (p != NULL) {
+				n = p + 1 - argv[0];
+				if (n > 1 && p[-1] != ':')
+					n--; /* Drop trailing separator */
+			}
+		}
+#else /* All other filename syntaxes */
+		if (argc > 0 && argv[0] != NULL)
+			p = strrchr(argv[0], SEP);
+		if (p != NULL) {
+			n = p + 1 - argv[0];
+#if SEP == '/' /* Special case for Unix filename syntax */
+			if (n > 1)
+				n--; /* Drop trailing separator */
+#endif /* Unix */
+		}
+#endif /* All others */
+		a = newsizedstringobject(argv[0], n);
+		if (a == NULL)
+			fatal("no mem for sys.path insertion");
+		if (inslistitem(path, 0, a) < 0)
+			fatal("sys.path.insert(0) failed");
+		DECREF(a);
+	}
 	DECREF(av);
 }
