@@ -4,41 +4,54 @@ The Netherlands.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior permission.
+Centrum or CWI or Corporation for National Research Initiatives or
+CNRI not be used in advertising or publicity pertaining to
+distribution of the software without specific, written prior
+permission.
 
-STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
-FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+While CWI is the initial source for this software, a modified version
+is made available by the Corporation for National Research Initiatives
+(CNRI) at the Internet address ftp://ftp.python.org.
+
+STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
+MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
+CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
+DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+PERFORMANCE OF THIS SOFTWARE.
 
 ******************************************************************/
 
-#ifdef sun
-#define FLORIDA_HACKS
+/* Posix threads interface */
+
+#include <stdlib.h>
+#include <pthread.h>
+
+#ifdef _AIX
+
+#ifndef SCHED_BG_NP
+/* SCHED_BG_NP is defined if using AIX DCE pthreads
+ * but it is unsupported by AIX 4 pthreads. Default
+ * attributes for AIX 4 pthreads equal to NULL. For
+ * AIX DCE pthreads they should be left unchanged.
+ */
+#define pthread_attr_default NULL
+#define pthread_mutexattr_default NULL
+#define pthread_condattr_default NULL
 #endif
 
-#ifdef FLORIDA_HACKS
-/* Hacks for Florida State Posix threads implementation */
-#undef _POSIX_THREADS
-#include "/ufs/guido/src/python/Contrib/pthreads/src/pthread.h"
+#else
 #define pthread_attr_default ((pthread_attr_t *)0)
 #define pthread_mutexattr_default ((pthread_mutexattr_t *)0)
 #define pthread_condattr_default ((pthread_condattr_t *)0)
-#define TRYLOCK_OFFSET 1
-#else /* !FLORIDA_HACKS */
-#include <pthread.h>
-#define TRYLOCK_OFFSET 0
-#endif /* FLORIDA_HACKS */
-#include <stdlib.h>
+#endif
 
 /* A pthread mutex isn't sufficient to model the Python lock type
  * because, according to Draft 5 of the docs (P1003.4a/D5), both of the
@@ -78,16 +91,13 @@ static void _init_thread _P0()
 
 int start_new_thread _P2(func, void (*func) _P((void *)), arg, void *arg)
 {
-#if defined(SGI_THREADS) && defined(USE_DL)
-	long addr, size;
-	static int local_initialized = 0;
-#endif /* SGI_THREADS and USE_DL */
 	pthread_t th;
 	int success;
 	dprintf(("start_new_thread called\n"));
 	if (!initialized)
 		init_thread();
-	success = pthread_create(&th, pthread_attr_default, func, arg);
+	success = pthread_create(&th, pthread_attr_default,
+				 (void* (*) _P((void *)))func, arg);
 	return success < 0 ? 0 : 1;
 }
 
@@ -271,10 +281,11 @@ void free_sema _P1(sema, type_sema sema)
 	dprintf(("free_sema(%lx) called\n", (long) sema));
 }
 
-void down_sema _P1(sema, type_sema sema)
+int down_sema _P2(sema, type_sema sema, waitflag, int waitflag)
 {
-	dprintf(("down_sema(%lx) called\n", (long) sema));
+	dprintf(("down_sema(%lx, %d) called\n", (long) sema, waitflag));
 	dprintf(("down_sema(%lx) return\n", (long) sema));
+	return -1;
 }
 
 void up_sema _P1(sema, type_sema sema)
