@@ -9,6 +9,9 @@
    (as it does on SCI ODT 3.0) */
 #undef BAD_STATIC_FORWARD
 
+/* Define if you have the Mach cthreads package */
+#undef C_THREADS
+
 /* Define to `long' if <time.h> doesn't define.  */
 #undef clock_t
 
@@ -22,6 +25,9 @@
 /* Define this if your time.h defines altzone */
 #undef HAVE_ALTZONE
 
+/* Define this if you have a K&R style C preprocessor */
+#undef HAVE_OLD_CPP
+
 /* Define if your compiler supports function prototypes */
 #undef HAVE_PROTOTYPES
 
@@ -32,6 +38,9 @@
 /* Define if you have POSIX threads */
 #undef _POSIX_THREADS
 
+/* Define to force use of thread-safe errno, h_errno, and other functions */
+#undef _REENTRANT
+
 /* Define if setpgrp() must be called as setpgrp(0, 0). */
 #undef SETPGRP_HAVE_ARG
 
